@@ -16,10 +16,17 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
+#include "config.h"
+
 #include	<stdio.h>
-#include 	<unistd.h>
+#include <stdlib.h>
+#include	<string.h>
 #include 	<errno.h>
 
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #if defined (__linux__)
 	#include 	<fcntl.h>
 	#include 	<sys/ioctl.h>
@@ -43,6 +50,7 @@
 
 #include	<sndfile.h>
 
+#define	SIGNED_SIZEOF(x)	((int) sizeof (x))
 #define	BUFFER_LEN		(2048)
 
 /*------------------------------------------------------------------------------
@@ -112,7 +120,8 @@ static int
 linux_open_dsp_device (int channels, int srate)
 {	int fd, stereo, temp, error ;
 
-	if ((fd = open ("/dev/dsp", O_WRONLY, 0)) == -1)
+	if ((fd = open ("/dev/dsp", O_WRONLY, 0)) == -1 &&
+		(fd = open ("/dev/sound/dsp", O_WRONLY, 0)) == -1)
 	{	perror("linux_open_dsp_device : open ") ;
 		exit (1) ;
 		} ;
@@ -181,8 +190,15 @@ macosx_audio_out_callback (AudioDeviceID device, const AudioTimeStamp* current_t
 	AudioBufferList*	data_out, const AudioTimeStamp* time_out,
 	void* client_data)
 {	MacOSXAudioData	*audio_data ;
-	int		k, size, sample_count, read_count ;
-	float	*buffer, rate = 44100 ;
+	int		size, sample_count, read_count ;
+	float	*buffer ;
+
+	/* Prevent compiler warnings. */
+	device = device ;
+	current_time = current_time ;
+	data_in = data _in ;
+	time_in = time_in ;
+	time_out = time_out ;
 	
 	audio_data = (MacOSXAudioData*) client_data ;
 
@@ -338,7 +354,7 @@ win32_play_data (Win32_Audio_Data *audio_data)
 
 	if (thisread > 0)
 	{	/* Fix buffer length is only a partial block. */
-		if (thisread * sizeof (short) < audio_data->bufferlen)
+		if (thisread * SIGNED_SIZEOF (short) < audio_data->bufferlen)
 			audio_data->whdr [audio_data->current].dwBufferLength = thisread * sizeof (short) ;
 
 		/* Queue the WAVEHDR */
@@ -578,7 +594,12 @@ main (int argc, char *argv [])
 		} ;
 
 #if defined (__linux__)
+	#if (defined (HAVE_ALSA) && HAVE_ALSA)
+		if (access ("/proc/asound/cards", R_OK) == 0)
+			alsa_play (argc, argv) ;
+	#else
 	linux_play (argc, argv) ;
+	#endif
 #elif (defined (__MACH__) && defined (__APPLE__))
 	macosx_play (argc, argv) ;
 #elif (defined (sun) && defined (unix))
@@ -598,3 +619,10 @@ main (int argc, char *argv [])
 
 	return 0 ;
 } /* main */
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: 8fc4110d-6cec-4e03-91df-0f384cabedac
+*/
