@@ -17,7 +17,6 @@
 */
 
 #include	<stdio.h>
-#include	<unistd.h>
 #include	<fcntl.h>
 #include	<string.h>
 #include	<ctype.h>
@@ -149,6 +148,11 @@ au_open	(SF_PRIVATE *psf)
 				error = pcm_init (psf) ;
 				break ;
 
+		case SF_FORMAT_ALAW :	/* 8-bit Alaw encoding. */
+				alaw_init (psf) ;
+				break ;
+
+		/* Lite remove start */
 		case  SF_FORMAT_FLOAT :	/* 32-bit floats. */
 				error = float32_init (psf) ;
 				break ;
@@ -157,10 +161,6 @@ au_open	(SF_PRIVATE *psf)
 				error = double64_init (psf) ;
 				break ;
 
-		case  SF_FORMAT_ALAW :	/* 8-bit Alaw encoding. */
-				alaw_init (psf) ;
-				break ;
-
 		case  SF_FORMAT_G721_32 :
 				if (psf->mode == SFM_READ)
 					error = au_g72x_reader_init (psf, AU_H_G721_32) ;
@@ -184,6 +184,7 @@ au_open	(SF_PRIVATE *psf)
 					error = au_g72x_writer_init (psf, AU_H_G723_40) ;
 				psf->sf.seekable = SF_FALSE ;
 				break ;
+		/* Lite remove end */
 
 		default :   break ;
 		} ;
@@ -229,23 +230,7 @@ static int
 au_close	(SF_PRIVATE  *psf)
 {
 	if (psf->mode == SFM_WRITE || psf->mode == SFM_RDWR)
-	{	/*  Now we know for certain the length of the file we can
-		 *  re-write correct values for the datasize header element.
-		 */
-
-		psf_fseek (psf, 0, SEEK_END) ;
-		psf->filelength = psf_ftell (psf) ;
-
-		psf->datalength = psf->filelength - AU_DATA_OFFSET ;
-		psf_fseek (psf, 0, SEEK_SET) ;
-
-		psf->sf.frames = psf->datalength / psf->blockwidth ;
-		au_write_header (psf, SF_FALSE) ;
-		} ;
-
-	if (psf->fdata)
-		free (psf->fdata) ;
-	psf->fdata = NULL ;
+		au_write_header (psf, SF_TRUE) ;
 
 	return 0 ;
 } /* au_close */
@@ -253,14 +238,15 @@ au_close	(SF_PRIVATE  *psf)
 static int
 au_write_header (SF_PRIVATE *psf, int calc_length)
 {	sf_count_t	current ;
-	int			encoding ;
+	int			encoding, datalength ;
+
+	if (psf->pipeoffset > 0)
+		return 0 ;
 
 	current = psf_ftell (psf) ;
 
 	if (calc_length)
-	{	psf_fseek (psf, 0, SEEK_END) ;
-		psf->filelength = psf_ftell (psf) ;
-		psf_fseek (psf, 0, SEEK_SET) ;
+	{	psf->filelength = psf_get_filelen (psf) ;
 
 		psf->datalength = psf->filelength - psf->dataoffset ;
 		if (psf->dataend)
@@ -276,15 +262,31 @@ au_write_header (SF_PRIVATE *psf, int calc_length)
 	/* Reset the current header length to zero. */
 	psf->header [0] = 0 ;
 	psf->headindex = 0 ;
+
+	/*
+	** Only attempt to seek if we are not writng to a pipe. If we are
+	** writing to a pipe we shouldn't be here anyway.
+	*/
+	if (psf->is_pipe == SF_FALSE)
 	psf_fseek (psf, 0, SEEK_SET) ;
 
+	/*
+	**	AU format files allow a datalength value of -1 if the datalength
+	**	is not know at the time the header is written.
+	**	Also use this value of -1 if the datalength > 2 gigabytes.
+	*/
+	if (psf->datalength	< 0 || psf->datalength > 0x7FFFFFFF)
+		datalength = -1 ;
+	else
+		datalength = (int) (psf->datalength & 0x7FFFFFFF) ;
+
 	if (psf->endian == SF_ENDIAN_BIG)
 	{	psf_binheader_writef (psf, "Em4", DOTSND_MARKER, AU_DATA_OFFSET) ;
-		psf_binheader_writef (psf, "Et8444", psf->datalength, encoding, psf->sf.samplerate, psf->sf.channels) ;
+		psf_binheader_writef (psf, "E4444", datalength, encoding, psf->sf.samplerate, psf->sf.channels) ;
 		}
 	else if  (psf->endian == SF_ENDIAN_LITTLE)
 	{	psf_binheader_writef (psf, "em4", DNSDOT_MARKER, AU_DATA_OFFSET) ;
-		psf_binheader_writef (psf, "et8444", psf->datalength, encoding, psf->sf.samplerate, psf->sf.channels) ;
+		psf_binheader_writef (psf, "e4444", datalength, encoding, psf->sf.samplerate, psf->sf.channels) ;
 		}
 	else
 		return (psf->error = SFE_BAD_OPEN_FORMAT) ;
@@ -349,13 +351,21 @@ au_read_header (SF_PRIVATE *psf)
 	else
 		return SFE_AU_NO_DOTSND ;
 
-
 	psf_log_printf (psf, "  Data Offset : %d\n", au_fmt.dataoffset) ;
 
-	if (au_fmt.datasize == -1 || au_fmt.dataoffset + au_fmt.datasize == psf->filelength)
+	if (psf->fileoffset > 0 && au_fmt.datasize == -1)
+	{	psf_log_printf (psf, "  Data Size   : -1\n") ;
+		return SFE_AU_EMBED_BAD_LEN ;
+		} ;
+	
+	if (psf->fileoffset > 0)
+	{	psf->filelength = au_fmt.dataoffset + au_fmt.datasize ;
+		psf_log_printf (psf, "  Data Size   : %d\n", au_fmt.datasize) ;
+		}
+	else if (au_fmt.datasize == -1 || au_fmt.dataoffset + au_fmt.datasize == psf->filelength)
 		psf_log_printf (psf, "  Data Size   : %d\n", au_fmt.datasize) ;
 	else if (au_fmt.dataoffset + au_fmt.datasize < psf->filelength)
-	{	psf->filelength = au_fmt.dataoffset + au_fmt.dataoffset ;
+	{	psf->filelength = au_fmt.dataoffset + au_fmt.datasize ;
 		psf_log_printf (psf, "  Data Size   : %d\n", au_fmt.datasize) ;
 		}
 	else
@@ -367,8 +377,8 @@ au_read_header (SF_PRIVATE *psf)
  	psf->dataoffset = au_fmt.dataoffset ;
 	psf->datalength = psf->filelength - psf->dataoffset ;
 
- 	if (psf_fseek (psf, psf->dataoffset, SEEK_SET) != psf->dataoffset)
-		return SFE_BAD_SEEK ;
+	if (psf_ftell (psf) < psf->dataoffset)
+		psf_binheader_readf (psf, "j", psf->dataoffset - psf_ftell (psf)) ;
 
 	psf->close = au_close ;
 
@@ -475,3 +485,10 @@ au_read_header (SF_PRIVATE *psf)
 
 	return 0 ;
 } /* au_read_header */
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: 31f691b1-cde9-4ed2-9469-6bca60fb9cd0
+*/
