@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2003 Erik de Castro Lopo <erikd@zip.com.au>
+** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@zip.com.au>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -26,54 +26,6 @@
 #include	"sfendian.h"
 #include	"common.h"
 
-/*-----------------------------------------------------------------------------------------------
-*/
-
-int
-subformat_to_bytewidth (int format)
-{
-	switch (format)
-	{	case SF_FORMAT_PCM_U8 :
-		case SF_FORMAT_PCM_S8 :
-				return 1 ;
-		case SF_FORMAT_PCM_16 :
-				return 2 ;
-		case SF_FORMAT_PCM_24 :
-				return 3 ;
-		case SF_FORMAT_PCM_32 :
-		case SF_FORMAT_FLOAT :
-				return 4 ;
-		case SF_FORMAT_DOUBLE :
-				return 8 ;
-		};
-
-	return 0 ;
-} /* subformat_to_bytewidth */
-
-int
-s_bitwidth_to_subformat (int bits)
-{	static int array [] =
-	{	SF_FORMAT_PCM_S8, SF_FORMAT_PCM_16, SF_FORMAT_PCM_24, SF_FORMAT_PCM_32
-		} ;
-
-	if (bits < 8 || bits > 32)
-		return 0 ;
-
-	return array [((bits + 7) / 8) - 1] ;
-} /* bitwidth_to_subformat */
-
-int
-u_bitwidth_to_subformat (int bits)
-{	static int array [] =
-	{	SF_FORMAT_PCM_U8, SF_FORMAT_PCM_16, SF_FORMAT_PCM_24, SF_FORMAT_PCM_32
-		} ;
-
-	if (bits < 8 || bits > 32)
-		return 0 ;
-
-	return array [((bits + 7) / 8) - 1] ;
-} /* bitwidth_to_subformat */
-
 /*-----------------------------------------------------------------------------------------------
 ** psf_log_printf allows libsndfile internal functions to print to an internal logbuffer which
 ** can later be displayed.
@@ -84,21 +36,20 @@ u_bitwidth_to_subformat (int bits)
 */
 
 #define LOG_PUTCHAR(a,b)									\
-			if ((a)->logindex < SF_BUFFER_LEN - 1)			\
+		{	if ((a)->logindex < SIGNED_SIZEOF ((a)->logbuffer) - 1)\
 			{	(a)->logbuffer [(a)->logindex++] = (b) ;	\
 				(a)->logbuffer [(a)->logindex] = 0 ;		\
 				}											\
-			else											\
-				break ;
+			}
 
 void
-psf_log_printf (SF_PRIVATE *psf, char *format, ...)
+psf_log_printf (SF_PRIVATE *psf, const char *format, ...)
 {	va_list			ap ;
 	unsigned int	u ;
-	int     		d, tens, shift, width, width_specifier ;
-	char    		c, *strptr, istr [5], lead_char ;
+	int				d, tens, shift, width, width_specifier, left_align ;
+	char			c, *strptr, istr [5], lead_char, sign_char ;
 
-	va_start(ap, format);
+	va_start (ap, format) ;
 
 	while ((c = *format++))
 	{	if (c != '%')
@@ -106,12 +57,40 @@ psf_log_printf (SF_PRIVATE *psf, char *format, ...)
 			continue ;
 			} ;
 
-		if (format [1] == '%') /* Handle %% */
+		if (format [0] == '%') /* Handle %% */
 		{ 	LOG_PUTCHAR (psf, '%') ;
 			format ++ ;
+			continue ;
 			} ;
 
-		lead_char = (format [0] == '0') ? '0' : ' ' ;
+		sign_char = 0 ;
+		left_align = SF_FALSE ;
+		while (1)
+		{	switch (format [0])
+			{	case ' ' :
+				case '+' :
+					sign_char = format [0] ;
+					format ++ ;
+					continue ;
+
+				case '-' :
+					left_align = SF_TRUE ;
+					format ++ ;
+					continue ;
+
+				default : break ;
+				} ;
+
+			break ;
+			} ;
+
+		if (format [0] == 0)
+			break ;
+
+		lead_char = ' ' ;
+		if (format [0] == '0')
+			lead_char = '0' ;
+
 		width_specifier = 0 ;
 		while ((c = *format++) && isdigit (c))
 			width_specifier = width_specifier * 10 + (c - '0') ;
@@ -123,23 +102,28 @@ psf_log_printf (SF_PRIVATE *psf, char *format, ...)
 
 			case 's': /* string */
 					strptr = va_arg (ap, char *) ;
+					if (strptr == NULL)
+						break ;
+					width_specifier -= strlen (strptr) ;
+					if (left_align == SF_FALSE)
+						while (width_specifier -- > 0)
+							LOG_PUTCHAR (psf, ' ') ;
 					while (*strptr)
 						LOG_PUTCHAR (psf, *strptr++) ;
-					break;
+					while (width_specifier -- > 0)
+						LOG_PUTCHAR (psf, ' ') ;
+					break ;
 
 			case 'd': /* int */
 					d = va_arg (ap, int) ;
 
-					if (d == 0)
-					{	while (-- width_specifier > 0)
-							LOG_PUTCHAR (psf, lead_char) ;
-						LOG_PUTCHAR (psf, '0') ;
-						break ;
-						}
 					if (d < 0)
-					{	LOG_PUTCHAR (psf, '-') ;
-						d = -d ;
+					{	d = -d ;
+						sign_char = '-' ;
+						if (lead_char != '0' && left_align == SF_FALSE)
+							width_specifier -- ;
 						} ;
+
 					tens = 1 ;
 					width = 1 ;
 					while (d / tens >= 10)
@@ -147,17 +131,39 @@ psf_log_printf (SF_PRIVATE *psf, char *format, ...)
 						width ++ ;
 						} ;
 
-					while (width_specifier > width)
-					{	LOG_PUTCHAR (psf, lead_char) ;
-						width_specifier-- ;
+					width_specifier -= width ;
+
+					if (sign_char == ' ')
+					{	LOG_PUTCHAR (psf, ' ') ;
+						width_specifier -- ;
 						} ;
 
+					if (left_align == SF_FALSE && lead_char != '0')
+					{	if (sign_char == '+')
+							width_specifier -- ;
+
+						while (width_specifier -- > 0)
+							LOG_PUTCHAR (psf, lead_char) ;
+						} ;
+
+					if (sign_char == '+' || sign_char == '-')
+					{	LOG_PUTCHAR (psf, sign_char) ;
+						width_specifier -- ;
+						} ;
+
+					if (left_align == SF_FALSE)
+						while (width_specifier -- > 0)
+							LOG_PUTCHAR (psf, lead_char) ;
+
 					while (tens > 0)
 					{	LOG_PUTCHAR (psf, '0' + d / tens) ;
 						d %= tens ;
 						tens /= 10 ;
 						} ;
-					break;
+
+					while (width_specifier -- > 0)
+						LOG_PUTCHAR (psf, lead_char) ;
+					break ;
 
 			case 'D': /* sf_count_t */
 					{	sf_count_t		D, Tens ;
@@ -192,17 +198,11 @@ psf_log_printf (SF_PRIVATE *psf, char *format, ...)
 							Tens /= 10 ;
 							} ;
 						} ;
-					break;
+					break ;
 
 			case 'u': /* unsigned int */
 					u = va_arg (ap, unsigned int) ;
 
-					if (u == 0)
-					{	while (-- width_specifier > 0)
-							LOG_PUTCHAR (psf, lead_char) ;
-						LOG_PUTCHAR (psf, '0') ;
-						break ;
-						}
 					tens = 1 ;
 					width = 1 ;
 					while (u / tens >= 10)
@@ -210,22 +210,44 @@ psf_log_printf (SF_PRIVATE *psf, char *format, ...)
 						width ++ ;
 						} ;
 
-					while (width_specifier > width)
-					{	LOG_PUTCHAR (psf, lead_char) ;
-						width_specifier-- ;
+					width_specifier -= width ;
+
+					if (sign_char == ' ')
+					{	LOG_PUTCHAR (psf, ' ') ;
+						width_specifier -- ;
 						} ;
 
+					if (left_align == SF_FALSE && lead_char != '0')
+					{	if (sign_char == '+')
+							width_specifier -- ;
+
+						while (width_specifier -- > 0)
+							LOG_PUTCHAR (psf, lead_char) ;
+						} ;
+
+					if (sign_char == '+' || sign_char == '-')
+					{	LOG_PUTCHAR (psf, sign_char) ;
+						width_specifier -- ;
+						} ;
+
+					if (left_align == SF_FALSE)
+						while (width_specifier -- > 0)
+							LOG_PUTCHAR (psf, lead_char) ;
+
 					while (tens > 0)
 					{	LOG_PUTCHAR (psf, '0' + u / tens) ;
 						u %= tens ;
 						tens /= 10 ;
 						} ;
-					break;
+
+					while (width_specifier -- > 0)
+						LOG_PUTCHAR (psf, lead_char) ;
+					break ;
 
 			case 'c': /* char */
 					c = va_arg (ap, int) & 0xFF ;
-					LOG_PUTCHAR (psf, c);
-					break;
+					LOG_PUTCHAR (psf, c) ;
+					break ;
 
 			case 'X': /* hex */
 					d = va_arg (ap, int) ;
@@ -253,10 +275,10 @@ psf_log_printf (SF_PRIVATE *psf, char *format, ...)
 						LOG_PUTCHAR (psf, (c > 9) ? c + 'A' - 10 : c + '0') ;
 						shift -= 4 ;
 						} ;
-					break;
+					break ;
 
 			case 'M': /* int2str */
-					d = va_arg (ap, int);
+					d = va_arg (ap, int) ;
 					if (CPU_IS_LITTLE_ENDIAN)
 					{	istr [0] = d & 0xFF ;
 						istr [1] = (d >> 8) & 0xFF ;
@@ -275,31 +297,7 @@ psf_log_printf (SF_PRIVATE *psf, char *format, ...)
 					{	c = *strptr++ ;
 						LOG_PUTCHAR (psf, c) ;
 						} ;
-					break;
-
-			case 'C': /* int2str */
-					{	sf_count_t lld, Tens ;
-
-						lld = va_arg (ap, sf_count_t);
-
-						if (lld == 0)
-						{	LOG_PUTCHAR (psf, '0') ;
 							break ;
-							}
-						if (lld < 0)
-						{	LOG_PUTCHAR (psf, '-') ;
-							lld = -lld ;
-							} ;
-						Tens = 1 ;
-						while (lld / Tens >= 10)
-							Tens *= 10 ;
-						while (Tens > 0)
-						{	LOG_PUTCHAR (psf, '0' + lld / Tens) ;
-							lld %= Tens ;
-							Tens /= 10 ;
-							} ;
-						} ;
-					break;
 
 			default :
 					LOG_PUTCHAR (psf, '*') ;
@@ -309,10 +307,11 @@ psf_log_printf (SF_PRIVATE *psf, char *format, ...)
 			} /* switch */
 		} /* while */
 
-	va_end (ap);
+	va_end (ap) ;
 	return ;
 } /* psf_log_printf */
 
+#ifndef PSF_LOG_PRINTF_ONLY
 /*-----------------------------------------------------------------------------------------------
 **  ASCII header printf functions.
 **  Some formats (ie NIST) use ascii text in their headers.
@@ -322,7 +321,7 @@ psf_log_printf (SF_PRIVATE *psf, char *format, ...)
 */
 
 void
-psf_asciiheader_printf (SF_PRIVATE *psf, char *format, ...)
+psf_asciiheader_printf (SF_PRIVATE *psf, const char *format, ...)
 {	va_list	argptr ;
 	int  maxlen ;
 	char *start ;
@@ -384,18 +383,18 @@ psf_asciiheader_printf (SF_PRIVATE *psf, char *format, ...)
 ** seg. fault when asked to write an int or short to a non-int/short aligned address.
 */
 
-#define	PUT_BYTE(psf,x)		if ((psf)->headindex < sizeof ((psf)->header) - 1)	\
+#define	PUT_BYTE(psf,x)		if ((psf)->headindex < SIGNED_SIZEOF ((psf)->header) - 1)	\
 							{	(psf)->header [(psf)->headindex++] = (x) ;   }
 
 #if (CPU_IS_BIG_ENDIAN == 1)
-#define	PUT_MARKER(psf,x)	if ((psf)->headindex < sizeof ((psf)->header) - 4)			\
+#define	PUT_MARKER(psf,x)	if ((psf)->headindex < SIGNED_SIZEOF ((psf)->header) - 4)	\
 							{	(psf)->header [(psf)->headindex++] = ((x) >> 24) ;		\
 								(psf)->header [(psf)->headindex++] = ((x) >> 16) ;		\
-								(psf)->header [(psf)->headindex++] = ((x) >>  8);		\
+								(psf)->header [(psf)->headindex++] = ((x) >> 8) ;		\
 								(psf)->header [(psf)->headindex++] = (x) ;   }
 
 #elif (CPU_IS_LITTLE_ENDIAN == 1)
-#define	PUT_MARKER(psf,x)	if ((psf)->headindex < sizeof ((psf)->header) - 4)			\
+#define	PUT_MARKER(psf,x)	if ((psf)->headindex < SIGNED_SIZEOF ((psf)->header) - 4)	\
 							{	(psf)->header [(psf)->headindex++] = (x) ;				\
 								(psf)->header [(psf)->headindex++] = ((x) >>  8) ;		\
 								(psf)->header [(psf)->headindex++] = ((x) >> 16) ;		\
@@ -406,38 +405,38 @@ psf_asciiheader_printf (SF_PRIVATE *psf, char *format, ...)
 #endif
 
 
-#define	PUT_BE_SHORT(psf,x)	if ((psf)->headindex < sizeof ((psf)->header) - 2)			\
+#define	PUT_BE_SHORT(psf,x)	if ((psf)->headindex < SIGNED_SIZEOF ((psf)->header) - 2)	\
 							{	(psf)->header [(psf)->headindex++] = ((x) >> 8) ;		\
 								(psf)->header [(psf)->headindex++] = (x) ; 		}
 
-#define	PUT_LE_SHORT(psf,x)	if ((psf)->headindex < sizeof ((psf)->header) - 2)			\
+#define	PUT_LE_SHORT(psf,x)	if ((psf)->headindex < SIGNED_SIZEOF ((psf)->header) - 2)	\
 							{	(psf)->header [(psf)->headindex++] = (x) ;				\
 								(psf)->header [(psf)->headindex++] = ((x) >> 8) ;	}
 
-#define	PUT_BE_3BYTE(psf,x)	if ((psf)->headindex < sizeof ((psf)->header) - 3)			\
+#define	PUT_BE_3BYTE(psf,x)	if ((psf)->headindex < SIGNED_SIZEOF ((psf)->header) - 3)	\
 							{	(psf)->header [(psf)->headindex++] = ((x) >> 16) ;		\
 								(psf)->header [(psf)->headindex++] = ((x) >> 8) ;		\
 								(psf)->header [(psf)->headindex++] = (x) ;		}
 
-#define	PUT_LE_3BYTE(psf,x)	if ((psf)->headindex < sizeof ((psf)->header) - 3)			\
+#define	PUT_LE_3BYTE(psf,x)	if ((psf)->headindex < SIGNED_SIZEOF ((psf)->header) - 3)	\
 							{	(psf)->header [(psf)->headindex++] = (x) ;				\
 								(psf)->header [(psf)->headindex++] = ((x) >> 8) ;   	\
 								(psf)->header [(psf)->headindex++] = ((x) >> 16) ;	}
 
-#define	PUT_BE_INT(psf,x)	if ((psf)->headindex < sizeof ((psf)->header) - 4)			\
+#define	PUT_BE_INT(psf,x)	if ((psf)->headindex < SIGNED_SIZEOF ((psf)->header) - 4)	\
 							{	(psf)->header [(psf)->headindex++] = ((x) >> 24) ;		\
 								(psf)->header [(psf)->headindex++] = ((x) >> 16) ;		\
 								(psf)->header [(psf)->headindex++] = ((x) >> 8) ;		\
 								(psf)->header [(psf)->headindex++] = (x) ;		}
 
-#define	PUT_LE_INT(psf,x)	if ((psf)->headindex < sizeof ((psf)->header) - 4)			\
+#define	PUT_LE_INT(psf,x)	if ((psf)->headindex < SIGNED_SIZEOF ((psf)->header) - 4)	\
 							{	(psf)->header [(psf)->headindex++] = (x) ;				\
 								(psf)->header [(psf)->headindex++] = ((x) >> 8) ;   	\
 								(psf)->header [(psf)->headindex++] = ((x) >> 16) ;		\
 								(psf)->header [(psf)->headindex++] = ((x) >> 24) ;	}
 
 #if (SIZEOF_SF_COUNT_T == 4)
-#define	PUT_BE_8BYTE(psf,x)	if ((psf)->headindex < sizeof ((psf)->header) - 8)			\
+#define	PUT_BE_8BYTE(psf,x)	if ((psf)->headindex < SIGNED_SIZEOF ((psf)->header) - 8)	\
 							{	(psf)->header [(psf)->headindex++] = 0 ;				\
 								(psf)->header [(psf)->headindex++] = 0 ;				\
 								(psf)->header [(psf)->headindex++] = 0 ;				\
@@ -447,7 +446,7 @@ psf_asciiheader_printf (SF_PRIVATE *psf, char *format, ...)
 								(psf)->header [(psf)->headindex++] = ((x) >> 8) ;		\
 								(psf)->header [(psf)->headindex++] = (x) ;		}
 
-#define	PUT_LE_8BYTE(psf,x)	if ((psf)->headindex < sizeof ((psf)->header) - 8)			\
+#define	PUT_LE_8BYTE(psf,x)	if ((psf)->headindex < SIGNED_SIZEOF ((psf)->header) - 8)	\
 							{	(psf)->header [(psf)->headindex++] = (x) ;				\
 								(psf)->header [(psf)->headindex++] = ((x) >> 8) ;   	\
 								(psf)->header [(psf)->headindex++] = ((x) >> 16) ;		\
@@ -458,7 +457,7 @@ psf_asciiheader_printf (SF_PRIVATE *psf, char *format, ...)
 								(psf)->header [(psf)->headindex++] = 0 ;	}
 
 #elif  (SIZEOF_SF_COUNT_T == 8)
-#define	PUT_BE_8BYTE(psf,x)	if ((psf)->headindex < sizeof ((psf)->header) - 8)			\
+#define	PUT_BE_8BYTE(psf,x)	if ((psf)->headindex < SIGNED_SIZEOF ((psf)->header) - 8)	\
 							{	(psf)->header [(psf)->headindex++] = ((x) >> 56) ;		\
 								(psf)->header [(psf)->headindex++] = ((x) >> 48) ;		\
 								(psf)->header [(psf)->headindex++] = ((x) >> 40) ;		\
@@ -468,7 +467,7 @@ psf_asciiheader_printf (SF_PRIVATE *psf, char *format, ...)
 								(psf)->header [(psf)->headindex++] = ((x) >> 8) ;		\
 								(psf)->header [(psf)->headindex++] = (x) ;		}
 
-#define	PUT_LE_8BYTE(psf,x)	if ((psf)->headindex < sizeof ((psf)->header) - 8)			\
+#define	PUT_LE_8BYTE(psf,x)	if ((psf)->headindex < SIGNED_SIZEOF ((psf)->header) - 8)	\
 							{	(psf)->header [(psf)->headindex++] = (x) ;				\
 								(psf)->header [(psf)->headindex++] = ((x) >> 8) ;   	\
 								(psf)->header [(psf)->headindex++] = ((x) >> 16) ;		\
@@ -480,8 +479,9 @@ psf_asciiheader_printf (SF_PRIVATE *psf, char *format, ...)
 #else
 #error "SIZEOF_SF_COUNT_T is not defined."
 #endif
+
 int
-psf_binheader_writef (SF_PRIVATE *psf, char *format, ...)
+psf_binheader_writef (SF_PRIVATE *psf, const char *format, ...)
 {	va_list	argptr ;
 	sf_count_t 		countdata ;
 	unsigned long 	longdata ;
@@ -495,7 +495,7 @@ psf_binheader_writef (SF_PRIVATE *psf, char *format, ...)
 
 	trunc_8to4 = SF_FALSE ;
 
-	va_start(argptr, format);
+	va_start (argptr, format) ;
 
 	while ((c = *format++))
 	{	switch (c)
@@ -665,7 +665,7 @@ psf_binheader_writef (SF_PRIVATE *psf, char *format, ...)
 			} ;
 		} ;
 
-	va_end(argptr);
+	va_end (argptr) ;
 	return count ;
 } /* psf_binheader_writef */
 
@@ -686,7 +686,7 @@ psf_binheader_writef (SF_PRIVATE *psf, char *format, ...)
 						  ((ptr) [2] <<  8) | ((ptr) [3]) )
 
 #elif (CPU_IS_LITTLE_ENDIAN == 1)
-#define	GET_MARKER(ptr)	( ((ptr) [0]      ) | ((ptr) [1] <<  8) |	\
+#define	GET_MARKER(ptr)	(	((ptr) [0])			| ((ptr) [1] << 8) |	\
 						  ((ptr) [2] << 16) | ((ptr) [3] << 24) )
 
 #else
@@ -724,6 +724,107 @@ psf_binheader_writef (SF_PRIVATE *psf, char *format, ...)
 
 #endif
 
+static int
+header_read (SF_PRIVATE *psf, void *ptr, int bytes)
+{	int count = 0 ;
+
+	if (psf->headindex + bytes > SIGNED_SIZEOF (psf->header))
+	{	if (psf->headend < SIGNED_SIZEOF (psf->header))
+			psf_log_printf (psf, "Warning : Further header read would overflow buffer.\n") ;
+
+		bytes = sizeof (psf->header) - psf->headindex ;
+		if (bytes <= 0)
+			return 0;
+		printf ("\nR sizeof (header): %u    headindex: %4u    bytes: %d\n", sizeof (psf->header), psf->headindex, bytes) ;
+		} ;
+		
+
+	if (psf->headindex + bytes > psf->headend)
+	{	count = psf_fread (psf->header + psf->headend, 1, bytes - (psf->headend - psf->headindex), psf) ;
+		if (count != bytes - (int) (psf->headend - psf->headindex))
+		{	psf_log_printf (psf, "Error : psf_fread returned short count.\n") ;
+			return 0 ;
+			} ;
+		psf->headend += count ;
+		} ;
+
+	memcpy (ptr, psf->header + psf->headindex, bytes) ;
+	psf->headindex += bytes ;
+
+	return bytes ;
+} /* header_read */
+
+static void
+header_seek (SF_PRIVATE *psf, sf_count_t position, int whence)
+{
+	switch (whence)
+	{	case SEEK_SET :
+			if (position > sizeof (psf->header))
+			{	/* Too much header to cache so just seek instead. */
+				psf_fseek (psf, position, whence) ;
+//printf ("A sizeof (header): %u    headindex: %4u\n", sizeof (psf->header), psf->headindex) ;
+				return ;
+				} ;
+			if (position > psf->headend)
+				psf->headend += psf_fread (psf->header + psf->headend, 1, position - psf->headend, psf) ;
+			psf->headindex = position ;
+//printf ("B sizeof (header): %u    headindex: %4u\n", sizeof (psf->header), psf->headindex) ;
+			break ;
+
+		case SEEK_CUR :
+			if (psf->headindex + position < 0 || psf->headindex >= SIGNED_SIZEOF (psf->header))
+				break ;
+
+			if (psf->headindex + position <= psf->headend)
+			{	psf->headindex += position ;
+				break ;
+				} ;
+
+			if (psf->headindex + position > SIGNED_SIZEOF (psf->header))
+			{	psf->headend += psf_fread (psf->header + psf->headend, 1, SIGNED_SIZEOF (psf->header) - psf->headend, psf) ;
+				psf->headindex = psf->headend = SIGNED_SIZEOF (psf->header) ;
+
+				psf_fseek (psf, position - (SIGNED_SIZEOF (psf->header) - psf->headend), SEEK_CUR) ;
+				break ;
+				} ;
+
+			psf->headend += psf_fread (psf->header + psf->headend, 1, position - (psf->headend - psf->headindex), psf) ;
+			psf->headindex = psf->headend ;
+			break ;
+
+		case SEEK_END :
+		default :
+			psf_log_printf (psf, "Bad whence param in header_seek().\n") ;
+			break ;
+		} ;
+
+	return ;
+} /* header_seek */
+
+static int
+header_gets (SF_PRIVATE *psf, char *ptr, int bufsize)
+{
+	int		k ;
+
+	for (k = 0 ; k < bufsize - 1 ; k++)
+	{	if (psf->headindex < psf->headend)
+		{	ptr [k] = psf->header [psf->headindex] ;
+			psf->headindex ++ ;
+			}
+		else
+		{	psf->headend += psf_fread (psf->header + psf->headend, 1, 1, psf) ;
+			ptr [k] = psf->header [psf->headindex] ;
+			psf->headindex = psf->headend ;
+			} ;
+
+		if (ptr [k] == '\n')
+			break ;
+		} ;
+
+	ptr [k] = 0 ;
+
+	return k ;
+} /* header_gets */
 
 int
 psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
@@ -733,12 +834,11 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 	unsigned int 	*intptr, intdata ;
 	unsigned short	*shortptr ;
 	char    		*charptr ;
-	int				position ;
 	float			*floatptr ;
 	double			*doubleptr ;
 	size_t			size ;
 	char			c ;
-	int				count = 0 ;
+	int				count = 0, position ;
 
 	if (! format)
 		return psf_ftell (psf) ;
@@ -758,14 +858,14 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 			case 'm' :
 					intptr = va_arg (argptr, unsigned int*) ;
 					ucptr = (unsigned char*) intptr ;
-					count += psf_fread (ucptr, 1, sizeof (int), psf) ;
+					count += header_read (psf, ucptr, sizeof (int)) ;
 					*intptr = GET_MARKER (ucptr) ;
 					break ;
 
 			case 'h' :
 					intptr = va_arg (argptr, unsigned int*) ;
 					ucptr = (unsigned char*) intptr ;
-					count += psf_fread (sixteen_bytes, 1, sizeof (sixteen_bytes), psf) ;
+					count += header_read (psf, sixteen_bytes, sizeof (sixteen_bytes)) ;
 					{	int k ;
 						intdata = 0 ;
 						for (k = 0 ; k < 16 ; k++)
@@ -776,13 +876,13 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 
 			case '1' :
 					charptr = va_arg (argptr, char*) ;
-					count += psf_fread (charptr, 1, sizeof (char), psf) ;
+					count += header_read (psf, charptr, sizeof (char)) ;
 					break ;
 
 			case '2' :
 					shortptr = va_arg (argptr, unsigned short*) ;
 					ucptr = (unsigned char*) shortptr ;
-					count += psf_fread (ucptr, 1, sizeof (short), psf) ;
+					count += header_read (psf, ucptr, sizeof (short)) ;
 					if (psf->rwf_endian == SF_ENDIAN_BIG)
 						*shortptr = GET_BE_SHORT (ucptr) ;
 					else
@@ -791,7 +891,7 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 
 			case '3' :
 					intptr = va_arg (argptr, unsigned int*) ;
-					count += psf_fread (sixteen_bytes, 1, 3, psf) ;
+					count += header_read (psf, sixteen_bytes, 3) ;
 					if (psf->rwf_endian == SF_ENDIAN_BIG)
 						*intptr = GET_BE_3BYTE (sixteen_bytes) ;
 					else
@@ -801,7 +901,7 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 			case '4' :
 					intptr = va_arg (argptr, unsigned int*) ;
 					ucptr = (unsigned char*) intptr ;
-					count += psf_fread (ucptr, 1, sizeof (int), psf) ;
+					count += header_read (psf, ucptr, sizeof (int)) ;
 					if (psf->rwf_endian == SF_ENDIAN_BIG)
 						*intptr = GET_BE_INT (ucptr) ;
 					else
@@ -810,7 +910,7 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 
 			case '8' :
 					countptr = va_arg (argptr, sf_count_t*) ;
-					count += psf_fread (sixteen_bytes, 1, 8, psf) ;
+					count += header_read (psf, sixteen_bytes, 8) ;
 					if (psf->rwf_endian == SF_ENDIAN_BIG)
 						countdata = GET_BE_8BYTE (sixteen_bytes) ;
 					else
@@ -821,7 +921,7 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 			case 'f' : /* Float conversion */
 					floatptr = va_arg (argptr, float *) ;
 					*floatptr = 0.0 ;
-					count += psf_fread (floatptr, 1, sizeof (float), psf) ;
+					count += header_read (psf, floatptr, sizeof (float)) ;
 					if (psf->rwf_endian == SF_ENDIAN_BIG)
 						*floatptr = float32_be_read ((unsigned char*) floatptr) ;
 					else
@@ -831,7 +931,7 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 			case 'd' : /* double conversion */
 					doubleptr = va_arg (argptr, double *) ;
 					*doubleptr = 0.0 ;
-					count += psf_fread (doubleptr, 1, sizeof (double), psf) ;
+					count += header_read (psf, doubleptr, sizeof (double)) ;
 					if (psf->rwf_endian == SF_ENDIAN_BIG)
 						*doubleptr = double64_be_read ((unsigned char*) doubleptr) ;
 					else
@@ -855,9 +955,14 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 					charptr = va_arg (argptr, char*) ;
 					size = va_arg (argptr, int) ;
 					if (size > 0)
-					{	memset (charptr, 0, size) ;
-						count += psf_fread (charptr, 1, size, psf) ;
-						} ;
+						count += header_read (psf, charptr, size) ;
+					break ;
+
+			case 'G' :
+					charptr = va_arg (argptr, char*) ;
+					size = va_arg (argptr, int) ;
+					if (size > 0)
+						count += header_gets (psf, charptr, size) ;
 					break ;
 
 			case 'z' :
@@ -875,15 +980,15 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 			case 'p' :
 					/* Get the seek position first. */
 					position = va_arg (argptr, int) ;
-					psf_fseek (psf, position, SEEK_SET) ;
-					count = 0 ;
+					header_seek (psf, position, SEEK_SET) ;
+					count = position ;
 					break ;
 
 			case 'j' :
 					/* Get the seek position first. */
 					position = va_arg (argptr, int) ;
-					psf_fseek (psf, position, SEEK_CUR) ;
-					count = 0 ;
+					header_seek (psf, position, SEEK_CUR) ;
+					count += position ;
 					break ;
 
 			default :
@@ -893,7 +998,7 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 			} ;
 		} ;
 
-	va_end (argptr);
+	va_end (argptr) ;
 
 	return count ;
 } /* psf_binheader_readf */
@@ -901,6 +1006,65 @@ psf_binheader_readf (SF_PRIVATE *psf, char const *format, ...)
 /*-----------------------------------------------------------------------------------------------
 */
 
+sf_count_t
+psf_default_seek (SF_PRIVATE *psf, int mode, sf_count_t samples_from_start)
+{	sf_count_t position, retval ;
+
+	if (! (psf->blockwidth && psf->dataoffset >= 0))
+	{	psf->error = SFE_BAD_SEEK ;
+		return	((sf_count_t) -1) ;
+		} ;
+
+	if (! psf->sf.seekable)
+	{	psf->error = SFE_NOT_SEEKABLE ;
+		return	((sf_count_t) -1) ;
+		} ;
+
+	position = psf->dataoffset + psf->blockwidth * samples_from_start ;
+
+	if ((retval = psf_fseek (psf, position, SEEK_SET)) != position)
+	{	psf->error = SFE_SEEK_FAILED ;
+		return ((sf_count_t) -1) ;
+		} ;
+
+	mode = mode ;
+
+	return samples_from_start ;
+} /* psf_default_seek */
+
+/*-----------------------------------------------------------------------------------------------
+*/
+
+void
+psf_hexdump (void *ptr, int len)
+{	char	ascii [17], *data ;
+	int		k, m ;
+
+	if ((data = ptr) == NULL)
+		return ;
+	if (len <= 0)
+		return ;
+
+	puts ("") ;
+	for (k = 0 ; k < len ; k += 16)
+	{	memset (ascii, ' ', sizeof (ascii)) ;
+
+		printf ("%08X: ", k) ;
+		for (m = 0 ; m < 16 && k + m < len ; m++)
+		{	printf (m == 8 ? " %02X " : "%02X ", data [k + m] & 0xFF) ;
+			ascii [m] = isprint (data [k + m]) ? data [k + m] : '.' ;
+			} ;
+
+		if (m <= 8) printf (" ") ;
+		for ( ; m < 16 ; m++) printf ("   ") ;
+
+		ascii [16] = 0 ;
+		printf (" %s\n", ascii) ;
+		} ;
+
+	puts ("") ;
+} /* psf_hexdump */
+
 void
 psf_log_SF_INFO (SF_PRIVATE *psf)
 {	psf_log_printf (psf, "---------------------------------\n") ;
@@ -916,27 +1080,6 @@ psf_log_SF_INFO (SF_PRIVATE *psf)
 	psf_log_printf (psf, "---------------------------------\n") ;
 } /* psf_dump_SFINFO */
 
-sf_count_t
-psf_default_seek (SF_PRIVATE *psf, int mode, sf_count_t samples_from_start)
-{	sf_count_t position, retval ;
-
-	if (! (psf->blockwidth && psf->dataoffset >= 0))
-	{	psf->error = SFE_BAD_SEEK ;
-		return	((sf_count_t) -1) ;
-		} ;
-
-	position = psf->dataoffset + psf->blockwidth * samples_from_start ;
-
-	if ((retval = psf_fseek (psf, position, SEEK_SET)) != position)
-	{	psf->error = SFE_SEEK_FAILED ;
-		return ((sf_count_t) -1) ;
-		} ;
-
-	mode = mode ;
-
-	return samples_from_start ;
-} /* psf_default_seek */
-
 /*========================================================================================
 */
 
@@ -983,6 +1126,59 @@ void  psf_get_date_str (char *str, int maxlen)
 	else
 		LSF_SNPRINTF (str, maxlen, "Unknown date") ;
 		
-
 	return ;
 } /* psf_get_date_str */
+
+int
+subformat_to_bytewidth (int format)
+{
+	switch (format)
+	{	case SF_FORMAT_PCM_U8 :
+		case SF_FORMAT_PCM_S8 :
+				return 1 ;
+		case SF_FORMAT_PCM_16 :
+				return 2 ;
+		case SF_FORMAT_PCM_24 :
+				return 3 ;
+		case SF_FORMAT_PCM_32 :
+		case SF_FORMAT_FLOAT :
+				return 4 ;
+		case SF_FORMAT_DOUBLE :
+				return 8 ;
+		} ;
+
+	return 0 ;
+} /* subformat_to_bytewidth */
+
+int
+s_bitwidth_to_subformat (int bits)
+{	static int array [] =
+	{	SF_FORMAT_PCM_S8, SF_FORMAT_PCM_16, SF_FORMAT_PCM_24, SF_FORMAT_PCM_32
+		} ;
+
+	if (bits < 8 || bits > 32)
+		return 0 ;
+
+	return array [((bits + 7) / 8) - 1] ;
+} /* bitwidth_to_subformat */
+
+int
+u_bitwidth_to_subformat (int bits)
+{	static int array [] =
+	{	SF_FORMAT_PCM_U8, SF_FORMAT_PCM_16, SF_FORMAT_PCM_24, SF_FORMAT_PCM_32
+		} ;
+
+	if (bits < 8 || bits > 32)
+		return 0 ;
+
+	return array [((bits + 7) / 8) - 1] ;
+} /* bitwidth_to_subformat */
+
+#endif /* PSF_LOG_PRINTF_ONLY */
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: 33e9795e-f717-461a-9feb-65d083a56395
+*/
