@@ -18,8 +18,8 @@
 
 
 #include	<stdio.h>
+#include	<stdlib.h>
 #include	<string.h>
-#include	<unistd.h>
 
 #include	"sndfile.h"
 #include	"config.h"
@@ -239,7 +239,7 @@ double64_init (SF_PRIVATE *psf)
 
 			default : break ;
 			} ;
-		};
+		} ;
 
 	psf->filelength = psf_get_filelen (psf) ;
 	psf->datalength = (psf->dataend) ? psf->dataend - psf->dataoffset :
@@ -281,7 +281,7 @@ double64_be_read (unsigned char *cptr)
 	double	dvalue ;
 
 	negative = (cptr [0] & 0x80) ? 1 : 0 ;
-	exponent = ((cptr [0] & 0x7F) << 4) | ((cptr [1] >> 4) & 0xF);
+	exponent = ((cptr [0] & 0x7F) << 4) | ((cptr [1] >> 4) & 0xF) ;
 
 	/* Might not have a 64 bit long, so load the mantissa into a double. */
 	dvalue = (((cptr [1] & 0xF) << 24) | (cptr [2] << 16) | (cptr [3] << 8) | cptr [4]) ;
@@ -313,7 +313,7 @@ double64_le_read (unsigned char *cptr)
 	double	dvalue ;
 
 	negative = (cptr [7] & 0x80) ? 1 : 0 ;
-	exponent = ((cptr [7] & 0x7F) << 4) | ((cptr [6] >> 4) & 0xF);
+	exponent = ((cptr [7] & 0x7F) << 4) | ((cptr [6] >> 4) & 0xF) ;
 
 	/* Might not have a 64 bit long, so load the mantissa into a double. */
 	dvalue = (((cptr [6] & 0xF) << 24) | (cptr [5] << 16) | (cptr [4] << 8) | cptr [3]) ;
@@ -438,9 +438,9 @@ double64_peak_update (SF_PRIVATE *psf, double *buffer, int count, int indx)
 				position = k ;
 				} ;
 
-		if (fmaxval > psf->peak.peak[chan].value)
-		{	psf->peak.peak[chan].value = fmaxval ;
-			psf->peak.peak[chan].position = psf->write_current + indx + (position / psf->sf.channels) ;
+		if (fmaxval > psf->peak.peak [chan].value)
+		{	psf->peak.peak [chan].value = fmaxval ;
+			psf->peak.peak [chan].position = psf->write_current + indx + (position / psf->sf.channels) ;
 			} ;
 		} ;
 
@@ -982,3 +982,10 @@ bd2d_write (double *buffer, int count)
 		} ;
 } /* bd2d_write */
 
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: 4ee243b7-8c7a-469b-869c-e9aa0ee3b77f
+*/
