@@ -17,7 +17,6 @@
 */
 
 #include	<stdio.h>
-#include	<unistd.h>
 #include	<fcntl.h>
 #include	<string.h>
 #include	<ctype.h>
@@ -192,7 +191,6 @@ dwd_read_header (SF_PRIVATE *psf)
 
 	psf->sf.samplerate = dwdh.srate ;
 	psf->sf.channels = dwdh.channels ;
-	psf->sf.seekable = SF_TRUE ;
 	psf->sf.sections = 1 ;
 	
 	return pcm_init (psf) ;
@@ -202,3 +200,10 @@ dwd_read_header (SF_PRIVATE *psf)
 */
 
 #endif
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: a5e1d2a6-a840-4039-a0e7-e1a43eb05a4f
+*/
