@@ -27,7 +27,7 @@
 */
 
 #include	<stdio.h>
-#include	<unistd.h>
+#include	<stdlib.h>
 #include	<string.h>
 
 #include	"sndfile.h"
@@ -37,8 +37,7 @@
 #include	"common.h"
 
 typedef struct
-{	int		filedes ;
-	int		dwm_maxsize, bit_width, max_delta, span ;
+{	int		dwm_maxsize, bit_width, max_delta, span ;
 	int		samplecount ;
 	int		bit_count, bits, last_delta_width, last_sample ;
 	struct
@@ -91,8 +90,6 @@ dwvw_init (SF_PRIVATE *psf, int bitwidth)
 
 	memset (pdwvw, 0, sizeof (DWVW_PRIVATE)) ;
 
-	pdwvw->filedes = psf->filedes ;
-
 	pdwvw->bit_width 	= bitwidth ;
 	pdwvw->dwm_maxsize	= bitwidth / 2 ;
 	pdwvw->max_delta	= 1 << (bitwidth - 1) ;
@@ -115,10 +112,10 @@ dwvw_init (SF_PRIVATE *psf, int bitwidth)
 		} ;
 
 	psf->seek	= dwvw_seek ;
-	psf->close  	= dwvw_close ;
+	psf->close  = dwvw_close ;
 
 	/* FIXME : This s bogus. */
-	SF_COUNT_MAX_POSITIVE (psf->sf.frames) ;
+	psf->sf.frames = SF_COUNT_MAX ;
 	psf->datalength = psf->sf.frames ;
 	/* EMXIF : This s bogus. */
 
@@ -145,26 +142,10 @@ dwvw_close  (SF_PRIVATE *psf)
 		/* Write the last buffer worth of data to disk. */
 		psf_fwrite (pdwvw->b.buffer, 1, pdwvw->b.index, psf) ;
 
-		/*  Now we know for certain the length of the file we can re-write
-		**	correct values for the FORM, COMM and SSND chunks.
-		*/
-
-		psf_fseek (psf, 0, SEEK_END) ;
-		psf->filelength = psf_ftell (psf) ;
-		psf_fseek (psf, 0, SEEK_SET) ;
-
-		psf->datalength = psf->filelength - psf->dataoffset ;
-
- 		psf->sf.frames = pdwvw->samplecount ;
-
 		if (psf->write_header)
-			psf->write_header (psf, SF_FALSE) ;
+			psf->write_header (psf, SF_TRUE) ;
 		} ;
 
-	if (psf->fdata)
-		free (psf->fdata) ;
-	psf->fdata = NULL ;
-
 	return 0 ;
 } /* dwvw_close */
 
@@ -676,3 +657,10 @@ dwvw_write_d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 
 	return total ;
 } /* dwvw_write_d */
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: 1ca09552-b01f-4d7f-9bcf-612f834fe41d
+*/
