@@ -17,8 +17,8 @@
 */
 
 #include	<stdio.h>
+#include	<stdlib.h>
 #include	<string.h>
-#include	<unistd.h>
 
 #include	"sndfile.h"
 #include	"config.h"
@@ -254,7 +254,7 @@ float32_be_read (unsigned char *cptr)
 	float	fvalue ;
 
 	negative = cptr [0] & 0x80 ;
-	exponent = ((cptr [0] & 0x7F) << 1) | ((cptr [1] & 0x80) ? 1 : 0);
+	exponent = ((cptr [0] & 0x7F) << 1) | ((cptr [1] & 0x80) ? 1 : 0) ;
 	mantissa = ((cptr [1] & 0x7F) << 16) | (cptr [2] << 8) | (cptr [3]) ;
 
 	if (! (exponent || mantissa))
@@ -282,7 +282,7 @@ float32_le_read (unsigned char *cptr)
 	float	fvalue ;
 
 	negative = cptr [3] & 0x80 ;
-	exponent = ((cptr [3] & 0x7F) << 1) | ((cptr [2] & 0x80) ? 1 : 0);
+	exponent = ((cptr [3] & 0x7F) << 1) | ((cptr [2] & 0x80) ? 1 : 0) ;
 	mantissa = ((cptr [2] & 0x7F) << 16) | (cptr [1] << 8) | (cptr [0]) ;
 
 	if (! (exponent || mantissa))
@@ -382,7 +382,7 @@ static void
 float32_peak_update (SF_PRIVATE *psf, float *buffer, int count, int indx)
 {	int 	chan ;
 	int		k, position ;
-	float	fmaxval;
+	float	fmaxval ;
 
 	for (chan = 0 ; chan < psf->sf.channels ; chan++)
 	{	fmaxval = fabs (buffer [chan]) ;
@@ -393,9 +393,9 @@ float32_peak_update (SF_PRIVATE *psf, float *buffer, int count, int indx)
 				position = k ;
 				} ;
 
-		if (fmaxval > psf->peak.peak[chan].value)
-		{	psf->peak.peak[chan].value = fmaxval ;
-			psf->peak.peak[chan].position = psf->write_current + indx + (position / psf->sf.channels) ;
+		if (fmaxval > psf->peak.peak [chan].value)
+		{	psf->peak.peak [chan].value = fmaxval ;
+			psf->peak.peak [chan].position = psf->write_current + indx + (position / psf->sf.channels) ;
 			} ;
 		} ;
 
@@ -939,3 +939,10 @@ f2bf_array (float *buffer, int count)
 		} ;
 } /* f2bf_array */
 
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: b6c34917-488c-4145-9648-f4371fc4c889
+*/
