@@ -17,7 +17,6 @@
 */
 
 #include	<stdio.h>
-#include	<unistd.h>
 #include	<fcntl.h>
 #include	<string.h>
 #include	<ctype.h>
@@ -69,7 +68,7 @@ static	int		ircam_read_header	(SF_PRIVATE *psf) ;
 
 static	int		get_encoding (int subformat) ;
 
-static	char*	get_encoding_str (int encoding) ;
+static	const char*	get_encoding_str (int encoding) ;
 
 /*------------------------------------------------------------------------------
 ** Public function.
@@ -220,7 +219,7 @@ ircam_read_header	(SF_PRIVATE *psf)
 	psf->dataoffset = IRCAM_DATA_OFFSET ;
 	psf->datalength = psf->filelength - psf->dataoffset ;
 
-	if (! psf->sf.frames && psf->blockwidth)
+	if (psf->sf.frames == 0 && psf->blockwidth)
 		psf->sf.frames = psf->datalength / psf->blockwidth ;
 
 	psf_log_printf (psf, "  Samples     : %d\n", psf->sf.frames) ;
@@ -244,6 +243,9 @@ ircam_write_header (SF_PRIVATE *psf, int calc_length)
 	float		samplerate ;
 	sf_count_t	current ;
 
+	if (psf->pipeoffset > 0)
+		return 0 ;
+
 	current = psf_ftell (psf) ;
 
 	calc_length = calc_length ;
@@ -251,26 +253,31 @@ ircam_write_header (SF_PRIVATE *psf, int calc_length)
 	/* This also sets psf->endian. */
 	encoding = get_encoding (psf->sf.format & SF_FORMAT_SUBMASK) ;
 	
-	if (! encoding)
+	if (encoding == 0)
 		return SFE_BAD_OPEN_FORMAT ;
 
 	/* Reset the current header length to zero. */
 	psf->header [0] = 0 ;
 	psf->headindex = 0 ;
+
+	if (psf->is_pipe == SF_FALSE)
 	psf_fseek (psf, 0, SEEK_SET) ;
 	
 	samplerate = psf->sf.samplerate ;
 
-	if (psf->endian == SF_ENDIAN_BIG)
-	{	psf_binheader_writef (psf, "Emf", IRCAM_02B_MARKER, samplerate) ;
+	switch (psf->endian)
+	{	case SF_ENDIAN_BIG :
+			psf_binheader_writef (psf, "Emf", IRCAM_02B_MARKER, samplerate) ;
 		psf_binheader_writef (psf, "E44", psf->sf.channels, encoding) ;
-		}
-	else if  (psf->endian == SF_ENDIAN_LITTLE)
-	{	psf_binheader_writef (psf, "emf", IRCAM_03L_MARKER, samplerate) ;
+			break ;
+
+		case SF_ENDIAN_LITTLE :
+			psf_binheader_writef (psf, "emf", IRCAM_03L_MARKER, samplerate) ;
 		psf_binheader_writef (psf, "e44", psf->sf.channels, encoding) ;
-		}
-	else
-		return SFE_BAD_OPEN_FORMAT ;
+			break ;
+
+		default : return SFE_BAD_OPEN_FORMAT ;
+		} ;
 
 	psf_binheader_writef (psf, "z", IRCAM_DATA_OFFSET - psf->headindex) ;
 	
@@ -303,7 +310,7 @@ get_encoding (int subformat)
 	return 0 ;
 } /* get_encoding */
 
-static	char*	
+static const char*
 get_encoding_str (int encoding)
 {	switch (encoding)
 	{	case IRCAM_PCM_16	: return "16 bit PCM" ;
@@ -315,3 +322,10 @@ get_encoding_str (int encoding)
 	return "Unknown encoding" ;
 } /* get_encoding_str */
 
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: f2714ab8-f286-4c94-9740-edaf673a1c71
+*/
