@@ -17,7 +17,6 @@
 */
 
 #include	<stdio.h>
-#include	<unistd.h>
 #include	<fcntl.h>
 #include	<string.h>
 #include	<ctype.h>
@@ -86,7 +85,10 @@ mat4_open	(SF_PRIVATE *psf)
 	subformat = psf->sf.format & SF_FORMAT_SUBMASK ;
 
 	if (psf->mode == SFM_WRITE || psf->mode == SFM_RDWR)
-	{	psf->endian = psf->sf.format & SF_FORMAT_ENDMASK ;
+	{	if (psf->is_pipe)
+			return SFE_NO_PIPE_WRITE ;
+
+		psf->endian = psf->sf.format & SF_FORMAT_ENDMASK ;
 		if (CPU_IS_LITTLE_ENDIAN && (psf->endian == SF_ENDIAN_CPU || psf->endian == 0))
 			psf->endian = SF_ENDIAN_LITTLE ;
 		else if (CPU_IS_BIG_ENDIAN && (psf->endian == SF_ENDIAN_CPU || psf->endian == 0))
@@ -204,7 +206,8 @@ static int
 mat4_read_header (SF_PRIVATE *psf)
 {	int		marker, namesize, rows, cols, imag ;
 	double	value ;
-	char	name [64], *marker_str ;
+	const char *marker_str ;
+	char	name [64] ;
 
 	psf_binheader_readf (psf, "pm", 0, &marker) ;
 
@@ -312,15 +315,12 @@ mat4_read_header (SF_PRIVATE *psf)
 				psf->filelength - psf->dataoffset, psf->sf.channels * psf->sf.frames * psf->bytewidth) ;
 		}
 	else if ((psf->filelength - psf->dataoffset) > psf->sf.channels * psf->sf.frames * psf->bytewidth)
-		psf->dataend = rows * cols * psf->bytewidth ;
+		psf->dataend = psf->dataoffset + rows * cols * psf->bytewidth ;
 
 	psf->datalength = psf->filelength - psf->dataoffset - psf->dataend ;
 
-	psf->sf.seekable = SF_TRUE ;
 	psf->sf.sections = 1 ;
 
-	psf->blockwidth = psf->sf.channels * psf->bytewidth ;
-
 	return 0 ;
 } /* mat4_read_header */
 
@@ -383,3 +383,10 @@ mat4_marker_to_str (int marker)
 	LSF_SNPRINTF (str, sizeof (str) - 1, "%08X", marker) ;
 	return str ;
 } /* mat4_marker_to_str */
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: f7e5f5d6-fc39-452e-bc4a-59627116ff59
+*/
