@@ -17,7 +17,6 @@
 */
 
 #include	<stdio.h>
-#include	<unistd.h>
 #include	<fcntl.h>
 #include	<string.h>
 #include	<ctype.h>
@@ -97,7 +96,10 @@ mat5_open	(SF_PRIVATE *psf)
 	subformat = psf->sf.format & SF_FORMAT_SUBMASK ;
 
 	if (psf->mode == SFM_WRITE || psf->mode == SFM_RDWR)
-	{	psf->endian = psf->sf.format & SF_FORMAT_ENDMASK ;
+	{	if (psf->is_pipe)
+			return SFE_NO_PIPE_WRITE ;
+
+		psf->endian = psf->sf.format & SF_FORMAT_ENDMASK ;
 		if (CPU_IS_LITTLE_ENDIAN && (psf->endian == SF_ENDIAN_CPU || psf->endian == 0))
 			psf->endian = SF_ENDIAN_LITTLE ;
 		else if (CPU_IS_BIG_ENDIAN && (psf->endian == SF_ENDIAN_CPU || psf->endian == 0))
@@ -151,8 +153,8 @@ mat5_close	(SF_PRIVATE  *psf)
 
 static int
 mat5_write_header (SF_PRIVATE *psf, int calc_length)
-{	static char	*sr_name = "samplerate\0\0\0\0\0\0\0\0\0\0\0" ;
-	static char	*wd_name = "wavedata\0" ;
+{	static const char	*sr_name = "samplerate\0\0\0\0\0\0\0\0\0\0\0" ;
+	static const char	*wd_name = "wavedata\0" ;
 	sf_count_t	current, datasize ;
 	int			encoding ;
 
@@ -495,3 +497,10 @@ mat5_read_header (SF_PRIVATE *psf)
 	return 0 ;
 } /* mat5_read_header */
 
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: dfdb6742-b2be-4be8-b390-d0c674e8bc8e
+*/
