@@ -17,8 +17,8 @@
 */
 
 #include	<stdio.h>
+#include	<stdlib.h>
 #include	<string.h>
-#include	<unistd.h>
 
 #include	"sndfile.h"
 #include	"config.h"
@@ -272,7 +272,7 @@ msadpcm_decode_block (SF_PRIVATE *psf, MSADPCM_PRIVATE *pms)
 
 		bytecode = pms->samples [k] & 0xF ;
 
-	    /** Compute next Adaptive Scale Factor (ASF) **/
+		/* Compute next Adaptive Scale Factor (ASF) */
 	    idelta = chan_idelta [chan] ;
 	    chan_idelta [chan] = (AdaptationTable [bytecode] * idelta) >> 8 ; /* => / 256 => FIXED_POINT_ADAPTATION_BASE == 256 */
 	    if (chan_idelta [chan] < 16) 
@@ -282,7 +282,7 @@ msadpcm_decode_block (SF_PRIVATE *psf, MSADPCM_PRIVATE *pms)
 		
     	predict = ((pms->samples [k - pms->channels] * AdaptCoeff1 [bpred [chan]]) 
 					+ (pms->samples [k - 2 * pms->channels] * AdaptCoeff2 [bpred [chan]])) >> 8 ; /* => / 256 => FIXED_POINT_COEFF_BASE == 256 */
-    	current = (bytecode * idelta) + predict;
+		current = (bytecode * idelta) + predict ;
     
 	    if (current > 32767) 
 			current = 32767 ;
@@ -301,7 +301,7 @@ msadpcm_read_block (SF_PRIVATE *psf, MSADPCM_PRIVATE *pms, short *ptr, int len)
 	
 	while (indx < len)
 	{	if (pms->blockcount >= pms->blocks && pms->samplecount >= pms->samplesperblock)
-		{	memset (&(ptr[indx]), 0, (size_t) ((len - indx) * sizeof (short))) ;
+		{	memset (&(ptr [indx]), 0, (size_t) ((len - indx) * sizeof (short))) ;
 			return total ;
 			} ;
 		
@@ -311,7 +311,7 @@ msadpcm_read_block (SF_PRIVATE *psf, MSADPCM_PRIVATE *pms, short *ptr, int len)
 		count = (pms->samplesperblock - pms->samplecount) * pms->channels ;
 		count = (len - indx > count) ? count : len - indx ;
 		
-		memcpy (&(ptr[indx]), &(pms->samples [pms->samplecount * pms->channels]), count * sizeof (short)) ;
+		memcpy (&(ptr [indx]), &(pms->samples [pms->samplecount * pms->channels]), count * sizeof (short)) ;
 		indx += count ;
 		pms->samplecount += count / pms->channels ;
 		total = indx ;
@@ -791,13 +791,13 @@ static	void
 choose_predictor (unsigned int channels, short *data, int *block_pred, int *idelta)
 {	unsigned int	chan, k, bpred, idelta_sum, best_bpred, best_idelta ;
 	
-	for (chan = 0 ; chan < channels; chan++)
+	for (chan = 0 ; chan < channels ; chan++)
 	{	best_bpred = best_idelta = 0 ;
 
 		for (bpred = 0 ; bpred < 7 ; bpred++)
 		{	idelta_sum = 0 ;
 			for (k = 2 ; k < 2 + IDELTA_COUNT ; k++)
-				idelta_sum += abs (data [k*channels] - ((data [(k-1)*channels] * AdaptCoeff1 [bpred] + data [(k-2)*channels] * AdaptCoeff2 [bpred]) >> 8)) ;
+				idelta_sum += abs (data [k * channels] - ((data [(k - 1) * channels] * AdaptCoeff1 [bpred] + data [(k - 2) * channels] * AdaptCoeff2 [bpred]) >> 8)) ;
 			idelta_sum /= (4 * IDELTA_COUNT) ;
 			
 			if (bpred == 0 || idelta_sum < best_idelta)
@@ -821,3 +821,10 @@ choose_predictor (unsigned int channels, short *data, int *block_pred, int *idel
 
 	return ;
 } /* choose_predictor */
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: a98908a3-5305-4935-872b-77d6a86c330f
+*/
