@@ -24,7 +24,6 @@
 */
 
 #include	<stdio.h>
-#include	<unistd.h>
 #include	<fcntl.h>
 #include	<string.h>
 #include	<ctype.h>
@@ -62,7 +61,10 @@ nist_open	(SF_PRIVATE *psf)
 	subformat = psf->sf.format & SF_FORMAT_SUBMASK ;
 	
 	if (psf->mode == SFM_WRITE || psf->mode == SFM_RDWR)
-	{	if ((psf->sf.format & SF_FORMAT_TYPEMASK) != SF_FORMAT_NIST)
+	{	if (psf->is_pipe)
+			return SFE_NO_PIPE_WRITE ;
+
+		if ((psf->sf.format & SF_FORMAT_TYPEMASK) != SF_FORMAT_NIST)
 			return	SFE_BAD_OPEN_FORMAT ;
 	
 		psf->endian = psf->sf.format & SF_FORMAT_ENDMASK ;
@@ -121,7 +123,6 @@ nist_read_header (SF_PRIVATE *psf)
 	char 	str [64], *cptr ;
 	long	samples ;
 	
-		
 	psf->sf.format = SF_FORMAT_NIST ;
 	
 	psf_header = (char*) psf->header ;
@@ -141,8 +142,6 @@ nist_read_header (SF_PRIVATE *psf)
 		cptr [0] = 0 ;
 		} ;
 
-	psf_log_printf (psf, "%s", psf_header) ;
-	
 	if (strstr (psf_header, bad_header) == psf_header)
 		return SFE_NIST_CRLF_CONVERISON ;
 
@@ -259,20 +258,7 @@ static int
 nist_close	(SF_PRIVATE  *psf)
 {
 	if (psf->mode == SFM_WRITE || psf->mode == SFM_RDWR)
-	{	/*  Now we know for certain the length of the file we can
-		**  re-write correct values for the datasize header element.
-		*/
-
-		psf_fseek (psf, 0, SEEK_END) ;
-		psf->filelength = psf_ftell (psf) ;
-
-		psf->dataoffset = NIST_HEADER_LENGTH ;
-		psf->datalength = psf->filelength - psf->dataoffset ;
-
-		psf->sf.frames = psf->datalength / psf->blockwidth ;
-
-		nist_write_header (psf, SF_FALSE) ;
-		} ;
+		nist_write_header (psf, SF_TRUE) ;
 
 	return 0 ;
 } /* nist_close */
@@ -282,7 +268,7 @@ nist_close	(SF_PRIVATE  *psf)
 
 static int
 nist_write_header (SF_PRIVATE  *psf, int calc_length)
-{	char		*end_str ;
+{	const char	*end_str ;
 	long		samples ;
 	sf_count_t	current ;
 
@@ -359,3 +345,10 @@ nist_write_header (SF_PRIVATE  *psf, int calc_length)
 } /* nist_write_header */
 
 
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: b45ed85d-9e22-4ad9-b78c-4b58b67152a8
+*/
