@@ -16,14 +16,15 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
+#include "config.h"
+
 #include	<stdio.h>
-#include	<unistd.h>
+#include <stdlib.h>
 #include	<fcntl.h>
 #include	<string.h>
 #include	<ctype.h>
 
 #include	"sndfile.h"
-#include	"config.h"
 #include	"sfendian.h"
 #include	"float_cast.h"
 #include	"common.h"
@@ -300,7 +301,6 @@ paf_write_header (SF_PRIVATE  *psf, int calc_length)
 	/* Reset the current header length to zero. */
 	psf->header [0] = 0 ;
 	psf->headindex = 0 ;
-	psf_fseek (psf, 0, SEEK_SET) ;
 
 	if (psf->endian == SF_ENDIAN_BIG)
 	{	/* Marker, version, endianness, samplerate */
@@ -350,6 +350,12 @@ paf24_init (SF_PRIVATE  *psf)
 	paf24size = sizeof (PAF24_PRIVATE) + psf->sf.channels *
 					(PAF24_BLOCK_SIZE + PAF24_SAMPLES_PER_BLOCK * sizeof (int)) ;
 
+	/*
+	**	Not exatly sure why this needs to be here but the tests
+	**	fail without it.
+	*/
+	psf->last_op = 0 ;
+
 	if (! (psf->fdata = malloc (paf24size)))
 		return SFE_MALLOC_FAILED ;
 
@@ -410,9 +416,9 @@ paf24_seek (SF_PRIVATE *psf, int mode, sf_count_t offset)
 {	PAF24_PRIVATE	*ppaf24 ;
 	int				newblock, newsample ;
 
-	if (! psf->fdata)
+	if (psf->fdata == NULL)
 	{	psf->error = SFE_INTERNAL ;
-		return	((sf_count_t) -1) ;
+		return SF_SEEK_ERROR ;
 		} ;
 
 	ppaf24 = (PAF24_PRIVATE*) psf->fdata ;
@@ -427,7 +433,7 @@ paf24_seek (SF_PRIVATE *psf, int mode, sf_count_t offset)
 	{	case SFM_READ :
 				if (offset > ppaf24->read_block * ppaf24->samplesperblock + ppaf24->read_count)
 				{	psf->error = SFE_BAD_SEEK ;
-					return	((sf_count_t) -1) ;
+					return SF_SEEK_ERROR ;
 					} ;
 
 				if (psf->last_op == SFM_WRITE && ppaf24->write_count)
@@ -442,21 +448,21 @@ paf24_seek (SF_PRIVATE *psf, int mode, sf_count_t offset)
 		case SFM_WRITE :
 				if (offset > ppaf24->sample_count)
 				{	psf->error = SFE_BAD_SEEK ;
-					return	((sf_count_t) -1) ;
+					return SF_SEEK_ERROR ;
 					} ;
 
 				if (psf->last_op == SFM_WRITE && ppaf24->write_count)
 					paf24_write_block (psf, ppaf24) ;
 
 				psf_fseek (psf, psf->dataoffset + newblock * ppaf24->blocksize, SEEK_SET) ;
-				ppaf24->read_block  = newblock ;
+				ppaf24->write_block = newblock ;
 				paf24_read_block (psf, ppaf24) ;
-				ppaf24->read_count = newsample ;
+				ppaf24->write_count = newsample ;
 				break ;
 
 		default :
 				psf->error = SFE_BAD_SEEK ;
-				return	((sf_count_t) -1) ;
+				return SF_SEEK_ERROR ;
 		} ;
 
 	return newblock * ppaf24->samplesperblock + newsample ;
@@ -466,7 +472,7 @@ static int
 paf24_close (SF_PRIVATE  *psf)
 {	PAF24_PRIVATE *ppaf24 ;
 
-	if (! psf->fdata)
+	if (psf->fdata == NULL)
 		return 0 ;
 
 	ppaf24 = (PAF24_PRIVATE*) psf->fdata ;
@@ -533,7 +539,7 @@ paf24_read (SF_PRIVATE *psf, PAF24_PRIVATE *ppaf24, int *ptr, int len)
 
 	while (total < len)
 	{	if (ppaf24->read_block * ppaf24->samplesperblock >= ppaf24->sample_count)
-		{	memset (&(ptr[total]), 0, (len - total) * sizeof (int)) ;
+		{	memset (&(ptr [total]), 0, (len - total) * sizeof (int)) ;
 			return total ;
 			} ;
 
@@ -543,7 +549,7 @@ paf24_read (SF_PRIVATE *psf, PAF24_PRIVATE *ppaf24, int *ptr, int len)
 		count = (ppaf24->samplesperblock - ppaf24->read_count) * ppaf24->channels ;
 		count = (len - total > count) ? count : len - total ;
 
-		memcpy (&(ptr[total]), &(ppaf24->samples [ppaf24->read_count * ppaf24->channels]), count * sizeof (int)) ;
+		memcpy (&(ptr [total]), &(ppaf24->samples [ppaf24->read_count * ppaf24->channels]), count * sizeof (int)) ;
 		total += count ;
 		ppaf24->read_count += count / ppaf24->channels ;
 		} ;
@@ -558,7 +564,7 @@ paf24_read_s (SF_PRIVATE *psf, short *ptr, sf_count_t len)
 	int				k, bufferlen, readcount, count ;
 	sf_count_t		total = 0 ;
 
-	if (! psf->fdata)
+	if (psf->fdata == NULL)
 		return 0 ;
 	ppaf24 = (PAF24_PRIVATE*) psf->fdata ;
 
@@ -580,7 +586,7 @@ paf24_read_i (SF_PRIVATE *psf, int *ptr, sf_count_t len)
 {	PAF24_PRIVATE *ppaf24 ;
 	int				total ;
 
-	if (! psf->fdata)
+	if (psf->fdata == NULL)
 		return 0 ;
 	ppaf24 = (PAF24_PRIVATE*) psf->fdata ;
 
@@ -597,7 +603,7 @@ paf24_read_f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
 	sf_count_t		total = 0 ;
 	float			normfact ;
 
-	if (! psf->fdata)
+	if (psf->fdata == NULL)
 		return 0 ;
 	ppaf24 = (PAF24_PRIVATE*) psf->fdata ;
 
@@ -624,7 +630,7 @@ paf24_read_d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 	sf_count_t		total = 0 ;
 	double 			normfact ;
 
-	if (! psf->fdata)
+	if (psf->fdata == NULL)
 		return 0 ;
 	ppaf24 = (PAF24_PRIVATE*) psf->fdata ;
 
@@ -724,7 +730,7 @@ paf24_write_s (SF_PRIVATE *psf, short *ptr, sf_count_t len)
 	int				k, bufferlen, writecount = 0, count ;
 	sf_count_t		total = 0 ;
 
-	if (! psf->fdata)
+	if (psf->fdata == NULL)
 		return 0 ;
 	ppaf24 = (PAF24_PRIVATE*) psf->fdata ;
 
@@ -749,7 +755,7 @@ paf24_write_i (SF_PRIVATE *psf, int *ptr, sf_count_t len)
 	int				writecount, count ;
 	sf_count_t		total = 0 ;
 
-	if (! psf->fdata)
+	if (psf->fdata == NULL)
 		return 0 ;
 	ppaf24 = (PAF24_PRIVATE*) psf->fdata ;
 
@@ -775,7 +781,7 @@ paf24_write_f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
 	sf_count_t		total = 0 ;
 	float			normfact ;
 
-	if (! psf->fdata)
+	if (psf->fdata == NULL)
 		return 0 ;
 	ppaf24 = (PAF24_PRIVATE*) psf->fdata ;
 
@@ -805,7 +811,7 @@ paf24_write_d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 	sf_count_t		total = 0 ;
 	double			normfact ;
 
-	if (! psf->fdata)
+	if (psf->fdata == NULL)
 		return 0 ;
 	ppaf24 = (PAF24_PRIVATE*) psf->fdata ;
 
@@ -828,3 +834,10 @@ paf24_write_d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 } /* paf24_write_d */
 
 
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: 477a5308-451e-4bbd-bab4-fab6caa4e884
+*/
