@@ -16,20 +16,21 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
+#include	"config.h"
+
+#include	<stdlib.h>
 #include	<string.h>
 #include	<ctype.h>
 
 #include	"sndfile.h"
-#include	"config.h"
 #include	"sfendian.h"
 #include	"common.h"
 
-
 #define		SNDFILE_MAGICK	0x1234C0DE
 
 typedef struct
 {	int 	error ;
-	char	*str ;
+	const char	*str ;
 } ErrorStruct ;
 
 static
@@ -49,6 +50,8 @@ ErrorStruct SndfileErrors [] =
 	{	SFE_BAD_SF_INCOMPLETE	, "SF_PRIVATE struct incomplete and end of header parsing." },
 	{	SFE_BAD_FILE_PTR		, "Bad FILE pointer." },
 	{	SFE_BAD_INT_PTR			, "Internal error, Bad pointer." },
+	{	SFE_BAD_STAT_SIZE		, "Error : software was misconfigured at compile time (sizeof statbuf.st_size)." },
+
 	{	SFE_MALLOC_FAILED		, "Internal malloc () failed." },
 	{	SFE_UNIMPLEMENTED		, "File contains data in an unimplemented format." },
 	{	SFE_BAD_READ_ALIGN  	, "Attempt to read a non-integer number of channels." },
@@ -58,6 +61,10 @@ ErrorStruct SndfileErrors [] =
 	{	SFE_NOT_WRITEMODE		, "Write attempted on file currently open for read." },
 	{	SFE_BAD_MODE_RW			, "This file format does not support read/write mode." },
 	{	SFE_BAD_SF_INFO			, "Internal error : SF_INFO struct incomplete." },
+	{	SFE_BAD_OFFSET			, "Error : supplied offset beyond end of file." },
+	{	SFE_NO_EMBED_SUPPORT	, "Error : embedding not supported for this file format." },
+	{	SFE_NO_EMBEDDED_RDWR	, "Error : cannot open embedded file read/write." },
+	{	SFE_NO_PIPE_WRITE		, "Error : this file format does not support pipe write." },
 	{	SFE_BAD_RDWR_FORMAT		, "Attempted to open read only format for RDWR." },
 
 	{	SFE_INTERLEAVE_MODE		, "Attempt to write to file with non-interleaved data." },
@@ -85,6 +92,7 @@ ErrorStruct SndfileErrors [] =
 	{	SFE_STR_MAX_COUNT		, "Error : Maximum string data count reached." },	
 	{	SFE_STR_BAD_TYPE		, "Error : Bad string data type." },	
 	{	SFE_STR_NO_ADD_END		, "Error : file type does not support strings added at end of file." },	
+	{	SFE_STR_BAD_STRING		, "Error : bad string." },
 	{	SFE_STR_WEIRD			, "Error : Weird string error." },	
 
 	{	SFE_WAV_NO_RIFF			, "Error in WAV file. No 'RIFF' chunk marker." },
@@ -118,9 +126,11 @@ ErrorStruct SndfileErrors [] =
 	{	SFE_AIFF_BAD_PEAK		, "Error in AIFF file. Bad 'PEAK' chunk." },
 	{	SFE_AIFF_NO_SSND		, "Error in AIFF file, bad 'SSND' chunk." },
 	{	SFE_AIFF_NO_DATA		, "Error in AIFF file, no sound data." },
+	{	SFE_AIFF_RW_SSND_NOT_LAST, "Error in AIFF file, RDWR only possible if SSND chunk at end of file." },
 
 	{	SFE_AU_UNKNOWN_FORMAT	, "Error in AU file, unknown format." },
 	{	SFE_AU_NO_DOTSND		, "Error in AU file, missing '.snd' or 'dns.' marker." },
+	{	SFE_AU_EMBED_BAD_LEN	, "Embedded AU file with unknown length." },
 
 	{	SFE_RAW_READ_BAD_SPEC	, "Error while opening RAW file for read. Must specify format and channels.\n"
 									"Possibly trying to open unsupported format."
@@ -137,6 +147,7 @@ ErrorStruct SndfileErrors [] =
 	{	SFE_SVX_NO_BODY			, "Error in 8SVX / 16SV file, no 'BODY' marker." },
 	{	SFE_SVX_NO_DATA			, "Error in 8SVX / 16SV file, no sound data." },
 	{	SFE_SVX_BAD_COMP		, "Error in 8SVX / 16SV file, unsupported compression format." },
+	{	SFE_SVX_BAD_NAME_LENGTH	, "Error in 8SVX / 16SV file, NAME chunk too long." },
 
 	{	SFE_NIST_BAD_HEADER		, "Error in NIST file, bad header." },
 	{	SFE_NIST_CRLF_CONVERISON, "Error : NIST file damaged by Windows CR -> CRLF conversion process."	},
@@ -151,6 +162,7 @@ ErrorStruct SndfileErrors [] =
 	{	SFE_VOC_MULTI_SECTION	, "Unimplemented VOC file feature, file contains multiple sound sections." },
 	{	SFE_VOC_MULTI_PARAM		, "Error in VOC file, file contains multiple bit or channel widths." },
 	{	SFE_VOC_SECTION_COUNT	, "Error in VOC file, too many sections." },
+	{	SFE_VOC_NO_PIPE			, "Error : not able to operate on VOC files over a pipe." },
 
 	{	SFE_IRCAM_NO_MARKER		, "Error in IRCAM file, bad IRCAM marker." },
 	{	SFE_IRCAM_BAD_CHANNELS	, "Error in IRCAM file, bad channel count." },
@@ -185,7 +197,12 @@ ErrorStruct SndfileErrors [] =
 
 	{	SFE_XI_BAD_HEADER		, "Error in XI file. Bad header." },
 	{	SFE_XI_EXCESS_SAMPLES	, "Error in XI file. Excess samples in file." },
-	{	SFE_XI_WHOOPS			, "Error. Whoops." },
+	{	SFE_XI_NO_PIPE			, "Error : not able to operate on XI files over a pipe." },
+
+	{	SFE_HTK_NO_PIPE			, "Error : not able to operate on HTK files over a pipe." },
+
+	{	SFE_SDS_NOT_SDS			, "Error : not an SDS file." },
+	{	SFE_SDS_BAD_BIT_WIDTH	, "Error : bad bit width for SDS file." },
 	
 	{	SFE_DWVW_BAD_BITWIDTH	, "Error : Bad bit width for DWVW encoding. Must be 12, 16 or 24." },
 	{	SFE_G72X_NOT_MONO		, "Error : G72x encoding does not support more than 1 channel." },
@@ -203,6 +220,8 @@ static int	validate_sfinfo (SF_INFO *sfinfo) ;
 static int	validate_psf (SF_PRIVATE *psf) ;
 static void	save_header_info (SF_PRIVATE *psf) ;
 static void	copy_filename (SF_PRIVATE *psf, const char *path) ;
+static int	psf_close (SF_PRIVATE  *psf) ;
+static int	psf_open_file (SF_PRIVATE *psf, int mode, SF_INFO *sfinfo) ;
 
 /*------------------------------------------------------------------------------
 ** Private (static) variables.
@@ -217,12 +236,18 @@ static char	sf_syserr [SF_SYSERR_LEN] = { 0 } ;
 
 #define	VALIDATE_SNDFILE_AND_ASSIGN_PSF(a,b,c)		\
 		{	if (! (a))								\
-				return SFE_BAD_SNDFILE_PTR ;		\
+			{	sf_errno = SFE_BAD_SNDFILE_PTR ;	\
+				return 0 ;							\
+				} ;									\
 			(b) = (SF_PRIVATE*) (a) ;				\
-			if (((b)->filedes) < 0)					\
-				return SFE_BAD_FILE_PTR ;			\
+			if (psf_filedes_valid (b) == 0)			\
+			{	(b)->error = SFE_BAD_FILE_PTR ;		\
+				return 0 ;							\
+				} ;									\
 			if ((b)->Magick != SNDFILE_MAGICK)		\
-				return	SFE_BAD_SNDFILE_PTR ;		\
+			{	(b)->error = SFE_BAD_SNDFILE_PTR ;	\
+				return 0 ;							\
+				} ;									\
 			if (c) (b)->error = 0 ;					\
 			}
 
@@ -233,307 +258,84 @@ static char	sf_syserr [SF_SYSERR_LEN] = { 0 } ;
 SNDFILE*
 sf_open	(const char *path, int mode, SF_INFO *sfinfo)
 {	SF_PRIVATE 	*psf ;
-	int			error ;
+	int			error = 0 ;
 
-	if (mode != SFM_READ && mode != SFM_WRITE && mode != SFM_RDWR)
-	{	sf_errno = SFE_BAD_OPEN_MODE ;
-		return	NULL ;
-		} ;
-
-	if (! path)
-	{	sf_errno = SFE_BAD_FILE ;
-		return	NULL ;
-		} ;
-
-	if (! sfinfo)
-	{	sf_errno = SFE_BAD_SF_INFO_PTR ;
-		return	NULL ;
-		} ;
-
-	if (mode == SFM_READ)
-	{	if ((sfinfo->format & SF_FORMAT_TYPEMASK) == SF_FORMAT_RAW)
-		{	if (sf_format_check (sfinfo) == 0)
-			{	sf_errno = SFE_RAW_BAD_FORMAT ;
-				return NULL ;
-				} ;
-			}
-		else
-			memset (sfinfo, 0, sizeof (SF_INFO)) ;
-		} ;
-
-	sf_errno = error = 0 ;
-	sf_logbuffer [0] = 0 ;
-
-	if ((psf = malloc (sizeof (SF_PRIVATE))) == NULL)
+	if ((psf = calloc (1, sizeof (SF_PRIVATE))) == NULL)
 	{	sf_errno = SFE_MALLOC_FAILED ;
 		return	NULL ;
 		} ;
 
 	memset (psf, 0, sizeof (SF_PRIVATE)) ;
-	memcpy (&(psf->sf), sfinfo, sizeof (SF_INFO)) ;
 
 	psf_log_printf (psf, "File : %s\n", path) ;
+
 	copy_filename (psf, path) ;
 
-	psf->Magick 		= SNDFILE_MAGICK ;
-	psf->norm_float 	= SF_TRUE ;
-	psf->norm_double	= SF_TRUE ;
-	psf->mode 			= mode ;
-	psf->filedes		= -1 ;
-	psf->filelength		= -1 ;
-	psf->dataoffset		= -1 ;
-	psf->datalength		= -1 ;
-	psf->read_current	= -1 ;
-	psf->write_current	= -1 ;
-	psf->auto_header 	= SF_FALSE ;
-	psf->read_dither 	= SF_FALSE ;
-	psf->write_dither 	= SF_FALSE ;
-	psf->rwf_endian		= SF_ENDIAN_LITTLE ;
-	psf->seek		= psf_default_seek ;
-
 	if (strcmp (path, "-") == 0)
-	{	/* File is a either stdin or stdout. */
-		psf->sf.seekable = SF_FALSE ;
-		switch (mode)
-		{	case SFM_RDWR :
-					error = SFE_OPEN_PIPE_RDWR ;
-					break ;
-
-			case SFM_READ :
-					psf->filedes = 0 ;
-					break ;
-
-			case SFM_WRITE :
-					psf->filedes = 1 ;
-					break ;
-
-			default :
-					error = SFE_BAD_OPEN_MODE ;
-					break ;
-			} ;
-		psf->filelength = 0 ;
-		}
-	else if ((psf->filedes = psf_open (psf, path, mode)) < 0)
-		error = SFE_OPEN_FAILED ;
-
-	if (error)
-	{	if (psf->filedes >= 0)
-			psf_fclose (psf) ;
-		sf_close (psf) ;
-		sf_errno = error ;
-		return NULL ;
-		} ;
-
-	/* File is open, so get the length. */
-	psf->filelength = psf_get_filelen (psf) ;
-
-	psf->sf.seekable = SF_TRUE ;
-	psf->sf.sections = 1 ;
-
-	if (mode == SFM_WRITE || (mode == SFM_RDWR && psf->filelength == 0))
-	{	/* If the file is being opened for write or RDWR and the file is currently
-		** empty, then the SF_INFO struct must contain valid data.
-		*/
-		if (sf_format_check (&(psf->sf)) == 0)
-			error = SFE_BAD_OPEN_FORMAT ;
-		}
-	else if ((psf->sf.format & SF_FORMAT_TYPEMASK) != SF_FORMAT_RAW)
-	{	psf_log_printf (psf, "Length : %D\n", psf->filelength) ;
-
-		/* If type RAW has not been specified then need to figure out file type. */
-		psf->sf.format = guess_file_type (psf, psf->filename) ;
-		} ;
+		error = psf_set_stdio (psf, mode) ;
+	else
+		error = psf_fopen (psf, path, mode) ;
 
+	if (error == 0)
+		error = psf_open_file (psf, mode, sfinfo) ;
+	
 	if (error)
 	{	sf_errno = error ;
 		if (error == SFE_SYSTEM)
 			LSF_SNPRINTF (sf_syserr, sizeof (sf_syserr), "%s", psf->syserr) ;
-		if (psf->filedes >= 0)
-			psf_fclose (psf) ;
-		sf_close (psf) ;
+		LSF_SNPRINTF (sf_logbuffer, sizeof (sf_logbuffer), "%s", psf->logbuffer) ;
+		psf_close (psf) ;
 		return NULL ;
 		} ;
 
-	/* Set bytewidth if known. */
-	switch (psf->sf.format & SF_FORMAT_SUBMASK)
-	{	case SF_FORMAT_PCM_S8 :
-		case SF_FORMAT_PCM_U8 :
-		case SF_FORMAT_ULAW :
-		case SF_FORMAT_ALAW :
-		case SF_FORMAT_DPCM_8 :
-				psf->bytewidth = 1 ;
-				break ;
-
-		case SF_FORMAT_PCM_16 :
-		case SF_FORMAT_DPCM_16 :
-				psf->bytewidth = 2 ;
-				break ;
-
-		case SF_FORMAT_PCM_24 :
-				psf->bytewidth = 3 ;
-				break ;
-
-		case SF_FORMAT_PCM_32 :
-		case SF_FORMAT_FLOAT :
-				psf->bytewidth = 4 ;
-				break ;
-
-		case SF_FORMAT_DOUBLE :
-				psf->bytewidth = 8 ;
-				break ;
-		} ;
-
-	/* Call the initialisation function for the relevant file type. */
-	switch (psf->sf.format & SF_FORMAT_TYPEMASK)
-	{	case	SF_FORMAT_WAV :
-				error = wav_open (psf) ;
-				break ;
-
-		case	SF_FORMAT_AIFF :
-				error = aiff_open (psf) ;
-				break ;
-
-		case	SF_FORMAT_AU :
-				error = au_open (psf) ;
-				break ;
-
-		case	SF_FORMAT_AU | SF_FORMAT_ULAW :
-				error = au_nh_open (psf) ;
-				break ;
-
-		case	SF_FORMAT_RAW :
-				error = raw_open (psf) ;
-				break ;
-
-		case	SF_FORMAT_PAF :
-				error = paf_open (psf) ;
-				break ;
-
-		case	SF_FORMAT_SVX :
-				error = svx_open (psf) ;
-				break ;
-
-		case	SF_FORMAT_NIST :
-				error = nist_open (psf) ;
-				break ;
-
-		case	SF_FORMAT_IRCAM :
-				error = ircam_open (psf) ;
-				break ;
-
-		case	SF_FORMAT_VOC :
-				error = voc_open (psf) ;
-				break ;
-
-		case	SF_FORMAT_W64 :
-				error = w64_open (psf) ;
-				break ;
-
-		case	SF_FORMAT_OGG :
-				error = ogg_open (psf) ;
-				break ;
-
-		case	SF_FORMAT_TXW :
-				error = txw_open (psf) ;
-				break ;
-
-		case	SF_FORMAT_WVE :
-				error = wve_open (psf) ;
-				break ;
-
-		case	SF_FORMAT_DWD :
-				error = dwd_open (psf) ;
-				break ;
-
-		case	SF_FORMAT_MAT4 :
-				error = mat4_open (psf) ;
-				break ;
-
-		case	SF_FORMAT_MAT5 :
-				error = mat5_open (psf) ;
-				break ;
-
-		case	SF_FORMAT_PVF :
-				error = pvf_open (psf) ;
-				break ;
-
-		case	SF_FORMAT_XI :
-				error = xi_open (psf) ;
-				break ;
-
-		case	SF_FORMAT_SD2 :
-				error = sd2_open (psf) ;
-				break ;
-
-		case	SF_FORMAT_REX2 :
-				error = rx2_open (psf) ;
-				break ;
-
-		default :
-				error = SFE_UNKNOWN_FORMAT ;
-		} ;
-
-	if (mode == SFM_RDWR && sf_format_check (&(psf->sf)) == 0)
-		error = SFE_BAD_RDWR_FORMAT ;
-
-	if (error)
-	{	sf_errno = error ;
-		if (error == SFE_SYSTEM)
-			LSF_SNPRINTF (sf_syserr, sizeof (sf_syserr), "%s", psf->syserr) ;
-		save_header_info (psf) ;
-		sf_close (psf) ;
-		return NULL ;
-		} ;
-
-	if (validate_sfinfo (&(psf->sf)) == 0)
-	{	psf_log_SF_INFO (psf) ;
-		save_header_info (psf) ;
-		sf_errno = SFE_BAD_SF_INFO ;
-		sf_close (psf) ;
-		return NULL ;
-		} ;
-
-	if (validate_psf (psf) == 0)
-	{	save_header_info (psf) ;
-		sf_errno = SFE_INTERNAL ;
-		sf_close (psf) ;
-		return NULL ;
-		} ;
-
-	psf->read_current  = 0 ;
-	psf->write_current = (psf->mode == SFM_RDWR) ? psf->sf.frames : 0 ;
-
 	memcpy (sfinfo, &(psf->sf), sizeof (SF_INFO)) ;
 
 	return (SNDFILE*) psf ;
 } /* sf_open */
 
+SNDFILE*
+sf_open_fd	(int fd, int mode, SF_INFO *sfinfo, int close_desc)
+{	SF_PRIVATE 	*psf ;
+	int			error ;
+	
+	if ((psf = calloc (1, sizeof (SF_PRIVATE))) == NULL)
+	{	sf_errno = SFE_MALLOC_FAILED ;
+		return	NULL ;
+		} ;
+
+	psf_set_file (psf, fd) ;
+	psf->is_pipe = psf_is_pipe (psf) ;
+	psf->fileoffset = psf_ftell (psf) ;
+	
+	if (! close_desc)
+		psf->do_not_close_descriptor = SF_TRUE ;
+
+	error = psf_open_file (psf, mode, sfinfo) ;
+	
+	if (error)
+	{	sf_errno = error ;
+		if (error == SFE_SYSTEM)
+			LSF_SNPRINTF (sf_syserr, sizeof (sf_syserr), "%s", psf->syserr) ;
+		LSF_SNPRINTF (sf_logbuffer, sizeof (sf_logbuffer), "%s", psf->logbuffer) ;
+		psf_close (psf) ;
+		return NULL ;
+		} ;
+
+	memcpy (sfinfo, &(psf->sf), sizeof (SF_INFO)) ;
+
+	return (SNDFILE*) psf ;
+} /* sf_open_fd */
+
 /*------------------------------------------------------------------------------
 */
 
 int
 sf_close	(SNDFILE *sndfile)
 {	SF_PRIVATE  *psf ;
-	int			error ;
 
 	VALIDATE_SNDFILE_AND_ASSIGN_PSF (sndfile, psf, 1) ;
 
-	if (psf->close)
-		error = psf->close (psf) ;
-
-	psf_fclose (psf) ;
-
-	if (psf->fdata)
-		free (psf->fdata) ;
-
-	if (psf->interleave)
-		free (psf->interleave) ;
-
-	memset (psf, 0, sizeof (SF_PRIVATE)) ;
-
-	free (psf) ;
-
-	return 0 ;
+	return psf_close (psf) ;
 } /* sf_close */
 
 /*==============================================================================
@@ -541,16 +343,24 @@ sf_close	(SNDFILE *sndfile)
 
 const char*
 sf_error_number	(int errnum)
-{	int	k ;
+{	static const char *bad_errnum =
+		"No error defined for this error number. This is a bug in libsndfile." ;
+	int	k ;
 
-	if (errnum < 0 || errnum >= SFE_MAX_ERROR)
+	if (errnum == SFE_MAX_ERROR)
 		return SndfileErrors [0].str ;
 
-	for (k = 0 ; SndfileErrors[k].str ; k++)
-		if (errnum == SndfileErrors[k].error)
-			return SndfileErrors[k].str ;
+	if (errnum < 0 || errnum > SFE_MAX_ERROR)
+	{	/* This really shouldn't happen in release versions. */
+		printf ("Not a valid error number (%d).\n", errnum) ;
+		return bad_errnum ;
+		} ;
 
-	return "No error defined for this error number. This is a bug in libsndfile." ;
+	for (k = 0 ; SndfileErrors [k].str ; k++)
+		if (errnum == SndfileErrors [k].error)
+			return SndfileErrors [k].str ;
+
+	return bad_errnum ;
 } /* sf_error_number */
 
 const char*
@@ -651,7 +461,7 @@ sf_format_check	(const SF_INFO *info)
 {	int	subformat, endian ;
 
 	subformat = info->format & SF_FORMAT_SUBMASK ;
-	endian = info->format & SF_FORMAT_SUBMASK ;
+	endian = info->format & SF_FORMAT_ENDMASK ;
 
 	/* This is the place where each file format can check if the suppiled
 	** SF_INFO struct is valid.
@@ -701,6 +511,8 @@ sf_format_check	(const SF_INFO *info)
 					return 1 ;
 				if (subformat == SF_FORMAT_GSM610 && info->channels == 1)
 					return 1 ;
+				if (subformat == SF_FORMAT_IMA_ADPCM && (info->channels == 1 || info->channels == 2))
+					return 1 ;
 				break ;
 
 		case SF_FORMAT_AU :
@@ -829,6 +641,26 @@ sf_format_check	(const SF_INFO *info)
 					return 1 ;
 				break ;
 
+		case SF_FORMAT_HTK :
+				/* HTK is strictly big endian. */
+				if (endian == SF_ENDIAN_LITTLE || endian == SF_ENDIAN_CPU)
+					return 0 ;
+				if (info->channels != 1)
+					return 0 ;
+				if (subformat == SF_FORMAT_PCM_16)
+					return 1 ;
+				break ;
+
+		case SF_FORMAT_SDS :
+				/* SDS is strictly big endian. */
+				if (endian == SF_ENDIAN_LITTLE || endian == SF_ENDIAN_CPU)
+					return 0 ;
+				if (info->channels != 1)
+					return 0 ;
+				if (subformat == SF_FORMAT_PCM_S8 || subformat == SF_FORMAT_PCM_16 || subformat == SF_FORMAT_PCM_24)
+					return 1 ;
+				break ;
+
 		/*-
 		case SF_FORMAT_SD2 :
 				/+* SD2 is strictly big endian. *+/
@@ -858,9 +690,9 @@ sf_command	(SNDFILE *sndfile, int command, void *data, int datasize)
 		if (data == NULL)
 			return (psf->error = SFE_BAD_CONTROL_CMD) ;
 		if (ENABLE_EXPERIMENTAL_CODE)
-			LSF_SNPRINTF (data, datasize, "%s-%s-exp", PACKAGE, VERSION) ;
+				LSF_SNPRINTF (data, datasize, "%s-%s-exp", PACKAGE_NAME, PACKAGE_VERSION) ;
 		else
-			LSF_SNPRINTF (data, datasize, "%s-%s", PACKAGE, VERSION) ;
+				LSF_SNPRINTF (data, datasize, "%s-%s", PACKAGE_NAME, PACKAGE_VERSION) ;
 		return 0 ;
 
 		case SFC_GET_SIMPLE_FORMAT_COUNT :
@@ -973,12 +805,12 @@ sf_command	(SNDFILE *sndfile, int command, void *data, int datasize)
 		case SFC_CALC_MAX_ALL_CHANNELS :
 			if (data == NULL || datasize != SIGNED_SIZEOF (double) * psf->sf.channels)
 				return (psf->error = SFE_BAD_CONTROL_CMD) ;
-			return psf_calc_max_all_channels (psf, (double *)data, SF_FALSE) ;
+			return psf_calc_max_all_channels (psf, (double*) data, SF_FALSE) ;
 
 		case SFC_CALC_NORM_MAX_ALL_CHANNELS :
 			if (data == NULL || datasize != SIGNED_SIZEOF (double) * psf->sf.channels)
 				return (psf->error = SFE_BAD_CONTROL_CMD) ;
-			return psf_calc_max_all_channels (psf, (double *)data, SF_TRUE) ;
+			return psf_calc_max_all_channels (psf, (double*) data, SF_TRUE) ;
 
 		case SFC_UPDATE_HEADER_NOW :
 			if (psf->write_header)
@@ -991,17 +823,32 @@ sf_command	(SNDFILE *sndfile, int command, void *data, int datasize)
 			break ;
 
 		case SFC_SET_ADD_DITHER_ON_WRITE :
-			if (psf->mode == SFM_WRITE || psf->mode == SFM_RDWR)
-				psf->write_dither = datasize ? SF_TRUE : SF_FALSE ;
+		case SFC_SET_ADD_DITHER_ON_READ :
+			/*
+			** FIXME !
+			** These are obsolete. Just return.
+			** Remove some time after version 1.0.8.
+			*/
 			break ;
 
-		case SFC_SET_ADD_DITHER_ON_READ :
+		case SFC_SET_DITHER_ON_WRITE :
+			if (data == NULL || datasize != SIGNED_SIZEOF (SF_DITHER_INFO))
+				return (psf->error = SFE_BAD_CONTROL_CMD) ;
+			memcpy (&psf->write_dither, data, sizeof (psf->write_dither)) ;
+			if (psf->mode == SFM_WRITE || psf->mode == SFM_RDWR)
+				dither_init (psf, SFM_WRITE) ;
+			break ;
+
+		case SFC_SET_DITHER_ON_READ :
+			if (data == NULL || datasize != SIGNED_SIZEOF (SF_DITHER_INFO))
+				return (psf->error = SFE_BAD_CONTROL_CMD) ;
+			memcpy (&psf->read_dither, data, sizeof (psf->read_dither)) ;
 			if (psf->mode == SFM_READ || psf->mode == SFM_RDWR)
-				psf->read_dither = datasize ? SF_TRUE : SF_FALSE ;
+				dither_init (psf, SFM_READ) ;
 			break ;
 
 		case SFC_FILE_TRUNCATE :
-			if (psf->mode != SFM_READ && psf->mode != SFM_RDWR)
+			if (psf->mode != SFM_WRITE && psf->mode != SFM_RDWR)
 				return SF_TRUE ;
 			if (datasize != sizeof (sf_count_t))
 				return SF_TRUE ;
@@ -1030,6 +877,15 @@ sf_command	(SNDFILE *sndfile, int command, void *data, int datasize)
 			sf_seek (sndfile, 0, SEEK_CUR) ;
 			break ;
 
+		case SFC_GET_EMBED_FILE_INFO :
+			if (data == NULL || datasize != sizeof (SF_EMBED_FILE_INFO))
+				return (psf->error = SFE_BAD_CONTROL_CMD) ;
+
+			((SF_EMBED_FILE_INFO*) data)->offset = psf->fileoffset ;
+			((SF_EMBED_FILE_INFO*) data)->length = psf->filelength ;
+			break ;
+
+		/* Lite remove start */
 		case SFC_TEST_IEEE_FLOAT_REPLACE :
 			psf->ieee_replace = (datasize) ? SF_TRUE : SF_FALSE ;
 			if ((psf->sf.format & SF_FORMAT_SUBMASK) == SF_FORMAT_FLOAT)
@@ -1039,6 +895,14 @@ sf_command	(SNDFILE *sndfile, int command, void *data, int datasize)
 			else
 				return (psf->error = SFE_BAD_CONTROL_CMD) ;
 			break ;
+		/* Lite remove end */
+
+		case SFC_SET_CLIPPING :
+			psf->add_clipping = (datasize) ? SF_TRUE : SF_FALSE ;
+			return psf->add_clipping ;
+
+		case SFC_GET_CLIPPING :
+			return psf->add_clipping ;
 
 		default :
 			/* Must be a file specific command. Pass it on. */
@@ -1093,8 +957,8 @@ sf_seek	(SNDFILE *sndfile, sf_count_t offset, int whence)
 				if (offset == 0)
 				{	if (psf->mode == SFM_READ)
 						return psf->read_current ;
-/*-					if (psf->mode == SFM_WRITE)
-						return psf->write_current ;-*/
+					if (psf->mode == SFM_WRITE)
+						return psf->write_current ;
 					} ;
 				if (psf->mode == SFM_READ)
 					seek_from_start = psf->read_current + offset ;
@@ -1171,7 +1035,7 @@ const char*
 sf_get_string (SNDFILE *sndfile, int str_type)
 {	SF_PRIVATE 	*psf ;
 
-	if ((psf = sndfile) == NULL)
+	if ((psf = (SF_PRIVATE*) sndfile) == NULL)
 		return NULL ;
 	if (psf->Magick != SNDFILE_MAGICK)
 		return NULL ;
@@ -1195,9 +1059,13 @@ sf_count_t
 sf_read_raw		(SNDFILE *sndfile, void *ptr, sf_count_t bytes)
 {	SF_PRIVATE 	*psf ;
 	sf_count_t	count ;
+	int			bytewidth, blockwidth ;
 
 	VALIDATE_SNDFILE_AND_ASSIGN_PSF (sndfile, psf, 1) ;
 
+	bytewidth = (psf->bytewidth > 0) ? psf->bytewidth : 1 ;
+	blockwidth = (psf->blockwidth > 0) ? psf->blockwidth : 1 ;
+
 	if (psf->mode == SFM_WRITE)
 	{	psf->error = SFE_NOT_READMODE ;
 		return	0 ;
@@ -1208,7 +1076,7 @@ sf_read_raw		(SNDFILE *sndfile, void *ptr, sf_count_t bytes)
 		return 0 ;
 		} ;
 
-	if (bytes % (psf->sf.channels * psf->bytewidth))
+	if (bytes % (psf->sf.channels * bytewidth))
 	{	psf->error = SFE_BAD_READ_ALIGN ;
 		return 0 ;
 		} ;
@@ -1216,9 +1084,9 @@ sf_read_raw		(SNDFILE *sndfile, void *ptr, sf_count_t bytes)
 	count = psf_fread (ptr, 1, bytes, psf) ;
 
 	if (count < bytes)
-		psf_memset (((char*)ptr) + count, 0, bytes - count) ;
+		psf_memset (((char*) ptr) + count, 0, bytes - count) ;
 
-	psf->read_current += count / psf->blockwidth ;
+	psf->read_current += count / blockwidth ;
 
 	psf->last_op = SFM_READ ;
 
@@ -1250,7 +1118,7 @@ sf_read_short	(SNDFILE *sndfile, short *ptr, sf_count_t len)
 		return 0 ; /* End of file. */
 		} ;
 
-	if (! psf->read_short || ! psf->seek)
+	if (! psf->read_short || psf->seek == NULL)
 	{	psf->error = SFE_UNIMPLEMENTED ;
 		return	0 ;
 		} ;
@@ -1297,7 +1165,7 @@ sf_readf_short		(SNDFILE *sndfile, short *ptr, sf_count_t frames)
 		return 0 ; /* End of file. */
 		} ;
 
-	if (! psf->read_short || ! psf->seek)
+	if (! psf->read_short || psf->seek == NULL)
 	{	psf->error = SFE_UNIMPLEMENTED ;
 		return 0 ;
 		} ;
@@ -1352,7 +1220,7 @@ sf_read_int		(SNDFILE *sndfile, int *ptr, sf_count_t len)
 		return 0 ;
 		} ;
 
-	if (! psf->read_int || ! psf->seek)
+	if (! psf->read_int || psf->seek == NULL)
 	{	psf->error = SFE_UNIMPLEMENTED ;
 		return 0 ;
 		} ;
@@ -1387,7 +1255,7 @@ sf_readf_int	(SNDFILE *sndfile, int *ptr, sf_count_t frames)
 {	SF_PRIVATE 	*psf ;
 	sf_count_t	count, extra ;
 
-	VALIDATE_SNDFILE_AND_ASSIGN_PSF (sndfile,psf, 1) ;
+	VALIDATE_SNDFILE_AND_ASSIGN_PSF (sndfile, psf, 1) ;
 
 	if (psf->mode == SFM_WRITE)
 	{	psf->error = SFE_NOT_READMODE ;
@@ -1399,7 +1267,7 @@ sf_readf_int	(SNDFILE *sndfile, int *ptr, sf_count_t frames)
 		return 0 ;
 		} ;
 
-	if (! psf->read_int || ! psf->seek)
+	if (! psf->read_int || psf->seek == NULL)
 	{	psf->error = SFE_UNIMPLEMENTED ;
 		return	0 ;
 		} ;
@@ -1454,7 +1322,7 @@ sf_read_float	(SNDFILE *sndfile, float *ptr, sf_count_t len)
 		return 0 ;
 		} ;
 
-	if (! psf->read_float || ! psf->seek)
+	if (! psf->read_float || psf->seek == NULL)
 	{	psf->error = SFE_UNIMPLEMENTED ;
 		return	0 ;
 		} ;
@@ -1501,7 +1369,7 @@ sf_readf_float	(SNDFILE *sndfile, float *ptr, sf_count_t frames)
 		return 0 ;
 		} ;
 
-	if (! psf->read_float || ! psf->seek)
+	if (! psf->read_float || psf->seek == NULL)
 	{	psf->error = SFE_UNIMPLEMENTED ;
 		return	0 ;
 		} ;
@@ -1556,7 +1424,7 @@ sf_read_double	(SNDFILE *sndfile, double *ptr, sf_count_t len)
 		return 0 ;
 		} ;
 
-	if (! psf->read_double || ! psf->seek)
+	if (! psf->read_double || psf->seek == NULL)
 	{	psf->error = SFE_UNIMPLEMENTED ;
 		return	0 ;
 		} ;
@@ -1603,7 +1471,7 @@ sf_readf_double	(SNDFILE *sndfile, double *ptr, sf_count_t frames)
 		return 0 ;
 		} ;
 
-	if (! psf->read_double || ! psf->seek)
+	if (! psf->read_double || psf->seek == NULL)
 	{	psf->error = SFE_UNIMPLEMENTED ;
 		return	0 ;
 		} ;
@@ -1640,15 +1508,19 @@ sf_count_t
 sf_write_raw	(SNDFILE *sndfile, void *ptr, sf_count_t len)
 {	SF_PRIVATE 	*psf ;
 	sf_count_t	count ;
+	int			bytewidth, blockwidth ;
 
 	VALIDATE_SNDFILE_AND_ASSIGN_PSF (sndfile, psf, 1) ;
 
+	bytewidth = (psf->bytewidth > 0) ? psf->bytewidth : 1 ;
+	blockwidth = (psf->blockwidth > 0) ? psf->blockwidth : 1 ;
+
 	if (psf->mode == SFM_READ)
 	{	psf->error = SFE_NOT_WRITEMODE ;
 		return 0 ;
 		} ;
 
-	if (len % (psf->sf.channels * psf->bytewidth))
+	if (len % (psf->sf.channels * bytewidth))
 	{	psf->error = SFE_BAD_WRITE_ALIGN ;
 		return 0 ;
 		} ;
@@ -1659,7 +1531,7 @@ sf_write_raw	(SNDFILE *sndfile, void *ptr, sf_count_t len)
 
 	count = psf_fwrite (ptr, 1, len, psf) ;
 
-	psf->write_current += count / psf->blockwidth ;
+	psf->write_current += count / blockwidth ;
 
 	if (psf->write_current > psf->sf.frames)
 		psf->sf.frames = psf->write_current ;
@@ -1689,7 +1561,7 @@ sf_write_short	(SNDFILE *sndfile, short *ptr, sf_count_t len)
 		return 0 ;
 		} ;
 
-	if (! psf->write_short || ! psf->seek)
+	if (! psf->write_short || psf->seek == NULL)
 	{	psf->error = SFE_UNIMPLEMENTED ;
 		return 0 ;
 		} ;
@@ -1702,7 +1574,7 @@ sf_write_short	(SNDFILE *sndfile, short *ptr, sf_count_t len)
 		psf->write_header (psf, SF_FALSE) ;
 	psf->have_written = SF_TRUE ;
 
-	count = psf->write_short (sndfile, ptr, len) ;
+	count = psf->write_short (psf, ptr, len) ;
 
 	psf->write_current += count / psf->sf.channels ;
 
@@ -1729,7 +1601,7 @@ sf_writef_short	(SNDFILE *sndfile, short *ptr, sf_count_t frames)
 		return 0 ;
 		} ;
 
-	if (! psf->write_short || ! psf->seek)
+	if (! psf->write_short || psf->seek == NULL)
 	{	psf->error = SFE_UNIMPLEMENTED ;
 		return 0 ;
 		} ;
@@ -1742,7 +1614,7 @@ sf_writef_short	(SNDFILE *sndfile, short *ptr, sf_count_t frames)
 		psf->write_header (psf, SF_FALSE) ;
 	psf->have_written = SF_TRUE ;
 
-	count = psf->write_short (sndfile, ptr, frames * psf->sf.channels) ;
+	count = psf->write_short (psf, ptr, frames * psf->sf.channels) ;
 
 	psf->write_current += count / psf->sf.channels ;
 
@@ -1777,7 +1649,7 @@ sf_write_int	(SNDFILE *sndfile, int *ptr, sf_count_t len)
 		return 0 ;
 		} ;
 
-	if (! psf->write_int || ! psf->seek)
+	if (! psf->write_int || psf->seek == NULL)
 	{	psf->error = SFE_UNIMPLEMENTED ;
 		return 0 ;
 		} ;
@@ -1790,7 +1662,7 @@ sf_write_int	(SNDFILE *sndfile, int *ptr, sf_count_t len)
 		psf->write_header (psf, SF_FALSE) ;
 	psf->have_written = SF_TRUE ;
 
-	count = psf->write_int (sndfile, ptr, len) ;
+	count = psf->write_int (psf, ptr, len) ;
 
 	psf->write_current += count / psf->sf.channels ;
 
@@ -1817,7 +1689,7 @@ sf_writef_int	(SNDFILE *sndfile, int *ptr, sf_count_t frames)
 		return 0 ;
 		} ;
 
-	if (! psf->write_int || ! psf->seek)
+	if (! psf->write_int || psf->seek == NULL)
 	{	psf->error = SFE_UNIMPLEMENTED ;
 		return 0 ;
 		} ;
@@ -1830,7 +1702,7 @@ sf_writef_int	(SNDFILE *sndfile, int *ptr, sf_count_t frames)
 		psf->write_header (psf, SF_FALSE) ;
 	psf->have_written = SF_TRUE ;
 
-	count = psf->write_int (sndfile, ptr, frames * psf->sf.channels) ;
+	count = psf->write_int (psf, ptr, frames * psf->sf.channels) ;
 
 	psf->write_current += count / psf->sf.channels ;
 
@@ -1865,7 +1737,7 @@ sf_write_float	(SNDFILE *sndfile, float *ptr, sf_count_t len)
 		return 0 ;
 		} ;
 
-	if (! psf->write_float || ! psf->seek)
+	if (! psf->write_float || psf->seek == NULL)
 	{	psf->error = SFE_UNIMPLEMENTED ;
 		return 0 ;
 		} ;
@@ -1878,7 +1750,7 @@ sf_write_float	(SNDFILE *sndfile, float *ptr, sf_count_t len)
 		psf->write_header (psf, SF_FALSE) ;
 	psf->have_written = SF_TRUE ;
 
-	count = psf->write_float (sndfile, ptr, len) ;
+	count = psf->write_float (psf, ptr, len) ;
 
 	psf->write_current += count / psf->sf.channels ;
 
@@ -1905,7 +1777,7 @@ sf_writef_float	(SNDFILE *sndfile, float *ptr, sf_count_t frames)
 		return 0 ;
 		} ;
 
-	if (! psf->write_float || ! psf->seek)
+	if (! psf->write_float || psf->seek == NULL)
 	{	psf->error = SFE_UNIMPLEMENTED ;
 		return 0 ;
 		} ;
@@ -1914,9 +1786,11 @@ sf_writef_float	(SNDFILE *sndfile, float *ptr, sf_count_t frames)
 		if (psf->seek (psf, SFM_WRITE, psf->write_current) < 0)
 			return 0 ;
 
+	if (psf->have_written == SF_FALSE && psf->write_header != NULL)
+		psf->write_header (psf, SF_FALSE) ;
 	psf->have_written = SF_TRUE ;
 
-	count = psf->write_float (sndfile, ptr, frames * psf->sf.channels) ;
+	count = psf->write_float (psf, ptr, frames * psf->sf.channels) ;
 
 	psf->write_current += count / psf->sf.channels ;
 
@@ -1951,7 +1825,7 @@ sf_write_double	(SNDFILE *sndfile, double *ptr, sf_count_t len)
 		return	0 ;
 		} ;
 
-	if (! psf->write_double || ! psf->seek)
+	if (! psf->write_double || psf->seek == NULL)
 	{	psf->error = SFE_UNIMPLEMENTED ;
 		return 0 ;
 		} ;
@@ -1964,7 +1838,7 @@ sf_write_double	(SNDFILE *sndfile, double *ptr, sf_count_t len)
 		psf->write_header (psf, SF_FALSE) ;
 	psf->have_written = SF_TRUE ;
 
-	count = psf->write_double (sndfile, ptr, len) ;
+	count = psf->write_double (psf, ptr, len) ;
 
 	psf->write_current += count / psf->sf.channels ;
 
@@ -1991,7 +1865,7 @@ sf_writef_double	(SNDFILE *sndfile, double *ptr, sf_count_t frames)
 		return 0 ;
 		} ;
 
-	if (! psf->write_double || ! psf->seek)
+	if (! psf->write_double || psf->seek == NULL)
 	{	psf->error = SFE_UNIMPLEMENTED ;
 		return 0 ;
 		} ;
@@ -2004,7 +1878,7 @@ sf_writef_double	(SNDFILE *sndfile, double *ptr, sf_count_t frames)
 		psf->write_header (psf, SF_FALSE) ;
 	psf->have_written = SF_TRUE ;
 
-	count = psf->write_double (sndfile, ptr, frames * psf->sf.channels) ;
+	count = psf->write_double (psf, ptr, frames * psf->sf.channels) ;
 
 	psf->write_current += count / psf->sf.channels ;
 
@@ -2041,6 +1915,7 @@ format_from_extension (const char *filename)
 	strncpy (buffer, cptr, sizeof (buffer)) ;
 	buffer [sizeof (buffer) - 1] = 0 ;
 
+	/* Convert everything in the buffer to lower case. */
 	cptr = buffer ;
 	while (*cptr)
 	{	*cptr = tolower (*cptr) ;
@@ -2071,27 +1946,27 @@ guess_file_type (SF_PRIVATE *psf, const char *filename)
 		return 0 ;
 		} ;
 
-	if (buffer [0] == MAKE_MARKER ('R','I','F','F') && buffer [2] == MAKE_MARKER ('W','A','V','E'))
+	if (buffer [0] == MAKE_MARKER ('R', 'I', 'F', 'F') && buffer [2] == MAKE_MARKER ('W', 'A', 'V', 'E'))
 		return SF_FORMAT_WAV ;
 
-	if (buffer [0] == MAKE_MARKER ('F','O','R','M'))
-	{	if (buffer [2] == MAKE_MARKER ('A','I','F','F') || buffer [2] == MAKE_MARKER ('A','I','F','C'))
+	if (buffer [0] == MAKE_MARKER ('F', 'O', 'R', 'M'))
+	{	if (buffer [2] == MAKE_MARKER ('A', 'I', 'F', 'F') || buffer [2] == MAKE_MARKER ('A', 'I', 'F', 'C'))
 			return SF_FORMAT_AIFF ;
-		if (buffer [2] == MAKE_MARKER ('8','S','V','X') || buffer [2] == MAKE_MARKER ('1','6','S','V'))
+		if (buffer [2] == MAKE_MARKER ('8', 'S', 'V', 'X') || buffer [2] == MAKE_MARKER ('1', '6', 'S', 'V'))
 			return SF_FORMAT_SVX ;
 		return 0 ;
 		} ;
 
-	if (buffer [0] == MAKE_MARKER ('.','s','n','d') || buffer [0] == MAKE_MARKER ('d','n','s','.'))
+	if (buffer [0] == MAKE_MARKER ('.', 's', 'n', 'd') || buffer [0] == MAKE_MARKER ('d', 'n', 's', '.'))
 		return SF_FORMAT_AU ;
 
-	if ((buffer [0] == MAKE_MARKER ('f','a','p',' ') || buffer [0] == MAKE_MARKER (' ','p','a','f')))
+	if ((buffer [0] == MAKE_MARKER ('f', 'a', 'p', ' ') || buffer [0] == MAKE_MARKER (' ', 'p', 'a', 'f')))
 		return SF_FORMAT_PAF ;
 
-	if (buffer [0] == MAKE_MARKER ('N','I','S','T'))
+	if (buffer [0] == MAKE_MARKER ('N', 'I', 'S', 'T'))
 		return SF_FORMAT_NIST ;
 
-	if (buffer [0] == MAKE_MARKER ('C','r','e','a') && buffer [1] == MAKE_MARKER ('t','i','v','e'))
+	if (buffer [0] == MAKE_MARKER ('C', 'r', 'e', 'a') && buffer [1] == MAKE_MARKER ('t', 'i', 'v', 'e'))
 		return SF_FORMAT_VOC ;
 
 	if ((buffer [0] & MAKE_MARKER (0xFF, 0xFF, 0xF8, 0xFF)) == MAKE_MARKER (0x64, 0xA3, 0x00, 0x00) ||
@@ -2142,6 +2017,10 @@ guess_file_type (SF_PRIVATE *psf, const char *filename)
 	if (buffer [0] == MAKE_MARKER (0x30, 0x26, 0xB2, 0x75) && buffer [1] == MAKE_MARKER (0x8E, 0x66, 0xCF, 0x11))
 		return 0 /*-SF_FORMAT_WMA-*/ ;
 
+	/* HMM (Hidden Markov Model) Tool Kit. */
+	if (2 * BEI2H_INT (buffer [0]) + 12 == psf->filelength && buffer [2] == MAKE_MARKER (0, 2, 0, 0))
+		return SF_FORMAT_HTK ;
+
 	if (buffer [0] == MAKE_MARKER ('f', 'L', 'a', 'C'))
 		return 0 /*-SF_FORMAT_FLAC-*/ ;
 
@@ -2155,6 +2034,9 @@ guess_file_type (SF_PRIVATE *psf, const char *filename)
 	if (buffer [0] == MAKE_MARKER ('a', 'j', 'k', 'g'))
 		return 0 /*-SF_FORMAT_SHN-*/ ;
 
+	if (OS_IS_MACOSX && (format = macos_guess_file_type (psf, filename)) != 0)
+		return format ;
+
 	/*	Detect wacky MacOS header stuff. This might be "Sound Designer II". */
 	memcpy (cptr , buffer, sizeof (buffer)) ;
 	if (cptr [0] == 0 && cptr [1] > 0 && psf->sf.seekable)
@@ -2235,3 +2117,297 @@ copy_filename (SF_PRIVATE *psf, const char *path)
 
 /*==============================================================================
 */
+
+static int
+psf_close (SF_PRIVATE  *psf)
+{	int			error ;
+
+	if (psf->close)
+		error = psf->close (psf) ;
+
+		psf_fclose (psf) ;
+
+	if (psf->fdata)
+		free (psf->fdata) ;
+
+	if (psf->interleave)
+		free (psf->interleave) ;
+
+	if (psf->dither)
+		free (psf->dither) ;
+
+	if (psf->format_desc)
+	{	memset (psf->format_desc, 0, strlen (psf->format_desc)) ;
+		free (psf->format_desc) ;
+		} ;
+
+	memset (psf, 0, sizeof (SF_PRIVATE)) ;
+	free (psf) ;
+
+	return 0 ;
+} /* psf_close */
+
+static int
+psf_open_file (SF_PRIVATE *psf, int mode, SF_INFO *sfinfo)
+{	int		error ;
+	
+	if (mode != SFM_READ && mode != SFM_WRITE && mode != SFM_RDWR)
+		return SFE_BAD_OPEN_MODE ;
+
+	if (sfinfo == NULL)
+		return SFE_BAD_SF_INFO_PTR ;
+
+	if (mode == SFM_READ)
+	{	if ((sfinfo->format & SF_FORMAT_TYPEMASK) == SF_FORMAT_RAW)
+		{	if (sf_format_check (sfinfo) == 0)
+				return SFE_RAW_BAD_FORMAT ;
+			}
+		else
+			memset (sfinfo, 0, sizeof (SF_INFO)) ;
+		} ;
+
+	sf_errno = error = 0 ;
+	sf_logbuffer [0] = 0 ;
+
+	memcpy (&(psf->sf), sfinfo, sizeof (SF_INFO)) ;
+
+	psf->Magick 		= SNDFILE_MAGICK ;
+	psf->norm_float 	= SF_TRUE ;
+	psf->norm_double	= SF_TRUE ;
+	psf->mode 			= mode ;
+	psf->dataoffset		= -1 ;
+	psf->datalength		= -1 ;
+	psf->read_current	= -1 ;
+	psf->write_current	= -1 ;
+	psf->auto_header 	= SF_FALSE ;
+	psf->rwf_endian		= SF_ENDIAN_LITTLE ;
+	psf->seek			= psf_default_seek ;
+
+	psf->sf.sections = 1 ;
+
+	psf->is_pipe = psf_is_pipe (psf) ;
+
+	if (psf->is_pipe)
+	{	psf->sf.seekable = SF_FALSE ;
+		psf->filelength = SF_COUNT_MAX ;
+		}
+	else
+	{	psf->sf.seekable = SF_TRUE ;
+
+	/* File is open, so get the length. */
+	psf->filelength = psf_get_filelen (psf) ;
+		} ;
+
+	if (psf->fileoffset > 0)
+	{	switch (psf->mode)
+		{	case SFM_READ :
+				if (psf->filelength < 44)
+				{	psf_log_printf (psf, "Short filelength: %D (fileoffset: %D)\n", psf->filelength, psf->fileoffset) ;
+					return SFE_BAD_OFFSET ;
+					} ;
+				break ;
+		
+			case SFM_WRITE :
+				psf->fileoffset = 0 ;
+				psf_fseek (psf, 0, SEEK_END) ;
+				psf->fileoffset = psf_ftell (psf) ;
+				break ;
+
+			case SFM_RDWR :
+				return SFE_NO_EMBEDDED_RDWR ;
+			} ;
+
+		psf_log_printf (psf, "Embedded file offset : %D\n", psf->fileoffset) ;
+		} ;
+
+	if (psf->filelength == SF_COUNT_MAX)
+		psf_log_printf (psf, "Length : unknown\n") ;
+	else
+	psf_log_printf (psf, "Length : %D\n", psf->filelength) ;
+
+	if (mode == SFM_WRITE || (mode == SFM_RDWR && psf->filelength == 0))
+	{	/* If the file is being opened for write or RDWR and the file is currently
+		** empty, then the SF_INFO struct must contain valid data.
+		*/
+		if (sf_format_check (&(psf->sf)) == 0)
+			return SFE_BAD_OPEN_FORMAT ;
+		}
+	else if ((psf->sf.format & SF_FORMAT_TYPEMASK) != SF_FORMAT_RAW)
+	{	/* If type RAW has not been specified then need to figure out file type. */
+		psf->sf.format = guess_file_type (psf, psf->filename) ;
+		} ;
+
+	/* Prevent unnecessary seeks */
+	psf->last_op = psf->mode ;
+
+	/* Set bytewidth if known. */
+	switch (psf->sf.format & SF_FORMAT_SUBMASK)
+	{	case SF_FORMAT_PCM_S8 :
+		case SF_FORMAT_PCM_U8 :
+		case SF_FORMAT_ULAW :
+		case SF_FORMAT_ALAW :
+		case SF_FORMAT_DPCM_8 :
+				psf->bytewidth = 1 ;
+				break ;
+
+		case SF_FORMAT_PCM_16 :
+		case SF_FORMAT_DPCM_16 :
+				psf->bytewidth = 2 ;
+				break ;
+
+		case SF_FORMAT_PCM_24 :
+				psf->bytewidth = 3 ;
+				break ;
+
+		case SF_FORMAT_PCM_32 :
+		case SF_FORMAT_FLOAT :
+				psf->bytewidth = 4 ;
+				break ;
+
+		case SF_FORMAT_DOUBLE :
+				psf->bytewidth = 8 ;
+				break ;
+		} ;
+
+	/* Call the initialisation function for the relevant file type. */
+	switch (psf->sf.format & SF_FORMAT_TYPEMASK)
+	{	case	SF_FORMAT_WAV :
+				error = wav_open (psf) ;
+				break ;
+
+		case	SF_FORMAT_AIFF :
+				error = aiff_open (psf) ;
+				break ;
+
+		case	SF_FORMAT_AU :
+				error = au_open (psf) ;
+				break ;
+
+		case	SF_FORMAT_AU | SF_FORMAT_ULAW :
+				error = au_nh_open (psf) ;
+				break ;
+
+		case	SF_FORMAT_RAW :
+				error = raw_open (psf) ;
+				break ;
+
+		case	SF_FORMAT_W64 :
+				error = w64_open (psf) ;
+				break ;
+
+		/* Lite remove start */
+		case	SF_FORMAT_PAF :
+				error = paf_open (psf) ;
+				break ;
+
+		case	SF_FORMAT_SVX :
+				error = svx_open (psf) ;
+				break ;
+
+		case	SF_FORMAT_NIST :
+				error = nist_open (psf) ;
+				break ;
+
+		case	SF_FORMAT_IRCAM :
+				error = ircam_open (psf) ;
+				break ;
+
+		case	SF_FORMAT_VOC :
+				error = voc_open (psf) ;
+				break ;
+
+		case	SF_FORMAT_SDS :
+				error = sds_open (psf) ;
+				break ;
+
+		case	SF_FORMAT_OGG :
+				error = ogg_open (psf) ;
+				break ;
+
+		case	SF_FORMAT_TXW :
+				error = txw_open (psf) ;
+				break ;
+
+		case	SF_FORMAT_WVE :
+				error = wve_open (psf) ;
+				break ;
+
+		case	SF_FORMAT_DWD :
+				error = dwd_open (psf) ;
+				break ;
+
+		case	SF_FORMAT_MAT4 :
+				error = mat4_open (psf) ;
+				break ;
+
+		case	SF_FORMAT_MAT5 :
+				error = mat5_open (psf) ;
+				break ;
+
+		case	SF_FORMAT_PVF :
+				error = pvf_open (psf) ;
+				break ;
+
+		case	SF_FORMAT_XI :
+				error = xi_open (psf) ;
+				break ;
+
+		case	SF_FORMAT_HTK :
+				error = htk_open (psf) ;
+				break ;
+
+		case	SF_FORMAT_SD2 :
+				error = sd2_open (psf) ;
+				break ;
+
+		case	SF_FORMAT_REX2 :
+				error = rx2_open (psf) ;
+				break ;
+		/* Lite remove end */
+
+		default :
+				error = SFE_UNKNOWN_FORMAT ;
+		} ;
+
+	if (error)
+		return error ;
+
+	/* For now, check whether embedding is supported. */
+	if (psf->fileoffset > 0 && 
+			((psf->sf.format & SF_FORMAT_TYPEMASK) != SF_FORMAT_WAV) && 
+			((psf->sf.format & SF_FORMAT_TYPEMASK) != SF_FORMAT_AIFF) &&
+			((psf->sf.format & SF_FORMAT_TYPEMASK) != SF_FORMAT_AU)
+			)
+		return SFE_NO_EMBED_SUPPORT ;
+
+	if (psf->fileoffset > 0)
+		psf_log_printf (psf, "Embedded file length : %D\n", psf->filelength) ;
+
+	if (mode == SFM_RDWR && sf_format_check (&(psf->sf)) == 0)
+		return SFE_BAD_RDWR_FORMAT ;
+
+	if (validate_sfinfo (&(psf->sf)) == 0)
+	{	psf_log_SF_INFO (psf) ;
+		save_header_info (psf) ;
+		return SFE_BAD_SF_INFO ;
+		} ;
+
+	if (validate_psf (psf) == 0)
+	{	save_header_info (psf) ;
+		return SFE_INTERNAL ;
+		} ;
+
+	psf->read_current  = 0 ;
+	psf->write_current = (psf->mode == SFM_RDWR) ? psf->sf.frames : 0 ;
+
+	memcpy (sfinfo, &(psf->sf), sizeof (SF_INFO)) ;
+
+	return 0 ;
+} /* psf_open_file */
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: cd4f9e91-a8ec-4154-9bf6-fe4b8c69a615
+*/
