@@ -24,7 +24,6 @@
 */
 
 #include	<stdio.h>
-#include	<unistd.h>
 #include	<fcntl.h>
 #include	<string.h>
 #include	<ctype.h>
@@ -370,3 +369,10 @@ txw_seek (SF_PRIVATE *psf, int mode, sf_count_t offset)
 } /* txw_seek */
 
 #endif
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: 4d0ba7af-b1c5-46b4-a900-7c6f59fd9a89
+*/
