@@ -16,9 +16,6 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-
-#include	<unistd.h>
-
 #include	"sndfile.h"
 #include	"float_cast.h"
 #include	"common.h"
@@ -64,9 +61,12 @@ ulaw_init (SF_PRIVATE *psf)
 	psf->bytewidth  = 1 ;
 	psf->blockwidth = psf->sf.channels ;
 
-	psf->filelength = psf_get_filelen (psf) ;
+	if (psf->filelength > psf->dataoffset)
 	psf->datalength = (psf->dataend) ? psf->dataend - psf->dataoffset :
 							psf->filelength - psf->dataoffset ;
+	else
+		psf->datalength = 0 ;
+
 	psf->sf.frames = psf->datalength / psf->blockwidth ;
 
 	return 0 ;
@@ -1044,3 +1044,10 @@ d2ulaw_array      (double *ptr, unsigned int count, unsigned char *buffer, doubl
 		} ;
 } /* d2ulaw_array */
 
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: 655cc790-f058-45e8-89c9-86967cccc37e
+*/
