@@ -24,12 +24,13 @@
 **	Utterly woeful.
 */
 
+#include "config.h"
+
 #include	<stdio.h>
-#include	<unistd.h>
+#include <stdlib.h>
 #include	<string.h>
 
 #include	"sndfile.h"
-#include	"config.h"
 #include	"sfendian.h"
 #include	"common.h"
 
@@ -103,6 +104,9 @@ int
 voc_open	(SF_PRIVATE *psf)
 {	int subformat, error = 0 ;
 
+	if (psf->is_pipe)
+		return SFE_VOC_NO_PIPE ;
+
 	if (psf->mode == SFM_READ || (psf->mode == SFM_RDWR && psf->filelength > 0))
 	{	if ((error = voc_read_header (psf)))
 			return error ;
@@ -230,20 +234,17 @@ voc_read_header	(SF_PRIVATE *psf)
 
 		psf->sf.samplerate = 1000000 / (256 - (rate_byte & 0xFF)) ;
 						
-		psf_log_printf (psf,	" Sound Data : %d\n"
-								"  sr   : %d => %dHz\n"
-								"  comp : %d\n", 
+		psf_log_printf (psf, " Sound Data : %d\n  sr   : %d => %dHz\n  comp : %d\n",
 								size, rate_byte, psf->sf.samplerate, compression) ;
 	
 		if (offset + size - 1 > psf->filelength)
 		{	psf_log_printf (psf, "Seems to be a truncated file.\n") ;
-			puts ("Seems to be a truncated file.\n") ;
-			psf_log_printf (psf, "offset : %d    size : %d    filelength : %d\n", offset, size, psf->filelength) ;
+			psf_log_printf (psf, "offset: %d    size: %d    sum: %d    filelength: %D\n", offset, size, offset + size, psf->filelength) ;
 			return SFE_VOC_BAD_SECTIONS ;
 			}
 		else if (offset + size - 1 < psf->filelength)
-		{	psf_log_printf (psf, "Seems to be a multi-segment file.\n") ;
-			psf_log_printf (psf, "offset : %d    size : %d    filelength : %d\n", offset, size, psf->filelength) ;
+		{	psf_log_printf (psf, "Seems to be a multi-segment file (#1).\n") ;
+			psf_log_printf (psf, "offset: %d    size: %d    sum: %d    filelength: %D\n", offset, size, offset + size, psf->filelength) ;
 			return SFE_VOC_BAD_SECTIONS ;
 			} ;
 			
@@ -301,13 +302,12 @@ voc_read_header	(SF_PRIVATE *psf)
 
 		if (offset + size - 1 > psf->filelength)
 		{	psf_log_printf (psf, "Seems to be a truncated file.\n") ;
-			puts ("Seems to be a truncated file.\n") ;
-			/*-printf ("offset : %d    size : %d    filelength : %ld\n", offset, size, psf->filelength) ;-*/
+			psf_log_printf (psf, "offset: %d    size: %d    sum: %d    filelength: %D\n", offset, size, offset + size, psf->filelength) ;
 			return SFE_VOC_BAD_SECTIONS ;
 			}
 		else if (offset + size - 1 < psf->filelength)
-		{	/*-printf ("offset : %d    size : %d    filelength : %ld\n", offset, size, psf->filelength) ;-*/
-			psf_log_printf (psf, "Seems to be a multi-segment file.\n") ;
+		{	psf_log_printf (psf, "Seems to be a multi-segment file (#2).\n") ;
+			psf_log_printf (psf, "offset: %d    size: %d    sum: %d    filelength: %D\n", offset, size, offset + size, psf->filelength) ;
 			return SFE_VOC_BAD_SECTIONS ;
 			} ;
 			
@@ -369,7 +369,7 @@ voc_read_header	(SF_PRIVATE *psf)
 			size = psf->filelength - 31 ;
 			}
 		else if (size + 31 < psf->filelength)
-			psf_log_printf (psf, "Seems to be a multi-segment file.\n") ;
+			psf_log_printf (psf, "Seems to be a multi-segment file (#3).\n") ;
 			
 		switch (encoding)
 		{	case 0 :
@@ -480,14 +480,14 @@ voc_write_header (SF_PRIVATE *psf, int calc_length)
 
 			case SF_FORMAT_PCM_16 :
 					psf->bytewidth = 2 ;
-					length = psf->sf.frames * psf->sf.channels * psf->bytewidth + 12;
+					length = psf->sf.frames * psf->sf.channels * psf->bytewidth + 12 ;
 					/* Marker, length, sample rate, bitwidth, stereo flag, encoding and fourt zero bytes. */
 					psf_binheader_writef (psf, "e1341124", VOC_EXTENDED_II, length, psf->sf.samplerate, 16, psf->sf.channels, 4, 0) ;
 					break ;
 
 			case SF_FORMAT_ALAW :
 					psf->bytewidth = 1 ;
-					length = psf->sf.frames * psf->sf.channels * psf->bytewidth + 12;
+					length = psf->sf.frames * psf->sf.channels * psf->bytewidth + 12 ;
 					psf_binheader_writef (psf, "e1341124", VOC_EXTENDED_II, length, psf->sf.samplerate, 8, psf->sf.channels, 6, 0) ;
 					break ;
 				
@@ -869,3 +869,10 @@ BLOCK 9 - data block that supersedes blocks 1 and 8.
         Data is stored left, right
 
 ------------------------------------------------------------------------*/
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: 40a50167-a81c-463a-9e1d-3282ff84e09d
+*/
