@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2003 Erik de Castro Lopo <erikd@zip.com.au>
+** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@zip.com.au>
 **  
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -17,7 +17,6 @@
 */
 
 #include	<stdio.h>
-#include	<unistd.h>
 #include	<string.h>
 #include	<ctype.h>
 #include	<time.h>
@@ -38,14 +37,14 @@
 */
 
 #define MAKE_HASH16(x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,xa,xb,xc,xd,xe,xf)	\
-				( (x0)      ^((x1)<< 1)^((x2)<< 2)^((x3)<< 3)^	\
-				  ((x4)<< 4)^((x5)<< 5)^((x6)<< 6)^((x7)<< 7)^	\
-				  ((x8)<< 8)^((x9)<< 9)^((xa)<<10)^((xb)<<11)^	\
-				  ((xc)<<12)^((xd)<<13)^((xe)<<14)^((xf)<<15)	)
+			(	(x0)			^ ((x1) << 1)	^ ((x2) << 2)	^ ((x3) << 3) ^	\
+				((x4) << 4) 	^ ((x5) << 5)	^ ((x6) << 6)	^ ((x7) << 7) ^	\
+				((x8) << 8) 	^ ((x9) << 9)	^ ((xa) << 10)	^ ((xb) << 11) ^ \
+				((xc) << 12) 	^ ((xd) << 13)	^ ((xe) << 14)	^ ((xf) << 15)	)
 
 #define MAKE_MARKER16(name,x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,xa,xb,xc,xd,xe,xf)	\
-			static unsigned char name [16] = { (x0),(x1),(x2),(x3),(x4),(x5), \
-				(x6),(x7),(x8),(x9),(xa),(xb),(xc),(xd),(xe),(xf) }
+			static unsigned char name [16] = { (x0), (x1), (x2), (x3), (x4), (x5), \
+				(x6), (x7), (x8), (x9), (xa), (xb), (xc), (xd), (xe), (xf) }
 
 #define	riff_HASH16 MAKE_HASH16 ('r', 'i', 'f', 'f', 0x2E, 0x91, 0xCF, 0x11, 0xA5, \
 								0xD6, 0x28, 0xDB, 0x04, 0xC1, 0x00, 0x00)
@@ -108,7 +107,7 @@ w64_open	(SF_PRIVATE *psf)
 	if (psf->mode == SFM_READ || (psf->mode == SFM_RDWR &&psf->filelength > 0))
 	{	if ((error = w64_read_header (psf, &blockalign, &framesperblock)))
 			return error ;
-		};
+		} ;
 		
 	if ((psf->sf.format & SF_FORMAT_TYPEMASK) != SF_FORMAT_W64)
 		return	SFE_BAD_OPEN_FORMAT ;
@@ -116,7 +115,10 @@ w64_open	(SF_PRIVATE *psf)
 	subformat = psf->sf.format & SF_FORMAT_SUBMASK ;
 
 	if (psf->mode == SFM_WRITE || psf->mode == SFM_RDWR)
-	{	psf->endian      = SF_ENDIAN_LITTLE ;		/* All W64 files are little endian. */
+	{	if (psf->is_pipe)
+			return SFE_NO_PIPE_WRITE ;
+
+		psf->endian = SF_ENDIAN_LITTLE ;		/* All W64 files are little endian. */
 	
 		psf->blockwidth  = psf->bytewidth * psf->sf.channels ;
 
@@ -125,7 +127,7 @@ w64_open	(SF_PRIVATE *psf)
 			framesperblock = -1 ;
 
 			/* FIXME : This block must go */
-			SF_COUNT_MAX_POSITIVE (psf->filelength) ;
+			psf->filelength = SF_COUNT_MAX ;
 			psf->datalength = psf->filelength ;
 			if (psf->sf.frames <= 0)
 				psf->sf.frames = (psf->blockwidth) ? psf->filelength / psf->blockwidth : psf->filelength ; 
@@ -151,14 +153,6 @@ w64_open	(SF_PRIVATE *psf)
 					error = pcm_init (psf) ;
 					break ;
 
-		case SF_FORMAT_FLOAT : 
-					error = float32_init (psf) ;
-					break ;
-
-		case SF_FORMAT_DOUBLE : 
-					error = double64_init (psf) ;
-					break ;
-
 		case SF_FORMAT_ULAW : 
 					error = ulaw_init (psf) ;
 					break ;
@@ -167,6 +161,15 @@ w64_open	(SF_PRIVATE *psf)
 					error = alaw_init (psf) ;
 					break ;
 
+		/* Lite remove start */
+		case SF_FORMAT_FLOAT :
+					error = float32_init (psf) ;
+					break ;
+
+		case SF_FORMAT_DOUBLE :
+					error = double64_init (psf) ;
+					break ;
+
 		case SF_FORMAT_IMA_ADPCM : 
 					error = wav_w64_ima_init (psf, blockalign, framesperblock) ;
 					break ;
@@ -174,6 +177,7 @@ w64_open	(SF_PRIVATE *psf)
 		case SF_FORMAT_MS_ADPCM : 
 					error = wav_w64_msadpcm_init (psf, blockalign, framesperblock) ;
 					break ;
+		/* Lite remove end */
 
 		case SF_FORMAT_GSM610 : 
 					error = gsm610_init (psf) ;
@@ -212,9 +216,9 @@ w64_read_header	(SF_PRIVATE *psf, int *blockalign, int *framesperblock)
 					bytesread += psf_binheader_readf (psf, "e8", &chunk_size) ;
 					
 					if (psf->filelength  < chunk_size)
-						psf_log_printf (psf, "riff : %d (should be %d)\n", chunk_size, psf->filelength) ;
+						psf_log_printf (psf, "riff : %D (should be %D)\n", chunk_size, psf->filelength) ;
 					else
-						psf_log_printf (psf, "riff : %d\n", chunk_size) ;
+						psf_log_printf (psf, "riff : %D\n", chunk_size) ;
 
 					parsestage |= HAVE_riff ;
 					break ;
@@ -235,7 +239,7 @@ w64_read_header	(SF_PRIVATE *psf, int *blockalign, int *framesperblock)
 						return SFE_W64_NO_FMT ;
 						
 					bytesread += psf_binheader_readf (psf, "e8", &chunk_size) ;
-					psf_log_printf (psf, " fmt : %d (0x%X)\n", chunk_size, chunk_size) ;
+					psf_log_printf (psf, " fmt : %D\n", chunk_size) ;
 						
 					/* size of 16 byte marker and 8 byte chunk_size value. */
 					chunk_size -= 24 ;
@@ -254,8 +258,7 @@ w64_read_header	(SF_PRIVATE *psf, int *blockalign, int *framesperblock)
 					{	sf_count_t frames ;
 					
 						psf_binheader_readf (psf, "e88", &chunk_size, &frames) ;
-						psf_log_printf (psf, "   fact : 0x%X\n"
-											 "     frames : %d\n", 
+						psf_log_printf (psf, "   fact : %D\n     frames : %D\n",
 										chunk_size, frames) ;
 						} ;
 					break ;
@@ -274,7 +277,7 @@ w64_read_header	(SF_PRIVATE *psf, int *blockalign, int *framesperblock)
 					if (chunk_size % 8)
 						chunk_size += 8 - (chunk_size % 8) ;
 
-					psf_log_printf (psf, "data : %d (0x%X)\n", chunk_size, psf->datalength) ;
+					psf_log_printf (psf, "data : %D\n", chunk_size) ;
 
 					parsestage |= HAVE_data ;
 
@@ -296,13 +299,13 @@ w64_read_header	(SF_PRIVATE *psf, int *blockalign, int *framesperblock)
 					break ;
 			} ;	/* switch (dword) */
 
-		if (! psf->sf.seekable && (parsestage & HAVE_data))
+		if (psf->sf.seekable == 0 && (parsestage & HAVE_data))
 			break ;
 
-		if (psf_ftell (psf) >= (int) (psf->filelength - (2 * sizeof (dword))))
+		if (psf_ftell (psf) >= (psf->filelength - (2 * SIGNED_SIZEOF (dword))))
 			break ;
 
-		if (psf->logindex >= sizeof (psf->logbuffer) - 2)
+		if (psf->logindex >= SIGNED_SIZEOF (psf->logbuffer) - 2)
 			return SFE_LOG_OVERRUN ;
 		} ; /* while (1) */
 		
@@ -311,6 +314,7 @@ w64_read_header	(SF_PRIVATE *psf, int *blockalign, int *framesperblock)
 
 	psf->endian = SF_ENDIAN_LITTLE ;		/* All WAV files are little endian. */
 	
+	if (psf_ftell (psf) != psf->dataoffset)
 	psf_fseek (psf, psf->dataoffset, SEEK_SET) ;
 	
 	psf->close = w64_close ;
@@ -371,15 +375,14 @@ w64_write_header (SF_PRIVATE *psf, int calc_length)
 	current = psf_ftell (psf) ;
 
 	if (calc_length)
-	{	psf_fseek (psf, 0, SEEK_END) ;
-		psf->filelength = psf_ftell (psf) ;
-		psf_fseek (psf, 0, SEEK_SET) ;
+	{	psf->filelength = psf_get_filelen (psf) ;
 
 		psf->datalength = psf->filelength - psf->dataoffset ;
 		if (psf->dataend)
 			psf->datalength -= psf->filelength - psf->dataend ;
 
-		psf->sf.frames = psf->datalength / (psf->bytewidth * psf->sf.channels) ;
+		if (psf->bytewidth)
+			psf->sf.frames = psf->datalength / (psf->bytewidth * psf->sf.channels) ;
 		} ;
 
 	/* Reset the current header length to zero. */
@@ -447,6 +450,7 @@ w64_write_header (SF_PRIVATE *psf, int calc_length)
 					add_fact_chunk = SF_TRUE ;
 					break ;
 
+		/* Lite remove start */
 		case SF_FORMAT_IMA_ADPCM : 
 					{	int		blockalign, framesperblock, bytespersec ;
 
@@ -495,6 +499,7 @@ w64_write_header (SF_PRIVATE *psf, int calc_length)
 
 					add_fact_chunk = SF_TRUE ;
 					break ;
+		/* Lite remove end */
 
 		case SF_FORMAT_GSM610 : 
 					{	int  bytespersec ;
@@ -545,24 +550,16 @@ static int
 w64_close	(SF_PRIVATE  *psf)
 {	
 	if (psf->mode == SFM_WRITE || psf->mode == SFM_RDWR)
-	{	/*  Now we know for certain the length of the file we can
-		 *  re-write correct values for the riff and data chunks.
-		 */
-
-		psf_fseek (psf, 0, SEEK_END) ;
-		psf->dataend = psf_ftell (psf) ;
-
-		psf_fseek (psf, 0, SEEK_END) ;
-		psf->filelength = psf_ftell (psf) ;
-		psf_fseek (psf, 0, SEEK_SET) ;
-		
-		psf->datalength = psf->filelength - psf->dataoffset - (psf->filelength - psf->dataend) ;
- 		psf->sf.frames = psf->datalength / (psf->bytewidth * psf->sf.channels) ;
-
-		w64_write_header (psf, SF_FALSE) ;
-		} ;
+		w64_write_header (psf, SF_TRUE) ;
 
 	return 0 ;
 } /* w64_close */
 
 
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: 9aa4e141-538a-4dd9-99c9-b3f0f2dd4f4a
+*/
