@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2003 Erik de Castro Lopo <erikd@zip.com.au>
+** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@zip.com.au>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -18,7 +18,6 @@
 
 
 #include	<stdio.h>
-#include	<unistd.h>
 #include	<string.h>
 #include	<ctype.h>
 #include	<time.h>
@@ -76,6 +75,7 @@
 
 /* Weird WAVPACK marker which can show up at the start of the DATA section. */
 #define wvpk_MARKER (MAKE_MARKER ('w', 'v', 'p', 'k'))
+#define OggS_MARKER (MAKE_MARKER ('O', 'g', 'g', 'S'))
 
 enum
 {	HAVE_RIFF	= 0x01,
@@ -118,7 +118,10 @@ wav_open	(SF_PRIVATE *psf)
 	subformat = psf->sf.format & SF_FORMAT_SUBMASK ;
 
 	if (psf->mode == SFM_WRITE || psf->mode == SFM_RDWR)
-	{	if ((psf->sf.format & SF_FORMAT_TYPEMASK) != SF_FORMAT_WAV)
+	{	if (psf->is_pipe)
+			return SFE_NO_PIPE_WRITE ;
+
+		if ((psf->sf.format & SF_FORMAT_TYPEMASK) != SF_FORMAT_WAV)
 			return	SFE_BAD_OPEN_FORMAT ;
 
 		psf->endian = SF_ENDIAN_LITTLE ;	/* All WAV files are little endian. */
@@ -161,14 +164,6 @@ wav_open	(SF_PRIVATE *psf)
 					error = pcm_init (psf) ;
 					break ;
 
-		case SF_FORMAT_FLOAT :
-					error = float32_init (psf) ;
-					break ;
-
-		case SF_FORMAT_DOUBLE :
-					error = double64_init (psf) ;
-					break ;
-
 		case SF_FORMAT_ULAW :
 					error = ulaw_init (psf) ;
 					break ;
@@ -177,6 +172,15 @@ wav_open	(SF_PRIVATE *psf)
 					error = alaw_init (psf) ;
 					break ;
 
+		/* Lite remove start */
+		case SF_FORMAT_FLOAT :
+					error = float32_init (psf) ;
+					break ;
+
+		case SF_FORMAT_DOUBLE :
+					error = double64_init (psf) ;
+					break ;
+
 		case SF_FORMAT_IMA_ADPCM :
 					error = wav_w64_ima_init (psf, blockalign, framesperblock) ;
 					break ;
@@ -184,6 +188,7 @@ wav_open	(SF_PRIVATE *psf)
 		case SF_FORMAT_MS_ADPCM :
 					error = wav_w64_msadpcm_init (psf, blockalign, framesperblock) ;
 					break ;
+		/* Lite remove end */
 
 		case SF_FORMAT_GSM610 :
 					error = gsm610_init (psf) ;
@@ -232,6 +237,8 @@ wav_read_header	(SF_PRIVATE *psf, int *blockalign, int *framesperblock)
 					{	psf_log_printf (psf, "RIFF : %u (should be %D)\n", (sf_count_t) RIFFsize, psf->filelength - 2 * SIGNED_SIZEOF (dword)) ;
 						RIFFsize = dword ;
 						}
+					else if (RIFFsize < 100)
+						psf_log_printf (psf, "RIFF : %u (should probably be bigger)\n", RIFFsize) ;
 					else
 						psf_log_printf (psf, "RIFF : %u\n", RIFFsize) ;
 
@@ -272,6 +279,11 @@ wav_read_header	(SF_PRIVATE *psf, int *blockalign, int *framesperblock)
 					psf->datalength = dword ;
 					psf->dataoffset = psf_ftell (psf) ;
 
+					if (dword == 0 && RIFFsize == 8 && psf->filelength > 44)
+					{	psf_log_printf (psf, "*** Looks like a WAV file which wasn't closed properly. Fixing it.\n") ;
+						psf->datalength = dword = psf->filelength - psf->dataoffset ;
+						} ;
+
 					if (psf->datalength > psf->filelength - psf->dataoffset)
 					{	psf_log_printf (psf, "data : %D (should be %D)\n", psf->datalength, psf->filelength - psf->dataoffset) ;
 						psf->datalength = psf->filelength - psf->dataoffset ;
@@ -420,7 +432,7 @@ wav_read_header	(SF_PRIVATE *psf, int *blockalign, int *framesperblock)
 			case MEXT_MARKER :
 			case PAD_MARKER :
 					psf_binheader_readf (psf, "e4", &dword) ;
-					psf_log_printf (psf, "%M : %d\n", marker, dword) ;
+					psf_log_printf (psf, "%M : %u\n", marker, dword) ;
 					dword += (dword & 1) ;
 					psf_binheader_readf (psf, "j", dword) ;
 					break ;
@@ -446,12 +458,12 @@ wav_read_header	(SF_PRIVATE *psf, int *blockalign, int *framesperblock)
 		if (! psf->sf.seekable && (parsestage & HAVE_data))
 			break ;
 
-		if (psf_ftell (psf) >= psf->filelength - sizeof (dword))
+		if (psf_ftell (psf) >= psf->filelength - SIGNED_SIZEOF (dword))
 		{	psf_log_printf (psf, "End\n") ;
 			break ;
 			} ;
 
-		if (psf->logindex >= sizeof (psf->logbuffer) - 2)
+		if (psf->logindex >= SIGNED_SIZEOF (psf->logbuffer) - 2)
 			return SFE_LOG_OVERRUN ;
 		} ; /* while (1) */
 
@@ -462,13 +474,15 @@ wav_read_header	(SF_PRIVATE *psf, int *blockalign, int *framesperblock)
 
 	psf_fseek (psf, psf->dataoffset, SEEK_SET) ;
 
-	/*
+	if (psf->is_pipe == 0)
+	{	/*
 	** Check for 'wvpk' at the start of the DATA section. Not able to
 	** handle this.
 	*/
 	psf_binheader_readf (psf, "e4", &marker) ;
-	if (marker == wvpk_MARKER)
+		if (marker == wvpk_MARKER || marker == OggS_MARKER)
 		return SFE_WAV_WVPK_DATA ;
+		} ;
 
 	/* Seek to start of DATA section. */
 	psf_fseek (psf, psf->dataoffset, SEEK_SET) ;
@@ -616,6 +630,7 @@ wav_write_header (SF_PRIVATE *psf, int calc_length)
 					add_fact_chunk = SF_TRUE ;
 					break ;
 
+		/* Lite remove start */
 		case SF_FORMAT_IMA_ADPCM :
 					{	int blockalign, framesperblock, bytespersec ;
 
@@ -662,6 +677,7 @@ wav_write_header (SF_PRIVATE *psf, int calc_length)
 
 					add_fact_chunk = SF_TRUE ;
 					break ;
+		/* Lite remove end */
 
 		case SF_FORMAT_GSM610 :
 					{	int	blockalign, framesperblock, bytespersec ;
@@ -945,7 +961,7 @@ wav_subchunk_parse (SF_PRIVATE *psf, int chunk)
 					break ;
 			} ;
 
-		if (psf->logindex >= sizeof (psf->logbuffer) - 2)
+		if (psf->logindex >= SIGNED_SIZEOF (psf->logbuffer) - 2)
 			return SFE_LOG_OVERRUN ;
 		} ;
 
@@ -1094,3 +1110,10 @@ wav_read_acid_chunk (SF_PRIVATE *psf, unsigned int chunklen)
 
 	return 0 ;
 } /* wav_read_acid_chunk */
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: 9c551689-a1d8-4905-9f56-26a204374f18
+*/
