@@ -17,7 +17,6 @@
 */
 
 #include	<stdio.h>
-#include	<unistd.h>
 #include	<fcntl.h>
 #include	<string.h>
 #include	<ctype.h>
@@ -116,3 +115,10 @@ wve_read_header (SF_PRIVATE *psf)
 */
 
 #endif
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: ba368cb5-523f-45e4-98c1-5b99a102f73f
+*/
