@@ -16,14 +16,15 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
+#include "config.h"
+
 #include	<stdio.h>
-#include	<unistd.h>
+#include <stdlib.h>
 #include	<fcntl.h>
 #include	<string.h>
 #include	<ctype.h>
 
 #include	"sndfile.h"
-#include	"config.h"
 #include	"sfendian.h"
 #include	"common.h"
 #include	"float_cast.h"
@@ -64,6 +65,9 @@ xi_open	(SF_PRIVATE *psf)
 {	XI_PRIVATE *pxi ;
 	int		subformat, error = 0 ;
 
+	if (psf->is_pipe)
+		return SFE_XI_NO_PIPE ;
+
 	if (psf->fdata)
 		pxi = psf->fdata ;
 	else if ((pxi = calloc (1, sizeof (XI_PRIVATE))) == NULL)
@@ -199,7 +203,6 @@ dpcm_init (SF_PRIVATE *psf)
 				psf_log_printf (psf, "dpcm_init() returning SFE_UNIMPLEMENTED\n") ;
 				return SFE_UNIMPLEMENTED ;
 			} ;
-
 		} ;
 
 	psf->filelength = psf_get_filelen (psf) ;
@@ -208,7 +211,7 @@ dpcm_init (SF_PRIVATE *psf)
 	psf->sf.frames  = psf->datalength / psf->blockwidth ;
 
 	return 0 ;
-} /* dpcm_int */
+} /* dpcm_init */
 
 /*==============================================================================
 */
@@ -219,7 +222,7 @@ dpcm_seek   (SF_PRIVATE *psf, int mode, sf_count_t offset)
 	int			total, bufferlen, len ;
 	
 	if ((pxi = psf->fdata) == NULL)
-		return SFE_XI_WHOOPS ;
+		return SFE_INTERNAL ;
 
 	if (psf->datalength < 0 || psf->dataoffset < 0)
 	{	psf->error = SFE_BAD_SEEK ;
@@ -270,10 +273,10 @@ static int
 xi_write_header (SF_PRIVATE *psf, int calc_length)
 {	XI_PRIVATE	*pxi ;
 	sf_count_t	current ;
-	char		*string ;
+	const char	*string ;
 
 	if ((pxi = psf->fdata) == NULL)
-		return SFE_XI_WHOOPS ;
+		return SFE_INTERNAL ;
 
 	calc_length = calc_length ; /* Avoid a compiler warning. */
 
@@ -343,7 +346,6 @@ xi_read_header (SF_PRIVATE *psf)
 	memset (sample_sizes, 0, sizeof (sample_sizes)) ;
 
 	buffer [20] = 0 ;
-
 	if (strcmp (buffer, "Extended Instrument:") != 0)
 		return SFE_XI_BAD_HEADER ;
 
@@ -388,7 +390,7 @@ xi_read_header (SF_PRIVATE *psf)
 		return SFE_XI_EXCESS_SAMPLES ;
 
 	/* Log all data for each sample. */
-	for (k = 0; k < sample_count; k++)
+	for (k = 0 ; k < sample_count ; k++)
 	{	psf_binheader_readf (psf, "e444", &(sample_sizes [k]), &loop_begin, &loop_end) ;
 
 		/* Read 5 know bytes, 1 unknown byte and 22 name bytes. */
@@ -1167,3 +1169,10 @@ d2dles_array (XI_PRIVATE *pxi, double *src, short *dest, int count, double normf
 	pxi->last_16 = last_val ;
 } /* d2dles_array */
 
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: 1ab2dbe0-29af-4d80-9c6f-cb21b67521bc
+*/
