@@ -16,10 +16,14 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-
+#include "config.h"
 
 #include	<stdio.h>
+#include <stdlib.h>
+
+#if HAVE_UNISTD_H
 #include	<unistd.h>
+#endif
 
 #include	<sndfile.h>
 
@@ -37,7 +41,7 @@ int
 main (void)
 {	SNDFILE	*file ;
 	SF_INFO sfinfo ;
-	char	*filename ;
+	const char	*filename ;
 	int		k ;
 	
 	filename = "test.raw" ;
@@ -165,25 +169,27 @@ main (void)
 
 static
 unsigned char alaw_encode (int sample)
-{	static int exp_lut[128] = {1,1,2,2,3,3,3,3,
-                               4,4,4,4,4,4,4,4,
-                               5,5,5,5,5,5,5,5,
-                               5,5,5,5,5,5,5,5,
-                               6,6,6,6,6,6,6,6,
-                               6,6,6,6,6,6,6,6,
-                               6,6,6,6,6,6,6,6,
-                               6,6,6,6,6,6,6,6,
-                               7,7,7,7,7,7,7,7,
-                               7,7,7,7,7,7,7,7,
-                               7,7,7,7,7,7,7,7,
-                               7,7,7,7,7,7,7,7,
-                               7,7,7,7,7,7,7,7,
-                               7,7,7,7,7,7,7,7,
-                               7,7,7,7,7,7,7,7,
-                               7,7,7,7,7,7,7,7};
+{	static int exp_lut [128] =
+	{	1, 1, 2, 2, 3, 3, 3, 3,
+		4, 4, 4, 4, 4, 4, 4, 4,
+		5, 5, 5, 5, 5, 5, 5, 5,
+		5, 5, 5, 5, 5, 5, 5, 5,
+		6, 6, 6, 6, 6, 6, 6, 6,
+		6, 6, 6, 6, 6, 6, 6, 6,
+		6, 6, 6, 6, 6, 6, 6, 6,
+		6, 6, 6, 6, 6, 6, 6, 6,
+		7, 7, 7, 7, 7, 7, 7, 7,
+		7, 7, 7, 7, 7, 7, 7, 7,
+		7, 7, 7, 7, 7, 7, 7, 7,
+		7, 7, 7, 7, 7, 7, 7, 7,
+		7, 7, 7, 7, 7, 7, 7, 7,
+		7, 7, 7, 7, 7, 7, 7, 7,
+		7, 7, 7, 7, 7, 7, 7, 7,
+		7, 7, 7, 7, 7, 7, 7, 7
+		} ;
 
-    int sign, exponent, mantissa;
-    unsigned char Alawbyte;
+    int sign, exponent, mantissa ;
+    unsigned char Alawbyte ;
 
     /* Get the sample into sign-magnitude. */
     sign = ((~sample) >> 8) & 0x80 ;			/* set aside the sign */
@@ -208,16 +214,16 @@ unsigned char alaw_encode (int sample)
 
 static
 int alaw_decode (unsigned int Alawbyte)
-{	static int exp_lut[8] = { 0, 264, 528, 1056, 2112, 4224, 8448, 16896 } ;
+{	static int exp_lut [8] = { 0, 264, 528, 1056, 2112, 4224, 8448, 16896 } ;
     int sign, exponent, mantissa, sample ;
 
     Alawbyte ^= 0x55 ;
-    sign = ( Alawbyte & 0x80 ) ;
+    sign = (Alawbyte & 0x80) ;
     Alawbyte &= 0x7f ;			/* get magnitude */
     if (Alawbyte >= 16)
 	{	exponent = (Alawbyte >> 4 ) & 0x07 ;
 		mantissa = Alawbyte & 0x0F ;
-		sample = exp_lut[exponent] + (mantissa << ( exponent + 3 )) ;
+		sample = exp_lut [exponent] + (mantissa << ( exponent + 3 )) ;
 		}
     else
 		sample = (Alawbyte << 4) + 8 ;
@@ -235,3 +241,10 @@ int alaw_decode (unsigned int Alawbyte)
 
 
 
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: d3fb5eae-b3a4-4c5b-90ab-e3daa64c4a57
+*/
