@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2001-2003 Erik de Castro Lopo <erikd@zip.com.au>
+** Copyright (C) 2001-2002 Erik de Castro Lopo <erikd@zip.com.au>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -16,11 +16,16 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-
+#include "config.h"
 
 #include	<stdio.h>
+#include <stdlib.h>
 #include	<string.h>
+
+#ifdef HAVE_UNISTD_H
 #include	<unistd.h>
+#endif
+
 #include	<math.h>
 
 #include	<sndfile.h>
@@ -30,11 +35,11 @@
 #define	BUFFER_LEN		(1<<10)
 #define LOG_BUFFER_SIZE	1024
 
-static	void	float_norm_test		(char *filename) ;
-static	void	double_norm_test	(char *filename) ;
+static	void	float_norm_test		(const char *filename) ;
+static	void	double_norm_test	(const char *filename) ;
 static	void	format_tests		(void) ;
-static	void	calc_peak_test		(int filetype, char *filename) ;
-static	void	truncate_test		(char *filename, int filetype) ;
+static	void	calc_peak_test		(int filetype, const char *filename) ;
+static	void	truncate_test		(const char *filename, int filetype) ;
 
 /* Force the start of this buffer to be double aligned. Sparc-solaris will
 ** choke if its not.
@@ -44,7 +49,7 @@ static	float	float_data  [BUFFER_LEN] ;
 static	double	double_data [BUFFER_LEN] ;
 
 int
-main (int argc, char *argv[])
+main (int argc, char *argv [])
 {	/*-char	*filename ;-*/
 	int		do_all = 0 ;
 	int		test_count = 0 ;
@@ -62,7 +67,7 @@ main (int argc, char *argv[])
 		exit (1) ;
 		} ;
 
-	do_all =! strcmp (argv [1], "all");
+	do_all =! strcmp (argv [1], "all") ;
 
 	if (do_all || ! strcmp (argv [1], "ver"))
 	{	char buffer [128] ;
@@ -96,7 +101,7 @@ main (int argc, char *argv[])
 		} ;
 
 	if (do_all || ! strcmp (argv [1], "trunc"))
-	{	truncate_test ("truncate.raw", SF_FORMAT_RAW | SF_FORMAT_DOUBLE) ;
+	{	truncate_test ("truncate.raw", SF_FORMAT_RAW | SF_FORMAT_PCM_32) ;
 		truncate_test ("truncate.au" , SF_FORMAT_AU | SF_FORMAT_PCM_16) ;
 		test_count++ ;
 		} ;
@@ -108,7 +113,7 @@ main (int argc, char *argv[])
 		return 1 ;
 		} ;
 
-	return 0;
+	return 0 ;
 } /* main */
 
 /*============================================================================================
@@ -116,7 +121,7 @@ main (int argc, char *argv[])
 */
 
 static void
-float_norm_test (char *filename)
+float_norm_test (const char *filename)
 {	SNDFILE			*file ;
 	SF_INFO			sfinfo ;
 	unsigned int	k ;
@@ -232,7 +237,7 @@ float_norm_test (char *filename)
 } /* float_norm_test */
 
 static void
-double_norm_test (char *filename)
+double_norm_test (const char *filename)
 {	SNDFILE			*file ;
 	SF_INFO			sfinfo ;
 	unsigned int	k ;
@@ -444,7 +449,7 @@ format_tests	(void)
 } /* format_tests */
 
 static	void
-calc_peak_test (int filetype, char *filename)
+calc_peak_test (int filetype, const char *filename)
 {	SNDFILE		*file ;
 	SF_INFO		sfinfo ;
 	int			k, format ;
@@ -553,10 +558,11 @@ calc_peak_test (int filetype, char *filename)
 } /* calc_peak_test */
 
 static void
-truncate_test (char *filename, int filetype)
+truncate_test (const char *filename, int filetype)
 {	SNDFILE 	*file ;
 	SF_INFO		sfinfo ;
 	sf_count_t	len ;
+	int			*int_data ;
 
 	print_test_name ("truncate_test", filename) ;
 
@@ -564,9 +570,11 @@ truncate_test (char *filename, int filetype)
 	sfinfo.format 	  = filetype ;
 	sfinfo.channels   = 2 ;
 
+	int_data = (int*) double_data ;
+
 	file = test_open_file_or_die (filename, SFM_RDWR, &sfinfo, __LINE__) ;
 
-	test_write_double_or_die (file, 0, double_data, BUFFER_LEN, __LINE__) ;
+	test_write_int_or_die (file, 0, int_data, BUFFER_LEN, __LINE__) ;
 
 	len = 100 ;
 	if (sf_command (file, SFC_FILE_TRUNCATE, &len, sizeof (len)))
@@ -582,3 +590,5 @@ truncate_test (char *filename, int filetype)
 	unlink (filename) ;
 	puts ("ok") ;
 } /* truncate_test */
+
+
