@@ -17,6 +17,7 @@
 */
 
 #include	<stdio.h>
+#include	<stdlib.h>
 #include	<string.h>
 #include	<unistd.h>
 #include	<math.h>
@@ -32,7 +33,7 @@
 #define		M_PI		3.14159265358979323846264338
 #endif
 
-static	void	lcomp_test_int		(char *str, char *filename, int filetype, double margin) ;
+static	void	lcomp_test_int	(const char *str, const char *filename, int filetype, double margin) ;
 
 static	int		error_function (double data, double orig, double margin) ;
 static	int		decay_response (int k) ;
@@ -47,13 +48,11 @@ static	double	orig_buffer [BUFFER_SIZE + 1] ;
 
 int		
 main (void)
-{	char	*filename ;
-
-	filename = "test.au" ;
+{	const char	*filename = "test.au" ;
 	
 	lcomp_test_int		("au_g721", filename, SF_ENDIAN_BIG    | SF_FORMAT_AU | SF_FORMAT_G721_32, 0.06) ;
 	
-	return 0;
+	return 0 ;
 } /* main */
 
 /*============================================================================================
@@ -61,7 +60,7 @@ main (void)
 */ 
  
 static void	
-lcomp_test_int (char *str, char *filename, int filetype, double margin)
+lcomp_test_int (const char *str, const char *filename, int filetype, double margin)
 {	SNDFILE		*file ;
 	SF_INFO		sfinfo ;
 	int			k, m, *orig, *data, sum_abs ;
@@ -292,8 +291,8 @@ gen_signal_double (double *data, double scale, int datalen)
 		else if (k > datalen - ramplen)
 			amp = scale * (datalen - k) / ((double) ramplen) ;
 
-		data [k] = amp * (0.4 * sin (33.3 * 2.0 * M_PI * ((double) (k+1)) / ((double) SAMPLE_RATE))
-							+ 0.3 * cos (201.1 * 2.0 * M_PI * ((double) (k+1)) / ((double) SAMPLE_RATE))) ;
+		data [k] = amp * (0.4 * sin (33.3 * 2.0 * M_PI * ((double) (k + 1)) / ((double) SAMPLE_RATE))
+							+ 0.3 * cos (201.1 * 2.0 * M_PI * ((double) (k + 1)) / ((double) SAMPLE_RATE))) ;
 		} ;
 	
 	return ;
@@ -304,7 +303,7 @@ error_function (double data, double orig, double margin)
 {	double error ;
 
 	if (fabs (orig) <= 500.0)
-		error = fabs (fabs (data) - fabs(orig)) / 2000.0 ;
+		error = fabs (fabs (data) - fabs (orig)) / 2000.0 ;
 	else if (fabs (orig) <= 1000.0)
 		error = fabs (data - orig) / 3000.0 ;
 	else
@@ -317,3 +316,10 @@ error_function (double data, double orig, double margin)
 	return 0 ;
 } /* error_function */
 
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: 368e2777-2848-4e16-a77f-4db841377c73
+*/
