@@ -16,24 +16,28 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
+#include "config.h"
+
 #include	<stdio.h>
+#include <stdlib.h>
 #include	<string.h>
-#include	<unistd.h>
 #include	<math.h>
 
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #include	<sndfile.h>
 
 #include	"dft_cmp.h"
 #include	"utils.h"
 
-/* Configureation header from ../src or ../Win32. */
-#include	<config.h>
-#include	<float_cast.h>
+#include "float_cast.h"
 
 #define	SAMPLE_RATE			11025
 
-static void	float_scaled_test	(char *filename, int allow_exit, int replace_float, int filetype, double target_snr) ;
-static void	double_scaled_test	(char *filename, int allow_exit, int replace_float, int filetype, double target_snr) ;
+static void	float_scaled_test	(const char *filename, int allow_exit, int replace_float, int filetype, double target_snr) ;
+static void	double_scaled_test	(const char *filename, int allow_exit, int replace_float, int filetype, double target_snr) ;
 
 static	double	orig_data [DFT_DATA_LENGTH] ;
 static	double	test_data [DFT_DATA_LENGTH] ;
@@ -86,9 +90,12 @@ main (int argc, char *argv [])
 	float_scaled_test	("adpcm.vox", allow_exit, SF_FALSE, SF_FORMAT_RAW | SF_FORMAT_VOX_ADPCM, -40.0) ;
 
 	float_scaled_test	("dpcm_16.xi", allow_exit, SF_FALSE, SF_FORMAT_XI | SF_FORMAT_DPCM_16, -90.0) ;
-	
 	float_scaled_test	("dpcm_8.xi" , allow_exit, SF_FALSE, SF_FORMAT_XI | SF_FORMAT_DPCM_8 , -41.0) ;
 
+	float_scaled_test	("pcm_s8.sds", allow_exit, SF_FALSE, SF_FORMAT_SDS | SF_FORMAT_PCM_S8, -90.0) ;
+	float_scaled_test	("pcm_16.sds", allow_exit, SF_FALSE, SF_FORMAT_SDS | SF_FORMAT_PCM_16, -140.0) ;
+	float_scaled_test	("pcm_24.sds", allow_exit, SF_FALSE, SF_FORMAT_SDS | SF_FORMAT_PCM_24, -170.0) ;
+
 	float_scaled_test	("replace_float.raw", allow_exit, SF_TRUE, SF_ENDIAN_LITTLE | SF_FORMAT_RAW  | SF_FORMAT_FLOAT, -163.0) ;
 
 	/*==============================================================================
@@ -129,9 +136,13 @@ main (int argc, char *argv [])
 	double_scaled_test	("dpcm_16.xi", allow_exit, SF_FALSE, SF_FORMAT_XI | SF_FORMAT_DPCM_16, -90.0) ;
 	double_scaled_test	("dpcm_8.xi" , allow_exit, SF_FALSE, SF_FORMAT_XI | SF_FORMAT_DPCM_8 , -42.0) ;
 
+	double_scaled_test	("pcm_s8.sds", allow_exit, SF_FALSE, SF_FORMAT_SDS | SF_FORMAT_PCM_S8, -90.0) ;
+	double_scaled_test	("pcm_16.sds", allow_exit, SF_FALSE, SF_FORMAT_SDS | SF_FORMAT_PCM_16, -140.0) ;
+	double_scaled_test	("pcm_24.sds", allow_exit, SF_FALSE, SF_FORMAT_SDS | SF_FORMAT_PCM_24, -180.0) ;
+
 	double_scaled_test	("replace_double.raw", allow_exit, SF_TRUE, SF_FORMAT_RAW | SF_FORMAT_DOUBLE, -300.0) ;
 
-	return 0;
+	return 0 ;
 } /* main */
 
 /*============================================================================================
@@ -139,7 +150,7 @@ main (int argc, char *argv [])
  */ 
 
 static void	
-float_scaled_test (char *filename, int allow_exit, int replace_float, int filetype, double target_snr)
+float_scaled_test (const char *filename, int allow_exit, int replace_float, int filetype, double target_snr)
 {	static	float	float_orig [DFT_DATA_LENGTH] ;
 	static	float	float_test [DFT_DATA_LENGTH] ;
 
@@ -207,7 +218,7 @@ float_scaled_test (char *filename, int allow_exit, int replace_float, int filety
 } /* float_scaled_test */
 
 static void	
-double_scaled_test (char *filename, int allow_exit, int replace_float, int filetype, double target_snr)
+double_scaled_test (const char *filename, int allow_exit, int replace_float, int filetype, double target_snr)
 {	SNDFILE		*file ;
 	SF_INFO		sfinfo ;
 	double		snr ;
@@ -263,3 +274,10 @@ double_scaled_test (char *filename, int allow_exit, int replace_float, int filet
 
 	return ;			
 } /* double_scaled_test */
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: c1043a35-f0aa-44af-9f7f-f193c67f140d
+*/
