@@ -16,11 +16,17 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
+#include "config.h"
+
 #include	<stdio.h>
+#include <stdlib.h>
 #include	<string.h>
-#include	<unistd.h>
 #include	<math.h>
 
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #include	<sndfile.h>
 
 #include	"utils.h"
@@ -32,15 +38,15 @@
 #define		M_PI		3.14159265358979323846264338
 #endif
 
-static	void	lcomp_test_short	(char *filename, int filetype, int chan, double margin) ;
-static	void	lcomp_test_int		(char *filename, int filetype, int chan, double margin) ;
-static	void	lcomp_test_float	(char *filename, int filetype, int chan, double margin) ;
-static	void	lcomp_test_double	(char *filename, int filetype, int chan, double margin) ;
+static	void	lcomp_test_short	(const char *filename, int filetype, int chan, double margin) ;
+static	void	lcomp_test_int		(const char *filename, int filetype, int chan, double margin) ;
+static	void	lcomp_test_float	(const char *filename, int filetype, int chan, double margin) ;
+static	void	lcomp_test_double	(const char *filename, int filetype, int chan, double margin) ;
 
-static	void	sdlcomp_test_short	(char *filename, int filetype, int chan, double margin) ;
-static	void	sdlcomp_test_int	(char *filename, int filetype, int chan, double margin) ;
-static	void	sdlcomp_test_float	(char *filename, int filetype, int chan, double margin) ;
-static	void	sdlcomp_test_double	(char *filename, int filetype, int chan, double margin) ;
+static	void	sdlcomp_test_short	(const char *filename, int filetype, int chan, double margin) ;
+static	void	sdlcomp_test_int	(const char *filename, int filetype, int chan, double margin) ;
+static	void	sdlcomp_test_float	(const char *filename, int filetype, int chan, double margin) ;
+static	void	sdlcomp_test_double	(const char *filename, int filetype, int chan, double margin) ;
 
 static	int		error_function (double data, double orig, double margin) ;
 static	int		decay_response (int k) ;
@@ -60,7 +66,7 @@ static	double	orig_buffer [BUFFER_SIZE + 1] ;
 static	double	smooth_buffer [BUFFER_SIZE + 1] ;
 
 int		
-main (int argc, char *argv[])
+main (int argc, char *argv [])
 {	int		do_all = 0 ;
 	int		test_count = 0 ;
 
@@ -82,13 +88,15 @@ main (int argc, char *argv[])
 	{	/* This is just a sanity test for PCM encoding. */
 		lcomp_test_short	("pcm.wav", SF_FORMAT_WAV | SF_FORMAT_PCM_16, 2, 0.00001) ;
 		lcomp_test_int		("pcm.wav", SF_FORMAT_WAV | SF_FORMAT_PCM_16, 2, 0.00001) ;
+		/* Lite remove start */
 		lcomp_test_float	("pcm.wav", SF_FORMAT_WAV | SF_FORMAT_PCM_16, 2, 0.005) ;
 		lcomp_test_double	("pcm.wav", SF_FORMAT_WAV | SF_FORMAT_PCM_16, 2, 0.005) ;
+		/* Lite remove end */
 		test_count++ ;
 		} ;
 		
 	/* For all the rest, if the file format supports more than 1 channel, use stereo. */
-
+	/* Lite remove start */
 	if (do_all || ! strcmp (argv [1], "wav_ima"))
 	{	lcomp_test_short	("ima.wav", SF_FORMAT_WAV | SF_FORMAT_IMA_ADPCM, 2, 0.18) ;
 		lcomp_test_int		("ima.wav", SF_FORMAT_WAV | SF_FORMAT_IMA_ADPCM, 2, 0.18) ;
@@ -115,20 +123,25 @@ main (int argc, char *argv[])
 
 		test_count++ ;
 		} ;
+	/* Lite remove end */
 
 	if (do_all || ! strcmp (argv [1], "wav_ulaw"))
 	{	lcomp_test_short	("ulaw.wav", SF_FORMAT_WAV | SF_FORMAT_ULAW, 2, 0.04) ;
 		lcomp_test_int		("ulaw.wav", SF_FORMAT_WAV | SF_FORMAT_ULAW, 2, 0.04) ;
+		/* Lite remove start */
 		lcomp_test_float	("ulaw.wav", SF_FORMAT_WAV | SF_FORMAT_ULAW, 2, 0.04) ;
 		lcomp_test_double	("ulaw.wav", SF_FORMAT_WAV | SF_FORMAT_ULAW, 2, 0.04) ;
+		/* Lite remove end */
 		test_count++ ;
 		} ;
 
 	if (do_all || ! strcmp (argv [1], "wav_alaw"))
 	{	lcomp_test_short	("alaw.wav", SF_FORMAT_WAV | SF_FORMAT_ALAW, 2, 0.04) ;
 		lcomp_test_int		("alaw.wav", SF_FORMAT_WAV | SF_FORMAT_ALAW, 2, 0.04) ;
+		/* Lite remove start */
 		lcomp_test_float	("alaw.wav", SF_FORMAT_WAV | SF_FORMAT_ALAW, 2, 0.04) ;
 		lcomp_test_double	("alaw.wav", SF_FORMAT_WAV | SF_FORMAT_ALAW, 2, 0.04) ;
+		/* Lite remove end */
 		test_count++ ;
 		} ;
 
@@ -136,24 +149,30 @@ main (int argc, char *argv[])
 	{	/* Don't do lcomp_test_XXX as the errors are too big. */
 		sdlcomp_test_short	("gsm610.wav", SF_FORMAT_WAV | SF_FORMAT_GSM610, 1, 0.24) ;
 		sdlcomp_test_int	("gsm610.wav", SF_FORMAT_WAV | SF_FORMAT_GSM610, 1, 0.24) ;
+		/* Lite remove start */
 		sdlcomp_test_float	("gsm610.wav", SF_FORMAT_WAV | SF_FORMAT_GSM610, 1, 0.24) ;
 		sdlcomp_test_double	("gsm610.wav", SF_FORMAT_WAV | SF_FORMAT_GSM610, 1, 0.24) ;
+		/* Lite remove end */
 		test_count++ ;
 		} ;
 
 	if (do_all || ! strcmp (argv [1], "aiff_ulaw"))
 	{	lcomp_test_short	("ulaw.aiff", SF_FORMAT_AIFF | SF_FORMAT_ULAW, 2, 0.04) ;
 		lcomp_test_int		("ulaw.aiff", SF_FORMAT_AIFF | SF_FORMAT_ULAW, 2, 0.04) ;
+		/* Lite remove start */
 		lcomp_test_float	("ulaw.aiff", SF_FORMAT_AIFF | SF_FORMAT_ULAW, 2, 0.04) ;
 		lcomp_test_double	("ulaw.aiff", SF_FORMAT_AIFF | SF_FORMAT_ULAW, 2, 0.04) ;
+		/* Lite remove end */
 		test_count++ ;
 		} ;
 
 	if (do_all || ! strcmp (argv [1], "aiff_alaw"))
 	{	lcomp_test_short	("alaw.aiff", SF_FORMAT_AIFF | SF_FORMAT_ALAW, 2, 0.04) ;
 		lcomp_test_int		("alaw.aiff", SF_FORMAT_AIFF | SF_FORMAT_ALAW, 2, 0.04) ;
+		/* Lite remove start */
 		lcomp_test_float	("alaw.aiff", SF_FORMAT_AIFF | SF_FORMAT_ALAW, 2, 0.04) ;
 		lcomp_test_double	("alaw.aiff", SF_FORMAT_AIFF | SF_FORMAT_ALAW, 2, 0.04) ;
+		/* Lite remove end */
 		test_count++ ;
 		} ;
 	
@@ -161,27 +180,41 @@ main (int argc, char *argv[])
 	{	/* Don't do lcomp_test_XXX as the errors are too big. */
 		sdlcomp_test_short	("gsm610.aiff", SF_FORMAT_AIFF | SF_FORMAT_GSM610, 1, 0.24) ;
 		sdlcomp_test_int	("gsm610.aiff", SF_FORMAT_AIFF | SF_FORMAT_GSM610, 1, 0.24) ;
+		/* Lite remove start */
 		sdlcomp_test_float	("gsm610.aiff", SF_FORMAT_AIFF | SF_FORMAT_GSM610, 1, 0.24) ;
 		sdlcomp_test_double	("gsm610.aiff", SF_FORMAT_AIFF | SF_FORMAT_GSM610, 1, 0.24) ;
+		/* Lite remove end */
 		test_count++ ;
 		} ;
 
+	if (do_all || ! strcmp (argv [1], "aiff_ima"))
+	{	lcomp_test_short	("ima.aiff", SF_FORMAT_AIFF | SF_FORMAT_IMA_ADPCM, 2, 0.18) ;
+		lcomp_test_int		("ima.aiff", SF_FORMAT_AIFF | SF_FORMAT_IMA_ADPCM, 2, 0.18) ;
+		lcomp_test_float	("ima.aiff", SF_FORMAT_AIFF | SF_FORMAT_IMA_ADPCM, 2, 0.18) ;
+		lcomp_test_double	("ima.aiff", SF_FORMAT_AIFF | SF_FORMAT_IMA_ADPCM, 2, 0.18) ;
+		} ;
+
 	if (do_all || ! strcmp (argv [1], "au_ulaw"))
 	{	lcomp_test_short	("ulaw.au", SF_ENDIAN_BIG    | SF_FORMAT_AU | SF_FORMAT_ULAW, 2, 0.04) ;
 		lcomp_test_int		("ulaw.au", SF_ENDIAN_LITTLE | SF_FORMAT_AU | SF_FORMAT_ULAW, 2, 0.04) ;
+		/* Lite remove start */
 		lcomp_test_float	("ulaw.au", SF_ENDIAN_LITTLE | SF_FORMAT_AU | SF_FORMAT_ULAW, 2, 0.04) ;
 		lcomp_test_double	("ulaw.au", SF_ENDIAN_BIG    | SF_FORMAT_AU | SF_FORMAT_ULAW, 2, 0.04) ;
+		/* Lite remove end */
 		test_count++ ;
 		} ;
 
 	if (do_all || ! strcmp (argv [1], "au_alaw"))
 	{	lcomp_test_short	("alaw.au", SF_ENDIAN_LITTLE | SF_FORMAT_AU | SF_FORMAT_ALAW, 2, 0.04) ;
 		lcomp_test_int		("alaw.au", SF_ENDIAN_BIG    | SF_FORMAT_AU | SF_FORMAT_ALAW, 2, 0.04) ;
+		/* Lite remove start */
 		lcomp_test_float	("alaw.au", SF_ENDIAN_BIG    | SF_FORMAT_AU | SF_FORMAT_ALAW, 2, 0.04) ;
 		lcomp_test_double	("alaw.au", SF_ENDIAN_LITTLE | SF_FORMAT_AU | SF_FORMAT_ALAW, 2, 0.04) ;
+		/* Lite remove end */
 		test_count++ ;
 		} ;
 
+	/* Lite remove start */
 	if (do_all || ! strcmp (argv [1], "au_g721"))
 	{	printf ("**** Fix this later : error bound should be 0.06 ****\n") ;
 		lcomp_test_short	("g721.au", SF_ENDIAN_LITTLE | SF_FORMAT_AU | SF_FORMAT_G721_32, 1, 0.7) ;
@@ -216,20 +249,25 @@ main (int argc, char *argv[])
 -*/
 		test_count++ ;
 		} ;
+	/* Lite remove end */
 
 	if (do_all || ! strcmp (argv [1], "raw_ulaw"))
 	{	lcomp_test_short	("ulaw.raw", SF_ENDIAN_LITTLE | SF_FORMAT_RAW | SF_FORMAT_ULAW, 2, 0.04) ;
 		lcomp_test_int		("ulaw.raw", SF_ENDIAN_BIG    | SF_FORMAT_RAW | SF_FORMAT_ULAW, 2, 0.04) ;
+		/* Lite remove start */
 		lcomp_test_float	("ulaw.raw", SF_ENDIAN_LITTLE | SF_FORMAT_RAW | SF_FORMAT_ULAW, 2, 0.04) ;
 		lcomp_test_double	("ulaw.raw", SF_ENDIAN_BIG    | SF_FORMAT_RAW | SF_FORMAT_ULAW, 2, 0.04) ;
+		/* Lite remove end */
 		test_count++ ;
 		} ;
 
 	if (do_all || ! strcmp (argv [1], "raw_alaw"))
 	{	lcomp_test_short	("alaw.raw", SF_ENDIAN_LITTLE | SF_FORMAT_RAW | SF_FORMAT_ALAW, 2, 0.04) ;
 		lcomp_test_int		("alaw.raw", SF_ENDIAN_BIG    | SF_FORMAT_RAW | SF_FORMAT_ALAW, 2, 0.04) ;
+		/* Lite remove start */
 		lcomp_test_float	("alaw.raw", SF_ENDIAN_LITTLE | SF_FORMAT_RAW | SF_FORMAT_ALAW, 2, 0.04) ;
 		lcomp_test_double	("alaw.raw", SF_ENDIAN_BIG    | SF_FORMAT_RAW | SF_FORMAT_ALAW, 2, 0.04) ;
+		/* Lite remove end */
 		test_count++ ;
 		} ;
 
@@ -243,6 +281,7 @@ main (int argc, char *argv[])
 		} ;
 -*/
 
+	/* Lite remove start */
 	if (do_all || ! strcmp (argv [1], "ircam_ulaw"))
 	{	lcomp_test_short	("ulaw.ircam", SF_ENDIAN_LITTLE | SF_FORMAT_IRCAM | SF_FORMAT_ULAW, 2, 0.04) ;
 		lcomp_test_int		("ulaw.ircam", SF_ENDIAN_BIG    | SF_FORMAT_IRCAM | SF_FORMAT_ULAW, 2, 0.04) ;
@@ -290,23 +329,29 @@ main (int argc, char *argv[])
 		lcomp_test_double	("alaw.voc", SF_FORMAT_VOC | SF_FORMAT_ALAW, 2, 0.04) ;
 		test_count++ ;
 		} ;
+	/* Lite remove end */
 
 	if (do_all || ! strcmp (argv [1], "w64_ulaw"))
 	{	lcomp_test_short	("ulaw.w64", SF_FORMAT_W64 | SF_FORMAT_ULAW, 2, 0.04) ;
 		lcomp_test_int		("ulaw.w64", SF_FORMAT_W64 | SF_FORMAT_ULAW, 2, 0.04) ;
+		/* Lite remove start */
 		lcomp_test_float	("ulaw.w64", SF_FORMAT_W64 | SF_FORMAT_ULAW, 2, 0.04) ;
 		lcomp_test_double	("ulaw.w64", SF_FORMAT_W64 | SF_FORMAT_ULAW, 2, 0.04) ;
+		/* Lite remove end */
 		test_count++ ;
 		} ;
 
 	if (do_all || ! strcmp (argv [1], "w64_alaw"))
 	{	lcomp_test_short	("alaw.w64", SF_FORMAT_W64 | SF_FORMAT_ALAW, 2, 0.04) ;
 		lcomp_test_int		("alaw.w64", SF_FORMAT_W64 | SF_FORMAT_ALAW, 2, 0.04) ;
+		/* Lite remove start */
 		lcomp_test_float	("alaw.w64", SF_FORMAT_W64 | SF_FORMAT_ALAW, 2, 0.04) ;
 		lcomp_test_double	("alaw.w64", SF_FORMAT_W64 | SF_FORMAT_ALAW, 2, 0.04) ;
+		/* Lite remove end */
 		test_count++ ;
 		} ;
 
+	/* Lite remove start */
 	if (do_all || ! strcmp (argv [1], "w64_ima"))
 	{	lcomp_test_short	("ima.w64", SF_FORMAT_W64 | SF_FORMAT_IMA_ADPCM, 2, 0.18) ;
 		lcomp_test_int		("ima.w64", SF_FORMAT_W64 | SF_FORMAT_IMA_ADPCM, 2, 0.18) ;
@@ -332,16 +377,20 @@ main (int argc, char *argv[])
 		sdlcomp_test_double	("msadpcm.w64", SF_FORMAT_W64 | SF_FORMAT_MS_ADPCM, 2, 0.36) ;
 		test_count++ ;
 		} ;
+	/* Lite remove end */
 
 	if (do_all || ! strcmp (argv [1], "w64_gsm610"))
 	{	/* Don't do lcomp_test_XXX as the errors are too big. */
 		sdlcomp_test_short	("gsm610.w64", SF_FORMAT_W64 | SF_FORMAT_GSM610, 1, 0.2) ;
 		sdlcomp_test_int	("gsm610.w64", SF_FORMAT_W64 | SF_FORMAT_GSM610, 1, 0.2) ;
+		/* Lite remove start */
 		sdlcomp_test_float	("gsm610.w64", SF_FORMAT_W64 | SF_FORMAT_GSM610, 1, 0.2) ;
 		sdlcomp_test_double	("gsm610.w64", SF_FORMAT_W64 | SF_FORMAT_GSM610, 1, 0.2) ;
+		/* Lite remove end */
 		test_count++ ;
 		} ;
 
+	/* Lite remove start */
 	if (do_all || ! strcmp (argv [1], "vox_adpcm"))
 	{	lcomp_test_short	("adpcm.vox", SF_FORMAT_RAW | SF_FORMAT_VOX_ADPCM, 1, 0.17) ;
 		lcomp_test_int		("adpcm.vox", SF_FORMAT_RAW | SF_FORMAT_VOX_ADPCM, 1, 0.17) ;
@@ -365,6 +414,7 @@ main (int argc, char *argv[])
 		lcomp_test_double	("16bit.xi", SF_FORMAT_XI | SF_FORMAT_DPCM_16, 1, 0.002) ;
 		test_count++ ;
 		} ;
+	/* Lite remove end */
 
 	if (test_count == 0)
 	{	printf ("************************************\n") ;
@@ -373,7 +423,7 @@ main (int argc, char *argv[])
 		return 1 ;
 		} ;
 
-	return 0;
+	return 0 ;
 } /* main */
 
 /*============================================================================================
@@ -381,7 +431,7 @@ main (int argc, char *argv[])
 */ 
  
 static void	
-lcomp_test_short (char *filename, int filetype, int channels, double margin)
+lcomp_test_short (const char *filename, int filetype, int channels, double margin)
 {	SNDFILE			*file ;
 	SF_INFO			sfinfo ;
 	int				k, m, seekpos, sum_abs ;
@@ -434,12 +484,12 @@ lcomp_test_short (char *filename, int filetype, int channels, double margin)
 		} ;
 	
 	if (sfinfo.frames < datalen / channels)
-	{	printf ("Too few.frames in file. (%ld should be a little more than %ld)\n", datalen, SF_COUNT_TO_LONG (sfinfo.frames)) ;
+	{	printf ("Too few frames in file. (%ld should be a little more than %ld)\n", SF_COUNT_TO_LONG (sfinfo.frames), datalen) ;
 		exit (1) ;
 		} ;
 
 	if (sfinfo.frames > (datalen + datalen / 2))
-	{	printf ("Too many.frames in file. (%ld should be a little more than %ld)\n", datalen, SF_COUNT_TO_LONG (sfinfo.frames)) ;
+	{	printf ("Too many frames in file. (%ld should be a little more than %ld)\n", SF_COUNT_TO_LONG (sfinfo.frames), datalen) ;
 		exit (1) ;
 		} ;
 
@@ -587,7 +637,7 @@ oct_save_short (orig, data, datalen) ;
 */ 
  
 static void	
-lcomp_test_int (char *filename, int filetype, int channels, double margin)
+lcomp_test_int (const char *filename, int filetype, int channels, double margin)
 {	SNDFILE		*file ;
 	SF_INFO		sfinfo ;
 	int			k, m, *orig, *data, sum_abs ;
@@ -792,7 +842,7 @@ lcomp_test_int (char *filename, int filetype, int channels, double margin)
 */ 
 
 static void	
-lcomp_test_float (char *filename, int filetype, int channels, double margin)
+lcomp_test_float (const char *filename, int filetype, int channels, double margin)
 {	SNDFILE			*file ;
 	SF_INFO			sfinfo ;
 	int				k, m, seekpos, sum_abs ;
@@ -1001,7 +1051,7 @@ lcomp_test_float (char *filename, int filetype, int channels, double margin)
 */ 
 
 static void	
-lcomp_test_double (char *filename, int filetype, int channels, double margin)
+lcomp_test_double (const char *filename, int filetype, int channels, double margin)
 {	SNDFILE			*file ;
 	SF_INFO			sfinfo ;
 	int				k, m, seekpos, sum_abs ;
@@ -1211,7 +1261,7 @@ lcomp_test_double (char *filename, int filetype, int channels, double margin)
 */
 
 static void	
-sdlcomp_test_short	(char *filename, int filetype, int channels, double margin)
+sdlcomp_test_short	(const char *filename, int filetype, int channels, double margin)
 {	SNDFILE			*file ;
 	SF_INFO			sfinfo ;
 	int				k, m, seekpos, sum_abs ;
@@ -1415,7 +1465,7 @@ channels = 1 ;
 } /* sdlcomp_test_short */
 
 static	void	
-sdlcomp_test_int	(char *filename, int filetype, int channels, double margin)
+sdlcomp_test_int	(const char *filename, int filetype, int channels, double margin)
 {	SNDFILE			*file ;
 	SF_INFO			sfinfo ;
 	int				k, m, seekpos, sum_abs ;
@@ -1623,7 +1673,7 @@ channels = 1 ;
 } /* sdlcomp_test_int */
 
 static void	
-sdlcomp_test_float	(char *filename, int filetype, int channels, double margin)
+sdlcomp_test_float	(const char *filename, int filetype, int channels, double margin)
 {	SNDFILE			*file ;
 	SF_INFO			sfinfo ;
 	int				k, m, seekpos ;
@@ -1834,7 +1884,7 @@ printf ("** fix this ** ") ;
 } /* sdlcomp_test_float */
 
 static void	
-sdlcomp_test_double	(char *filename, int filetype, int channels, double margin)
+sdlcomp_test_double	(const char *filename, int filetype, int channels, double margin)
 {	SNDFILE			*file ;
 	SF_INFO			sfinfo ;
 	int				k, m, seekpos ;
@@ -2095,7 +2145,7 @@ error_function (double data, double orig, double margin)
 {	double error ;
 
 	if (fabs (orig) <= 500.0)
-		error = fabs (fabs (data) - fabs(orig)) / 2000.0 ;
+		error = fabs (fabs (data) - fabs (orig)) / 2000.0 ;
 	else if (fabs (orig) <= 1000.0)
 		error = fabs (data - orig) / 3000.0 ;
 	else
@@ -2163,3 +2213,10 @@ smoothed_diff_double (double *data, unsigned int datalen)
 	data [datalen-1] = data [datalen-2] ;
 	
 } /* smoothed_diff_double */
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch 
+** revision control system.
+**
+** arch-tag: 5eb86888-3311-48b8-920e-c2a0393b7ad2
+*/
