@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2003 Erik de Castro Lopo <erikd@zip.com.au>
+** Copyright (C) 1999-2002 Erik de Castro Lopo <erikd@zip.com.au>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -16,34 +16,44 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
+#include "config.h"
+
 #include	<stdio.h>
+#include <stdlib.h>
 #include	<string.h>
+
+#ifdef HAVE_UNISTD_H
 #include	<unistd.h>
-#include	<math.h>
+#endif
 
 #include	<sndfile.h>
 
+#include "float_cast.h"
 #include	"utils.h"
 
 #define	BUFFER_SIZE		(1<<15)
 
-static void	pcm_test_bits_8  (char *filename, int filetype, int hash) ;
-static void	pcm_test_bits_16 (char *filename, int filetype, int hash) ;
-static void	pcm_test_bits_24 (char *filename, int filetype, int hash) ;
-static void	pcm_test_bits_32 (char *filename, int filetype, int hash) ;
+static void	lrintf_test (void) ;
 
-static void pcm_test_float   (char *filename, int filetype, int hash, int replace_float) ;
-static void pcm_test_double  (char *filename, int filetype, int hash, int replace_float) ;
+static void	pcm_test_bits_8	(const char *filename, int filetype, int hash) ;
+static void	pcm_test_bits_16	(const char *filename, int filetype, int hash) ;
+static void	pcm_test_bits_24	(const char *filename, int filetype, int hash) ;
+static void	pcm_test_bits_32	(const char *filename, int filetype, int hash) ;
+
+static void pcm_test_float	(const char *filename, int filetype, int hash, int replace_float) ;
+static void pcm_test_double	(const char *filename, int filetype, int hash, int replace_float) ;
 
 /* Data written to the file. */
-static	double	data_out [(BUFFER_SIZE/sizeof(double))+1] ;
+static	double	data_out [(BUFFER_SIZE / sizeof (double)) + 1] ;
 
 /* Data read back from the file. */
-static	double	data_in  [(BUFFER_SIZE/sizeof(double))+1] ;
+static	double	data_in	[(BUFFER_SIZE / sizeof (double)) + 1] ;
 
 int
 main (void)
 {
+	lrintf_test () ;
+
 	pcm_test_bits_8  ("pcm-s8.raw", SF_FORMAT_RAW | SF_FORMAT_PCM_S8, 0x9ae33814)  ;
 	pcm_test_bits_8  ("pcm-u8.raw", SF_FORMAT_RAW | SF_FORMAT_PCM_U8, 0x651d4694) ;
 
@@ -56,6 +66,7 @@ main (void)
 	pcm_test_bits_32 ("le-pcm32.raw", SF_ENDIAN_LITTLE | SF_FORMAT_RAW | SF_FORMAT_PCM_32, 0x04c84a70) ;
 	pcm_test_bits_32 ("be-pcm32.raw", SF_ENDIAN_BIG    | SF_FORMAT_RAW | SF_FORMAT_PCM_32, 0x069c84f6) ;
 
+	/* Lite remove start */
 	pcm_test_float   ("le-float.raw", SF_ENDIAN_LITTLE | SF_FORMAT_RAW | SF_FORMAT_FLOAT, 0xbb836603, SF_FALSE) ;
 	pcm_test_float   ("be-float.raw", SF_ENDIAN_BIG    | SF_FORMAT_RAW | SF_FORMAT_FLOAT, 0x903cd8fc, SF_FALSE) ;
 
@@ -69,8 +80,9 @@ main (void)
 
 	pcm_test_double  ("le-double.raw", SF_ENDIAN_LITTLE | SF_FORMAT_RAW | SF_FORMAT_DOUBLE, 0xbf84448e, SF_TRUE) ;
 	pcm_test_double  ("be-double.raw", SF_ENDIAN_BIG    | SF_FORMAT_RAW | SF_FORMAT_DOUBLE, 0xaf3d9fb5, SF_TRUE) ;
+	/* Lite remove end */
 
-	return 0;
+	return 0 ;
 } /* main */
 
 /*============================================================================================
@@ -78,16 +90,46 @@ main (void)
 */
 
 static void
-pcm_test_bits_8  (char *filename, int filetype, int hash)
+lrintf_test (void)
+{	int k, items ;
+	float	*float_data ;
+	int		*int_data ;
+
+	print_test_name ("lrintf_test", "") ;
+
+	items = 1024 ;
+
+	float_data = (float*) data_out ;
+	int_data = (int*) data_in ;
+
+	for (k = 0 ; k < items ; k++)
+		float_data [k] = (k * ((k % 2) ? 333333.0 : -333333.0)) ;
+
+	for (k = 0 ; k < items ; k++)
+		int_data [k] = lrintf (float_data [k]) ;
+
+	for (k = 0 ; k < items ; k++)
+		if (fabs (int_data [k] - float_data [k]) > 1.0)
+		{	printf ("\n\nLine %d: float : Incorrect sample (#%d : %f => %d).\n", __LINE__, k, float_data [k], int_data [k]) ;
+			exit (1) ;
+			} ;
+
+	printf ("ok\n") ;
+} /* lrintf_test */
+
+static void
+pcm_test_bits_8 (const char *filename, int filetype, int hash)
 {	SNDFILE		*file ;
 	SF_INFO		sfinfo ;
 	int			k, items, zero_count ;
 	short		*short_out, *short_in ;
 	int			*int_out, *int_in ;
+	/* Lite remove start */
 	float		*float_out, *float_in ;
 	double		*double_out, *double_in ;
+	/* Lite remove end */
 
-	print_test_name ("pcm_test_bits_8 ", filename) ;
+	print_test_name ("pcm_test_bits_8", filename) ;
 
 	items = 127 ;
 
@@ -209,6 +251,7 @@ pcm_test_bits_8  (char *filename, int filetype, int hash)
 
 	sf_close (file) ;
 
+	/* Lite remove start */
 	/*--------------------------------------------------------------------------
 	** Test sf_read/write_float ()
 	*/
@@ -335,20 +378,23 @@ pcm_test_bits_8  (char *filename, int filetype, int hash)
 			} ;
 
 	sf_close (file) ;
+	/* Lite remove end */
 
 	printf ("ok\n") ;
 	unlink (filename) ;
-} /* pcm_test_bits_8  */
+} /* pcm_test_bits_8 */
 
 static void
-pcm_test_bits_16 (char *filename, int filetype, int hash)
+pcm_test_bits_16 (const char *filename, int filetype, int hash)
 {	SNDFILE		*file ;
 	SF_INFO		sfinfo ;
 	int			k, items, zero_count ;
 	short		*short_out, *short_in ;
 	int			*int_out, *int_in ;
+	/* Lite remove start */
 	float		*float_out, *float_in ;
 	double		*double_out, *double_in ;
+	/* Lite remove end */
 
 	print_test_name ("pcm_test_bits_16", filename) ;
 
@@ -472,6 +518,7 @@ pcm_test_bits_16 (char *filename, int filetype, int hash)
 
 	sf_close (file) ;
 
+	/* Lite remove start */
 	/*--------------------------------------------------------------------------
 	** Test sf_read/write_float ()
 	*/
@@ -598,20 +645,23 @@ pcm_test_bits_16 (char *filename, int filetype, int hash)
 			} ;
 
 	sf_close (file) ;
+	/* Lite remove end */
 
 	printf ("ok\n") ;
 	unlink (filename) ;
 } /* pcm_test_bits_16 */
 
 static void
-pcm_test_bits_24 (char *filename, int filetype, int hash)
+pcm_test_bits_24 (const char *filename, int filetype, int hash)
 {	SNDFILE		*file ;
 	SF_INFO		sfinfo ;
 	int			k, items, zero_count ;
 	short		*short_out, *short_in ;
 	int			*int_out, *int_in ;
+	/* Lite remove start */
 	float		*float_out, *float_in ;
 	double		*double_out, *double_in ;
+	/* Lite remove end */
 
 	print_test_name ("pcm_test_bits_24", filename) ;
 
@@ -735,6 +785,7 @@ pcm_test_bits_24 (char *filename, int filetype, int hash)
 
 	sf_close (file) ;
 
+	/* Lite remove start */
 	/*--------------------------------------------------------------------------
 	** Test sf_read/write_float ()
 	*/
@@ -861,20 +912,23 @@ pcm_test_bits_24 (char *filename, int filetype, int hash)
 			} ;
 
 	sf_close (file) ;
+	/* Lite remove end */
 
 	printf ("ok\n") ;
 	unlink (filename) ;
 } /* pcm_test_bits_24 */
 
 static void
-pcm_test_bits_32 (char *filename, int filetype, int hash)
+pcm_test_bits_32 (const char *filename, int filetype, int hash)
 {	SNDFILE		*file ;
 	SF_INFO		sfinfo ;
 	int			k, items, zero_count ;
 	short		*short_out, *short_in ;
 	int			*int_out, *int_in ;
+	/* Lite remove start */
 	float		*float_out, *float_in ;
 	double		*double_out, *double_in ;
+	/* Lite remove end */
 
 	print_test_name ("pcm_test_bits_32", filename) ;
 
@@ -998,6 +1052,7 @@ pcm_test_bits_32 (char *filename, int filetype, int hash)
 
 	sf_close (file) ;
 
+	/* Lite remove start */
 	/*--------------------------------------------------------------------------
 	** Test sf_read/write_float ()
 	*/
@@ -1124,6 +1179,7 @@ pcm_test_bits_32 (char *filename, int filetype, int hash)
 			} ;
 
 	sf_close (file) ;
+	/* Lite remove end */
 
 	printf ("ok\n") ;
 	unlink (filename) ;
@@ -1135,7 +1191,7 @@ pcm_test_bits_32 (char *filename, int filetype, int hash)
 */
 
 static void
-pcm_test_float (char *filename, int filetype, int hash, int replace_float)
+pcm_test_float (const char *filename, int filetype, int hash, int replace_float)
 {	SNDFILE			*file ;
 	SF_INFO			sfinfo ;
 	int				k, items, frames ;
@@ -1232,7 +1288,7 @@ pcm_test_float (char *filename, int filetype, int hash, int replace_float)
 		if (fabs (data [k]) > 1e-100 && fabs (error / data [k]) > 1e-5)
 		{	printf ("\n\nError (%s:%d) Mono : Incorrect sample (#%d : %f => %f).\n", __FILE__, __LINE__, k, ((double) k) / 100.0, data [k]) ;
 			exit (1) ;
-			};
+			} ;
 		} ;
 
 	/* Seek to offset from current position. */
@@ -1244,11 +1300,11 @@ pcm_test_float (char *filename, int filetype, int hash, int replace_float)
 		if (fabs (data [k]) > 1e-100 && fabs (error / data [k]) > 1e-5)
 		{	printf ("\n\nError (%s:%d) Mono : Incorrect sample (#%d : %f => %f).\n", __FILE__, __LINE__, k, ((double) k) / 100.0, data [k]) ;
 			exit (1) ;
-			};
-		};
+			} ;
+		} ;
 
 	/* Seek to offset from end of file. */
-	test_seek_or_die (file, -(sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
+	test_seek_or_die (file, -1 * (sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
 
 	test_read_double_or_die (file, 0, data + 10, 4, __LINE__) ;
 	for (k = 10 ; k < 14 ; k++)
@@ -1256,7 +1312,7 @@ pcm_test_float (char *filename, int filetype, int hash, int replace_float)
 		if (fabs (data [k]) > 1e-100 && fabs (error / data [k]) > 1e-5)
 		{	printf ("\n\nError (%s:%d) Mono : Incorrect sample (#%d : %f => %f).\n", __FILE__, __LINE__, k, ((double) k) / 100.0, data [k]) ;
 			exit (1) ;
-			};
+			} ;
 		} ;
 
 	sf_close (file) ;
@@ -1355,7 +1411,7 @@ pcm_test_float (char *filename, int filetype, int hash, int replace_float)
 		if (fabs (data [k]) > 1e-100 && fabs (error / data [k]) > 1e-5)
 		{	printf ("\n\nError (%s:%d) Stereo : Incorrect sample (#%d : %f => %f).\n", __FILE__, __LINE__, k, ((double) k) / 100.0, data [k]) ;
 			exit (1) ;
-			};
+			} ;
 		} ;
 
 	/* Seek to offset from current position. */
@@ -1367,11 +1423,11 @@ pcm_test_float (char *filename, int filetype, int hash, int replace_float)
 		if (fabs (data [k]) > 1e-100 && fabs (error / data [k]) > 1e-5)
 		{	printf ("\n\nError (%s:%d) Stereo : Incorrect sample (#%d : %f => %f).\n", __FILE__, __LINE__, k, ((double) k) / 100.0, data [k]) ;
 			exit (1) ;
-			};
+			} ;
 		} ;
 
 	/* Seek to offset from end of file. */
-	test_seek_or_die (file, -(sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
+	test_seek_or_die (file, -1 * (sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
 
 	test_readf_double_or_die (file, 0, data + 20, 2, __LINE__) ;
 	for (k = 20 ; k < 24 ; k++)
@@ -1379,7 +1435,7 @@ pcm_test_float (char *filename, int filetype, int hash, int replace_float)
 		if (fabs (data [k]) > 1e-100 && fabs (error / data [k]) > 1e-5)
 		{	printf ("\n\nError (%s:%d) Stereo : Incorrect sample (#%d : %f => %f).\n", __FILE__, __LINE__, k, ((double) k) / 100.0, data [k]) ;
 			exit (1) ;
-			};
+			} ;
 		} ;
 
 	sf_close (file) ;
@@ -1389,7 +1445,7 @@ pcm_test_float (char *filename, int filetype, int hash, int replace_float)
 } /* pcm_test_float */
 
 static void
-pcm_test_double (char *filename, int	filetype, int hash, int replace_float)
+pcm_test_double (const char *filename, int	filetype, int hash, int replace_float)
 {	SNDFILE			*file ;
 	SF_INFO			sfinfo ;
 	int				k, items, frames ;
@@ -1498,7 +1554,7 @@ pcm_test_double (char *filename, int	filetype, int hash, int replace_float)
 		if (fabs (data [k]) > 1e-100 && fabs (error / data [k]) > 1e-5)
 		{	printf ("\n\nError (%s:%d) Mono : Incorrect sample (#%d : %f => %f).\n", __FILE__, __LINE__, k, ((double) k) / 100.0, data [k]) ;
 			exit (1) ;
-			};
+			} ;
 		} ;
 
 	/* Seek to offset from current position. */
@@ -1510,11 +1566,11 @@ pcm_test_double (char *filename, int	filetype, int hash, int replace_float)
 		if (fabs (data [k]) > 1e-100 && fabs (error / data [k]) > 1e-5)
 		{	printf ("\n\nError (%s:%d) Mono : Incorrect sample (#%d : %f => %f).\n", __FILE__, __LINE__, k, ((double) k) / 100.0, data [k]) ;
 			exit (1) ;
-			};
-		};
+			} ;
+		} ;
 
 	/* Seek to offset from end of file. */
-	test_seek_or_die (file, -(sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
+	test_seek_or_die (file, -1 * (sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
 
 	test_read_double_or_die (file, 0, data + 10, 4, __LINE__) ;
 	for (k = 10 ; k < 14 ; k++)
@@ -1522,7 +1578,7 @@ pcm_test_double (char *filename, int	filetype, int hash, int replace_float)
 		if (fabs (data [k]) > 1e-100 && fabs (error / data [k]) > 1e-5)
 		{	printf ("\n\nError (%s:%d) Mono : Incorrect sample (#%d : %f => %f).\n", __FILE__, __LINE__, k, ((double) k) / 100.0, data [k]) ;
 			exit (1) ;
-			};
+			} ;
 		} ;
 
 	sf_close (file) ;
@@ -1629,7 +1685,7 @@ pcm_test_double (char *filename, int	filetype, int hash, int replace_float)
 		if (fabs (data [k]) > 1e-100 && fabs (error / data [k]) > 1e-5)
 		{	printf ("\n\nError (%s:%d) Stereo : Incorrect sample (#%d : %f => %f).\n", __FILE__, __LINE__, k, ((double) k) / 100.0, data [k]) ;
 			exit (1) ;
-			};
+			} ;
 		} ;
 
 	/* Seek to offset from current position. */
@@ -1641,11 +1697,11 @@ pcm_test_double (char *filename, int	filetype, int hash, int replace_float)
 		if (fabs (data [k]) > 1e-100 && fabs (error / data [k]) > 1e-5)
 		{	printf ("\n\nError (%s:%d) Stereo : Incorrect sample (#%d : %f => %f).\n", __FILE__, __LINE__, k, ((double) k) / 100.0, data [k]) ;
 			exit (1) ;
-			};
+			} ;
 		} ;
 
 	/* Seek to offset from end of file. */
-	test_seek_or_die (file, -(sfinfo.frames -10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
+	test_seek_or_die (file, -1 * (sfinfo.frames -10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
 
 	test_readf_double_or_die (file, 0, data + 20, 4, __LINE__) ;
 	for (k = 20 ; k < 24 ; k++)
@@ -1653,7 +1709,7 @@ pcm_test_double (char *filename, int	filetype, int hash, int replace_float)
 		if (fabs (data [k]) > 1e-100 && fabs (error / data [k]) > 1e-5)
 		{	printf ("\n\nError (%s:%d) Stereo : Incorrect sample (#%d : %f => %f).\n", __FILE__, __LINE__, k, ((double) k) / 100.0, data [k]) ;
 			exit (1) ;
-			};
+			} ;
 		} ;
 
 	sf_close (file) ;
@@ -1664,3 +1720,5 @@ pcm_test_double (char *filename, int	filetype, int hash, int replace_float)
 
 /*==============================================================================
 */
+
+
