@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2002,2003 Erik de Castro Lopo <erikd@zip.com.au>
+** Copyright (C) 2002 Erik de Castro Lopo <erikd@zip.com.au>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,9 +24,24 @@
 */
 
 
+
+#include "config.h"
+
 #include	<stdio.h>
+#include <stdlib.h>
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#if (HAVE_DECL_S_IRGRP == 0)
+#include <sf_unistd.h>
+#endif
+
 #include	<string.h>
 #include	<math.h>
+#include	<fcntl.h>
+#include	<sys/stat.h>
 
 #include	<sndfile.h>
 
@@ -47,7 +62,7 @@ gen_windowed_sine (double *data, int len, double maximum)
 	**	Choose a frequency of 1/32 so that it aligns perfectly with a DFT
 	**	bucket to minimise spreading of energy over more than one bucket.
 	**	Also do not want to make the frequency too high as some of the
-	**	codec (ie gsm610) have a quite severe high frequency roll off.
+	**	codecs (ie gsm610) have a quite severe high frequency roll off.
 	*/
 	len /= 2 ;
 
@@ -55,7 +70,7 @@ gen_windowed_sine (double *data, int len, double maximum)
 	{	data [k] = sin (2.0 * k * M_PI * 1.0 / 32.0 + 0.4) ;
 
 		/* Apply Hanning Window. */
-		data [k] *= maximum * (0.5 - 0.5 * cos (2.0 * M_PI * k / ((len)-1))) ;
+		data [k] *= maximum * (0.5 - 0.5 * cos (2.0 * M_PI * k / ((len) - 1))) ;
 		}
 
 	return ;
@@ -63,7 +78,7 @@ gen_windowed_sine (double *data, int len, double maximum)
 
 
 void
-check_file_hash_or_die  (char *filename, unsigned int target_hash, int line_num)
+check_file_hash_or_die (const char *filename, unsigned int target_hash, int line_num)
 {	static unsigned char	buffer [2048] ;
 	unsigned int	hash1, hash2 ;
 	FILE 	*file ;
@@ -104,7 +119,7 @@ check_file_hash_or_die  (char *filename, unsigned int target_hash, int line_num)
 } /* check_file_hash_or_die */
 
 void
-print_test_name (char *test, const char *filename)
+print_test_name (const char *test, const char *filename)
 {	int count ;
 
 	if (test == NULL || filename == NULL)
@@ -114,7 +129,7 @@ print_test_name (char *test, const char *filename)
 
 	printf ("    %-20s : %s ", test, filename) ;
 
-	count = 20 - strlen (filename) ;
+	count = 24 - strlen (filename) ;
 	while (count -- > 0)
 		putchar ('.') ;
 	putchar (' ') ;
@@ -278,7 +293,7 @@ check_log_buffer_or_die (SNDFILE *file)
 } /* check_log_buffer_or_die */
 
 int
-string_in_log_buffer (SNDFILE *file, char *s)
+string_in_log_buffer (SNDFILE *file, const char *s)
 {	static char	buffer [LOG_BUFFER_SIZE] ;
 	int			count ;
 
@@ -306,28 +321,81 @@ dump_log_buffer (SNDFILE *file)
 	/* Get the log buffer data. */
 	count = sf_command	(file, SFC_GET_LOG_INFO, buffer, LOG_BUFFER_SIZE) ;
 
-	puts (buffer) ;
+	if (strlen (buffer) < 1)
+		puts ("Log buffer empty.\n") ;
+	else
+		puts (buffer) ;
 
 	return ;
 } /* dump_log_buffer */
 
 SNDFILE *
 test_open_file_or_die (const char *filename, int mode, SF_INFO *sfinfo, int line_num)
-{	SNDFILE *file ;
-	char *modestr ;
+{	static int count = 0 ;
 
-	if (! (file = sf_open (filename, mode, sfinfo)))
-	{	switch (mode)
-		{	case SFM_READ :		modestr = "SFM_READ" ; break ;
-			case SFM_WRITE :	modestr = "SFM_WRITE" ; break ;
-			case SFM_RDWR :		modestr = "SFM_RDWR" ; break ;
-			default : modestr = "????" ; break ;
+	SNDFILE *file ;
+	const char *modestr, *func_name ;
+	int oflags = 0, omode = 0 ;
+
+	/*
+	** Need to test both sf_open() and sf_open_fd().
+	** Do so alternately.
+	*/
+
+	switch (mode)
+	{	case SFM_READ :
+				modestr = "SFM_READ" ;
+				oflags = O_RDONLY ;
+				omode = 0 ;
+				break ;
+
+		case SFM_WRITE :
+				modestr = "SFM_WRITE" ; 
+				oflags = O_WRONLY | O_CREAT | O_TRUNC ;
+				omode = S_IRUSR | S_IWUSR | S_IRGRP ;
+				break ;
+
+		case SFM_RDWR :		
+				modestr = "SFM_RDWR" ; 
+				oflags = O_RDWR | O_CREAT ;
+				omode = S_IRUSR | S_IWUSR | S_IRGRP ;
+				break ;
+		default : 
+				printf ("\n\nLine %d: Bad mode.\n", line_num) ;
+				fflush (stdout) ;
+				exit (1) ;
+		} ;
+
+#if (defined (__CYGWIN__) || defined (WIN32) || defined (_WIN32))
+	/* Stupid fscking windows. */
+	oflags |= O_BINARY ;
+#endif
+
+	if (((++count) & 1) == 1)
+	{	int fd ;
+	
+		if (omode == 0)
+			fd = open (filename, oflags) ;
+		else
+			fd = open (filename, oflags, omode) ;
+		
+		if (fd < 0)
+		{	perror ("open") ;
+			exit (1) ;
 			} ;
 		
-		printf ("\n\nLine %d: sf_open (%s) failed : ", line_num, modestr) ;
-		fflush (stdout) ;
-		dump_log_buffer (file) ;
+		func_name = "sf_open_fd" ;
+		file = sf_open_fd (fd, mode, sfinfo, SF_TRUE) ;
+		}
+	else
+	{	func_name = "sf_open" ;
+		file = sf_open (filename, mode, sfinfo) ;
+		} ;
+
+	if (file == NULL)
+	{	printf ("\n\nLine %d: %s (%s) failed : ", line_num, func_name, modestr) ;
 		puts (sf_strerror (NULL)) ;
+		dump_log_buffer (file) ;
 		exit (1) ;
 		} ;
 
@@ -363,7 +431,7 @@ test_read_write_position_or_die (SNDFILE *file, int line_num, int pass, sf_count
 void
 test_seek_or_die (SNDFILE *file, sf_count_t offset, int whence, sf_count_t new_pos, int channels, int line_num)
 {	sf_count_t position ;
-	char		*channel_name, *whence_name ;
+	const char	*channel_name, *whence_name ;
 
 	switch (whence)
 	{	case SEEK_SET :
@@ -710,3 +778,4 @@ test_writef_double_or_die (SNDFILE *file, int pass, double *test, sf_count_t fra
 
 
 
+
