@@ -16,9 +16,16 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
+#include "config.h"
+
 #include	<stdio.h>
+#include <stdlib.h>
 #include	<string.h>
+
+#ifdef HAVE_UNISTD_H
 #include	<unistd.h>
+#endif
+
 #include	<math.h>
 
 #if (defined (WIN32) || defined (_WIN32))
@@ -35,12 +42,12 @@ static int truncate (const char *filename, int ignored) ;
 
 #define	SILLY_WRITE_COUNT	(234)
 
-static void	pcm_test_char	(char *str, int filetype, int long_file_ok) ;
-static void	pcm_test_short	(char *str, int filetype, int long_file_ok) ;
-static void	pcm_test_24bit	(char *str, int filetype, int long_file_ok) ;
-static void	pcm_test_int	(char *str, int filetype, int long_file_ok) ;
-static void	pcm_test_float	(char *str, int filetype, int long_file_ok) ;
-static void	pcm_test_double	(char *str, int filetype, int long_file_ok) ;
+static void	pcm_test_char (const char *str, int filetype, int long_file_ok) ;
+static void	pcm_test_short (const char *str, int filetype, int long_file_ok) ;
+static void	pcm_test_24bit (const char *str, int filetype, int long_file_ok) ;
+static void	pcm_test_int (const char *str, int filetype, int long_file_ok) ;
+static void	pcm_test_float (const char *str, int filetype, int long_file_ok) ;
+static void	pcm_test_double (const char *str, int filetype, int long_file_ok) ;
 
 
 static	double	orig_data [DATA_LENGTH] ;
@@ -68,39 +75,42 @@ main (int argc, char **argv)
 		exit (1) ;
 		} ;
 
-	do_all = !strcmp (argv [1], "all");
+	do_all = !strcmp (argv [1], "all") ;
 
 	if (do_all || ! strcmp (argv [1], "wav"))
 	{	pcm_test_char 	("char.wav"  , SF_FORMAT_WAV | SF_FORMAT_PCM_U8, SF_FALSE) ;
 		pcm_test_short	("short.wav" , SF_FORMAT_WAV | SF_FORMAT_PCM_16, SF_FALSE) ;
 		pcm_test_24bit	("24bit.wav" , SF_FORMAT_WAV | SF_FORMAT_PCM_24, SF_FALSE) ;
 		pcm_test_int	("int.wav"   , SF_FORMAT_WAV | SF_FORMAT_PCM_32, SF_FALSE) ;
+		/* Lite remove start */
 		pcm_test_float	("float.wav" , SF_FORMAT_WAV | SF_FORMAT_FLOAT , SF_FALSE) ;
-		pcm_test_double	("double.wav", SF_FORMAT_WAV | SF_FORMAT_DOUBLE, SF_FALSE) ;
+		pcm_test_double	("double.wav"	, SF_FORMAT_WAV | SF_FORMAT_DOUBLE, SF_FALSE) ;
+		/* Lite remove end */
 		test_count++ ;
 		} ;
 
 	if (do_all || ! strcmp (argv [1], "aiff"))
-	{	pcm_test_char	("char_u8.aiff", SF_FORMAT_AIFF | SF_FORMAT_PCM_U8, SF_FALSE) ;
-		pcm_test_char	("char_s8.aiff", SF_FORMAT_AIFF | SF_FORMAT_PCM_S8, SF_FALSE) ;
+	{	pcm_test_char	("char_u8.aiff"	, SF_FORMAT_AIFF | SF_FORMAT_PCM_U8, SF_FALSE) ;
+		pcm_test_char	("char_s8.aiff"	, SF_FORMAT_AIFF | SF_FORMAT_PCM_S8, SF_FALSE) ;
 		pcm_test_short	("short.aiff"  , SF_FORMAT_AIFF | SF_FORMAT_PCM_16, SF_FALSE) ;
 		pcm_test_24bit	("24bit.aiff"  , SF_FORMAT_AIFF | SF_FORMAT_PCM_24, SF_FALSE) ;
 		pcm_test_int	("int.aiff"    , SF_FORMAT_AIFF | SF_FORMAT_PCM_32, SF_FALSE) ;
 
-		pcm_test_short	("short_sowt.aifc", SF_ENDIAN_LITTLE | SF_FORMAT_AIFF | SF_FORMAT_PCM_16, SF_FALSE) ;
-		pcm_test_24bit	("24bit_sowt.aifc", SF_ENDIAN_LITTLE | SF_FORMAT_AIFF | SF_FORMAT_PCM_24, SF_FALSE) ;
+		pcm_test_short	("short_sowt.aifc"	, SF_ENDIAN_LITTLE | SF_FORMAT_AIFF | SF_FORMAT_PCM_16, SF_FALSE) ;
+		pcm_test_24bit	("24bit_sowt.aifc"	, SF_ENDIAN_LITTLE | SF_FORMAT_AIFF | SF_FORMAT_PCM_24, SF_FALSE) ;
 		pcm_test_int	("int_sowt.aifc"  , SF_ENDIAN_LITTLE | SF_FORMAT_AIFF | SF_FORMAT_PCM_32, SF_FALSE) ;
 
-		pcm_test_short	("short_twos.aifc", SF_ENDIAN_BIG | SF_FORMAT_AIFF | SF_FORMAT_PCM_16, SF_FALSE) ;
-		pcm_test_24bit	("24bit_twos.aifc", SF_ENDIAN_BIG | SF_FORMAT_AIFF | SF_FORMAT_PCM_24, SF_FALSE) ;
+		pcm_test_short	("short_twos.aifc"	, SF_ENDIAN_BIG | SF_FORMAT_AIFF | SF_FORMAT_PCM_16, SF_FALSE) ;
+		pcm_test_24bit	("24bit_twos.aifc"	, SF_ENDIAN_BIG | SF_FORMAT_AIFF | SF_FORMAT_PCM_24, SF_FALSE) ;
 		pcm_test_int	("int_twos.aifc"  , SF_ENDIAN_BIG | SF_FORMAT_AIFF | SF_FORMAT_PCM_32, SF_FALSE) ;
 
+		/* Lite remove start */
 		pcm_test_short	("dwvw16.aifc", SF_FORMAT_AIFF | SF_FORMAT_DWVW_16, SF_TRUE) ;
 		pcm_test_24bit	("dwvw24.aifc", SF_FORMAT_AIFF | SF_FORMAT_DWVW_24, SF_TRUE) ;
 
 		pcm_test_float	("float.aifc" , SF_FORMAT_AIFF | SF_FORMAT_FLOAT , SF_FALSE) ;
-		pcm_test_double	("double.aifc", SF_FORMAT_AIFF | SF_FORMAT_DOUBLE, SF_FALSE) ;
-
+		pcm_test_double	("double.aifc"	, SF_FORMAT_AIFF | SF_FORMAT_DOUBLE, SF_FALSE) ;
+		/* Lite remove end */
 		test_count++ ;
 		} ;
 
@@ -109,39 +119,44 @@ main (int argc, char **argv)
 		pcm_test_short	("short.au" , SF_FORMAT_AU | SF_FORMAT_PCM_16, SF_FALSE) ;
 		pcm_test_24bit	("24bit.au" , SF_FORMAT_AU | SF_FORMAT_PCM_24, SF_FALSE) ;
 		pcm_test_int	("int.au"   , SF_FORMAT_AU | SF_FORMAT_PCM_32, SF_FALSE) ;
+		/* Lite remove start */
 		pcm_test_float	("float.au" , SF_FORMAT_AU | SF_FORMAT_FLOAT , SF_FALSE) ;
 		pcm_test_double	("double.au", SF_FORMAT_AU | SF_FORMAT_DOUBLE, SF_FALSE) ;
+		/* Lite remove end */
 
 		pcm_test_char	("char_le.au"  , SF_ENDIAN_LITTLE | SF_FORMAT_AU | SF_FORMAT_PCM_S8, SF_FALSE) ;
 		pcm_test_short	("short_le.au" , SF_ENDIAN_LITTLE | SF_FORMAT_AU | SF_FORMAT_PCM_16, SF_FALSE) ;
 		pcm_test_24bit	("24bit_le.au" , SF_ENDIAN_LITTLE | SF_FORMAT_AU | SF_FORMAT_PCM_24, SF_FALSE) ;
 		pcm_test_int	("int_le.au"   , SF_ENDIAN_LITTLE | SF_FORMAT_AU | SF_FORMAT_PCM_32, SF_FALSE) ;
+		/* Lite remove start */
 		pcm_test_float	("float_le.au" , SF_ENDIAN_LITTLE | SF_FORMAT_AU | SF_FORMAT_FLOAT , SF_FALSE) ;
-		pcm_test_double	("double_le.au", SF_ENDIAN_LITTLE | SF_FORMAT_AU | SF_FORMAT_DOUBLE, SF_FALSE) ;
-
+		pcm_test_double	("double_le.au"	, SF_ENDIAN_LITTLE | SF_FORMAT_AU | SF_FORMAT_DOUBLE, SF_FALSE) ;
+		/* Lite remove end */
 		test_count++ ;
 		} ;
 
 	if (do_all || ! strcmp (argv [1], "raw"))
-	{	pcm_test_char	("char_s8.raw", SF_FORMAT_RAW | SF_FORMAT_PCM_S8, SF_FALSE) ;
-		pcm_test_char	("char_u8.raw", SF_FORMAT_RAW | SF_FORMAT_PCM_U8, SF_FALSE) ;
+	{	pcm_test_char	("char_s8.raw"	, SF_FORMAT_RAW | SF_FORMAT_PCM_S8, SF_FALSE) ;
+		pcm_test_char	("char_u8.raw"	, SF_FORMAT_RAW | SF_FORMAT_PCM_U8, SF_FALSE) ;
 
-		pcm_test_short	("short_le.raw", SF_ENDIAN_LITTLE | SF_FORMAT_RAW | SF_FORMAT_PCM_16, SF_FALSE) ;
-		pcm_test_short	("short_be.raw", SF_ENDIAN_BIG    | SF_FORMAT_RAW | SF_FORMAT_PCM_16, SF_FALSE) ;
-		pcm_test_24bit	("24bit_le.raw", SF_ENDIAN_LITTLE | SF_FORMAT_RAW | SF_FORMAT_PCM_24, SF_FALSE) ;
-		pcm_test_24bit	("24bit_be.raw", SF_ENDIAN_BIG    | SF_FORMAT_RAW | SF_FORMAT_PCM_24, SF_FALSE) ;
+		pcm_test_short	("short_le.raw"	, SF_ENDIAN_LITTLE	| SF_FORMAT_RAW | SF_FORMAT_PCM_16, SF_FALSE) ;
+		pcm_test_short	("short_be.raw"	, SF_ENDIAN_BIG		| SF_FORMAT_RAW | SF_FORMAT_PCM_16, SF_FALSE) ;
+		pcm_test_24bit	("24bit_le.raw"	, SF_ENDIAN_LITTLE	| SF_FORMAT_RAW | SF_FORMAT_PCM_24, SF_FALSE) ;
+		pcm_test_24bit	("24bit_be.raw"	, SF_ENDIAN_BIG		| SF_FORMAT_RAW | SF_FORMAT_PCM_24, SF_FALSE) ;
 		pcm_test_int	("int_le.raw"  , SF_ENDIAN_LITTLE | SF_FORMAT_RAW | SF_FORMAT_PCM_32, SF_FALSE) ;
 		pcm_test_int	("int_be.raw"  , SF_ENDIAN_BIG    | SF_FORMAT_RAW | SF_FORMAT_PCM_32, SF_FALSE) ;
 
-		pcm_test_float	("float_le.raw", SF_ENDIAN_LITTLE | SF_FORMAT_RAW | SF_FORMAT_FLOAT , SF_FALSE) ;
-		pcm_test_float	("float_be.raw", SF_ENDIAN_BIG    | SF_FORMAT_RAW | SF_FORMAT_FLOAT , SF_FALSE) ;
+		/* Lite remove start */
+		pcm_test_float	("float_le.raw"	, SF_ENDIAN_LITTLE	| SF_FORMAT_RAW | SF_FORMAT_FLOAT , SF_FALSE) ;
+		pcm_test_float	("float_be.raw"	, SF_ENDIAN_BIG		| SF_FORMAT_RAW | SF_FORMAT_FLOAT , SF_FALSE) ;
 
 		pcm_test_double	("double_le.raw", SF_ENDIAN_LITTLE | SF_FORMAT_RAW | SF_FORMAT_DOUBLE, SF_FALSE) ;
 		pcm_test_double	("double_be.raw", SF_ENDIAN_BIG    | SF_FORMAT_RAW | SF_FORMAT_DOUBLE, SF_FALSE) ;
-
+		/* Lite remove end */
 		test_count++ ;
 		} ;
 
+	/* Lite remove start */
 	if (do_all || ! strcmp (argv [1], "paf"))
 	{	pcm_test_char	("char_le.paf", SF_ENDIAN_LITTLE | SF_FORMAT_PAF | SF_FORMAT_PCM_S8, SF_FALSE) ;
 		pcm_test_char	("char_be.paf", SF_ENDIAN_BIG    | SF_FORMAT_PAF | SF_FORMAT_PCM_S8, SF_FALSE) ;
@@ -149,14 +164,12 @@ main (int argc, char **argv)
 		pcm_test_short	("short_be.paf", SF_ENDIAN_BIG    | SF_FORMAT_PAF | SF_FORMAT_PCM_16, SF_FALSE) ;
 		pcm_test_24bit	("24bit_le.paf", SF_ENDIAN_LITTLE | SF_FORMAT_PAF | SF_FORMAT_PCM_24, SF_TRUE) ;
 		pcm_test_24bit	("24bit_be.paf", SF_ENDIAN_BIG    | SF_FORMAT_PAF | SF_FORMAT_PCM_24, SF_TRUE) ;
-
 		test_count++ ;
 		} ;
 
 	if (do_all || ! strcmp (argv [1], "svx"))
 	{	pcm_test_char	("char.svx" , SF_FORMAT_SVX | SF_FORMAT_PCM_S8, SF_FALSE) ;
 		pcm_test_short	("short.svx", SF_FORMAT_SVX | SF_FORMAT_PCM_16, SF_FALSE) ;
-
 		test_count++ ;
 		} ;
 
@@ -167,18 +180,16 @@ main (int argc, char **argv)
 		pcm_test_24bit	("24bit_be.nist", SF_ENDIAN_BIG    | SF_FORMAT_NIST | SF_FORMAT_PCM_24, SF_FALSE) ;
 		pcm_test_int	("int_le.nist"  , SF_ENDIAN_LITTLE | SF_FORMAT_NIST | SF_FORMAT_PCM_32, SF_FALSE) ;
 		pcm_test_int 	("int_be.nist"  , SF_ENDIAN_BIG    | SF_FORMAT_NIST | SF_FORMAT_PCM_32, SF_FALSE) ;
-
 		test_count++ ;
 		} ;
 
 	if (do_all || ! strcmp (argv [1], "ircam"))
-	{	pcm_test_short	("short_be.ircam", SF_ENDIAN_BIG    | SF_FORMAT_IRCAM | SF_FORMAT_PCM_16, SF_FALSE) ;
-		pcm_test_short	("short_le.ircam", SF_ENDIAN_LITTLE | SF_FORMAT_IRCAM | SF_FORMAT_PCM_16, SF_FALSE) ;
+	{	pcm_test_short	("short_be.ircam"	, SF_ENDIAN_BIG	| SF_FORMAT_IRCAM | SF_FORMAT_PCM_16, SF_FALSE) ;
+		pcm_test_short	("short_le.ircam"	, SF_ENDIAN_LITTLE	| SF_FORMAT_IRCAM | SF_FORMAT_PCM_16, SF_FALSE) ;
 		pcm_test_int	("int_be.ircam"  , SF_ENDIAN_BIG    | SF_FORMAT_IRCAM | SF_FORMAT_PCM_32, SF_FALSE) ;
 		pcm_test_int 	("int_le.ircam"  , SF_ENDIAN_LITTLE | SF_FORMAT_IRCAM | SF_FORMAT_PCM_32, SF_FALSE) ;
-		pcm_test_float	("float_be.ircam", SF_ENDIAN_BIG    | SF_FORMAT_IRCAM | SF_FORMAT_FLOAT , SF_FALSE) ;
-		pcm_test_float	("float_le.ircam", SF_ENDIAN_LITTLE | SF_FORMAT_IRCAM | SF_FORMAT_FLOAT , SF_FALSE) ;
-
+		pcm_test_float	("float_be.ircam"	, SF_ENDIAN_BIG	| SF_FORMAT_IRCAM | SF_FORMAT_FLOAT , SF_FALSE) ;
+		pcm_test_float	("float_le.ircam"	, SF_ENDIAN_LITTLE	| SF_FORMAT_IRCAM | SF_FORMAT_FLOAT , SF_FALSE) ;
 		test_count++ ;
 		} ;
 
@@ -189,17 +200,6 @@ main (int argc, char **argv)
 		test_count++ ;
 		} ;
 
-	if (do_all || ! strcmp (argv [1], "w64"))
-	{	pcm_test_char 	("char.w64"  , SF_FORMAT_W64 | SF_FORMAT_PCM_U8, SF_FALSE) ;
-		pcm_test_short	("short.w64" , SF_FORMAT_W64 | SF_FORMAT_PCM_16, SF_FALSE) ;
-		pcm_test_24bit	("24bit.w64" , SF_FORMAT_W64 | SF_FORMAT_PCM_24, SF_FALSE) ;
-		pcm_test_int	("int.w64"   , SF_FORMAT_W64 | SF_FORMAT_PCM_32, SF_FALSE) ;
-		pcm_test_float	("float.w64" , SF_FORMAT_W64 | SF_FORMAT_FLOAT , SF_FALSE) ;
-		pcm_test_double	("double.w64", SF_FORMAT_W64 | SF_FORMAT_DOUBLE, SF_FALSE) ;
-
-		test_count++ ;
-		} ;
-
 	if (do_all || ! strcmp (argv [1], "mat4"))
 	{	pcm_test_short	("short_be.mat4" , SF_ENDIAN_BIG    | SF_FORMAT_MAT4 | SF_FORMAT_PCM_16, SF_FALSE) ;
 		pcm_test_short	("short_le.mat4" , SF_ENDIAN_LITTLE | SF_FORMAT_MAT4 | SF_FORMAT_PCM_16, SF_FALSE) ;
@@ -207,9 +207,8 @@ main (int argc, char **argv)
 		pcm_test_int 	("int_le.mat4"   , SF_ENDIAN_LITTLE | SF_FORMAT_MAT4 | SF_FORMAT_PCM_32, SF_FALSE) ;
 		pcm_test_float	("float_be.mat4" , SF_ENDIAN_BIG    | SF_FORMAT_MAT4 | SF_FORMAT_FLOAT , SF_FALSE) ;
 		pcm_test_float	("float_le.mat4" , SF_ENDIAN_LITTLE | SF_FORMAT_MAT4 | SF_FORMAT_FLOAT , SF_FALSE) ;
-		pcm_test_double	("double_be.mat4", SF_ENDIAN_BIG    | SF_FORMAT_MAT4 | SF_FORMAT_DOUBLE, SF_FALSE) ;
-		pcm_test_double	("double_le.mat4", SF_ENDIAN_LITTLE | SF_FORMAT_MAT4 | SF_FORMAT_DOUBLE, SF_FALSE) ;
-
+		pcm_test_double	("double_be.mat4"	, SF_ENDIAN_BIG	| SF_FORMAT_MAT4 | SF_FORMAT_DOUBLE, SF_FALSE) ;
+		pcm_test_double	("double_le.mat4"	, SF_ENDIAN_LITTLE	| SF_FORMAT_MAT4 | SF_FORMAT_DOUBLE, SF_FALSE) ;
 		test_count++ ;
 		} ;
 
@@ -222,23 +221,40 @@ main (int argc, char **argv)
 		pcm_test_int 	("int_le.mat5"   , SF_ENDIAN_LITTLE | SF_FORMAT_MAT5 | SF_FORMAT_PCM_32, SF_FALSE) ;
 		pcm_test_float	("float_be.mat5" , SF_ENDIAN_BIG    | SF_FORMAT_MAT5 | SF_FORMAT_FLOAT , SF_FALSE) ;
 		pcm_test_float	("float_le.mat5" , SF_ENDIAN_LITTLE | SF_FORMAT_MAT5 | SF_FORMAT_FLOAT , SF_FALSE) ;
-		pcm_test_double	("double_be.mat5", SF_ENDIAN_BIG    | SF_FORMAT_MAT5 | SF_FORMAT_DOUBLE, SF_FALSE) ;
-		pcm_test_double	("double_le.mat5", SF_ENDIAN_LITTLE | SF_FORMAT_MAT5 | SF_FORMAT_DOUBLE, SF_FALSE) ;
-
+		pcm_test_double	("double_be.mat5"	, SF_ENDIAN_BIG	| SF_FORMAT_MAT5 | SF_FORMAT_DOUBLE, SF_FALSE) ;
+		pcm_test_double	("double_le.mat5"	, SF_ENDIAN_LITTLE	| SF_FORMAT_MAT5 | SF_FORMAT_DOUBLE, SF_FALSE) ;
 		test_count++ ;
 		} ;
 
 	if (do_all || ! strcmp (argv [1], "pvf"))
 	{	pcm_test_char 	("char.pvf"  , SF_FORMAT_PVF | SF_FORMAT_PCM_S8, SF_FALSE) ;
-		pcm_test_short	("short.pvf" , SF_FORMAT_PVF | SF_FORMAT_PCM_16, SF_FALSE) ;
+		pcm_test_short	("short.pvf", SF_FORMAT_PVF | SF_FORMAT_PCM_16, SF_FALSE) ;
 		pcm_test_int	("int.pvf"   , SF_FORMAT_PVF | SF_FORMAT_PCM_32, SF_FALSE) ;
-
 		test_count++ ;
 		} ;
 
 	if (do_all || ! strcmp (argv [1], "htk"))
-	{	pcm_test_short	("short.htk" , SF_FORMAT_HTK | SF_FORMAT_PCM_16, SF_FALSE) ;
+	{	pcm_test_short	("short.htk", SF_FORMAT_HTK | SF_FORMAT_PCM_16, SF_FALSE) ;
+		test_count++ ;
+		} ;
+	/* Lite remove end */
 
+	if (do_all || ! strcmp (argv [1], "w64"))
+	{	pcm_test_char	("char.w64"		, SF_FORMAT_W64 | SF_FORMAT_PCM_U8, SF_FALSE) ;
+		pcm_test_short	("short.w64"	, SF_FORMAT_W64 | SF_FORMAT_PCM_16, SF_FALSE) ;
+		pcm_test_24bit	("24bit.w64"	, SF_FORMAT_W64 | SF_FORMAT_PCM_24, SF_FALSE) ;
+		pcm_test_int	("int.w64"		, SF_FORMAT_W64 | SF_FORMAT_PCM_32, SF_FALSE) ;
+		/* Lite remove start */
+		pcm_test_float	("float.w64"	, SF_FORMAT_W64 | SF_FORMAT_FLOAT , SF_FALSE) ;
+		pcm_test_double	("double.w64"	, SF_FORMAT_W64 | SF_FORMAT_DOUBLE, SF_FALSE) ;
+		/* Lite remove end */
+		test_count++ ;
+		} ;
+
+	if (do_all || ! strcmp (argv [1], "sds"))
+	{	pcm_test_char	("char.sds"		, SF_FORMAT_SDS | SF_FORMAT_PCM_S8, SF_TRUE) ;
+		pcm_test_short	("short.sds"	, SF_FORMAT_SDS | SF_FORMAT_PCM_16, SF_TRUE) ;
+		pcm_test_24bit	("24bit.sds"	, SF_FORMAT_SDS | SF_FORMAT_PCM_24, SF_TRUE) ;
 		test_count++ ;
 		} ;
 
@@ -249,14 +265,14 @@ main (int argc, char **argv)
 		return 1 ;
 		} ;
 
-	return 0;
+	return 0 ;
 } /* main */
 
 /*============================================================================================
 **	Helper functions and macros.
 */
 
-static void	create_short_file (char *filename) ;
+static void	create_short_file (const char *filename) ;
 
 #define	CHAR_ERROR(x,y)		(abs ((x) - (y)) > 255)
 #define	INT_ERROR(x,y)		(((x) - (y)) != 0)
@@ -264,7 +280,7 @@ static void	create_short_file (char *filename) ;
 #define	FLOAT_ERROR(x,y)	(fabs ((x) - (y)) > 1e-5)
 
 #define CONVERT_DATA(k,len,new,orig)	\
-			{	for ((k) = 0 ; (k) < (len) ; (k)++)	\
+			{	for ((k) = 0 ; (k) < (len) ; (k) ++)	\
 					(new) [k] = (orig) [k] ;		\
 				}
 
@@ -273,7 +289,7 @@ static void	create_short_file (char *filename) ;
 */
 
 static void
-pcm_test_char (char *filename, int filetype, int long_file_ok)
+pcm_test_char (const char *filename, int filetype, int long_file_ok)
 {	SNDFILE		*file ;
 	SF_INFO		sfinfo ;
 	short		*orig, *test ;
@@ -382,7 +398,7 @@ pcm_test_char (char *filename, int filetype, int long_file_ok)
 			} ;
 
 	/* Seek to offset from end of file. */
-	test_seek_or_die (file, -(sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
+	test_seek_or_die (file, -1 * (sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
 
 	test_read_short_or_die (file, 0, test + 10, 4, __LINE__) ;
 	for (k = 10 ; k < 14 ; k++)
@@ -463,7 +479,7 @@ pcm_test_char (char *filename, int filetype, int long_file_ok)
 	{	orig [20] = pass * 2 ;
 
 		/* Write some data. */
-		test_write_short_or_die (file, 0, orig, DATA_LENGTH, __LINE__) ;
+		test_write_short_or_die (file, pass, orig, DATA_LENGTH, __LINE__) ;
 
 		test_read_write_position_or_die (file, __LINE__, pass, (pass - 1) * DATA_LENGTH, pass * DATA_LENGTH) ;
 
@@ -656,7 +672,7 @@ pcm_test_char (char *filename, int filetype, int long_file_ok)
 			} ;
 
 	/* Seek to offset from end of file. */
-	test_seek_or_die (file, -(sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
+	test_seek_or_die (file, -1 * (sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
 
 	test_readf_short_or_die (file, 0, test + 20, 2, __LINE__) ;
 	for (k = 20 ; k < 24 ; k++)
@@ -678,7 +694,7 @@ pcm_test_char (char *filename, int filetype, int long_file_ok)
 */
 
 static void
-pcm_test_short (char *filename, int filetype, int long_file_ok)
+pcm_test_short (const char *filename, int filetype, int long_file_ok)
 {	SNDFILE		*file ;
 	SF_INFO		sfinfo ;
 	short		*orig, *test ;
@@ -787,7 +803,7 @@ pcm_test_short (char *filename, int filetype, int long_file_ok)
 			} ;
 
 	/* Seek to offset from end of file. */
-	test_seek_or_die (file, -(sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
+	test_seek_or_die (file, -1 * (sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
 
 	test_read_short_or_die (file, 0, test + 10, 4, __LINE__) ;
 	for (k = 10 ; k < 14 ; k++)
@@ -868,7 +884,7 @@ pcm_test_short (char *filename, int filetype, int long_file_ok)
 	{	orig [20] = pass * 2 ;
 
 		/* Write some data. */
-		test_write_short_or_die (file, 0, orig, DATA_LENGTH, __LINE__) ;
+		test_write_short_or_die (file, pass, orig, DATA_LENGTH, __LINE__) ;
 
 		test_read_write_position_or_die (file, __LINE__, pass, (pass - 1) * DATA_LENGTH, pass * DATA_LENGTH) ;
 
@@ -1061,7 +1077,7 @@ pcm_test_short (char *filename, int filetype, int long_file_ok)
 			} ;
 
 	/* Seek to offset from end of file. */
-	test_seek_or_die (file, -(sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
+	test_seek_or_die (file, -1 * (sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
 
 	test_readf_short_or_die (file, 0, test + 20, 2, __LINE__) ;
 	for (k = 20 ; k < 24 ; k++)
@@ -1083,7 +1099,7 @@ pcm_test_short (char *filename, int filetype, int long_file_ok)
 */
 
 static void
-pcm_test_24bit (char *filename, int filetype, int long_file_ok)
+pcm_test_24bit (const char *filename, int filetype, int long_file_ok)
 {	SNDFILE		*file ;
 	SF_INFO		sfinfo ;
 	int		*orig, *test ;
@@ -1192,7 +1208,7 @@ pcm_test_24bit (char *filename, int filetype, int long_file_ok)
 			} ;
 
 	/* Seek to offset from end of file. */
-	test_seek_or_die (file, -(sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
+	test_seek_or_die (file, -1 * (sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
 
 	test_read_int_or_die (file, 0, test + 10, 4, __LINE__) ;
 	for (k = 10 ; k < 14 ; k++)
@@ -1273,7 +1289,7 @@ pcm_test_24bit (char *filename, int filetype, int long_file_ok)
 	{	orig [20] = pass * 2 ;
 
 		/* Write some data. */
-		test_write_int_or_die (file, 0, orig, DATA_LENGTH, __LINE__) ;
+		test_write_int_or_die (file, pass, orig, DATA_LENGTH, __LINE__) ;
 
 		test_read_write_position_or_die (file, __LINE__, pass, (pass - 1) * DATA_LENGTH, pass * DATA_LENGTH) ;
 
@@ -1466,7 +1482,7 @@ pcm_test_24bit (char *filename, int filetype, int long_file_ok)
 			} ;
 
 	/* Seek to offset from end of file. */
-	test_seek_or_die (file, -(sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
+	test_seek_or_die (file, -1 * (sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
 
 	test_readf_int_or_die (file, 0, test + 20, 2, __LINE__) ;
 	for (k = 20 ; k < 24 ; k++)
@@ -1488,7 +1504,7 @@ pcm_test_24bit (char *filename, int filetype, int long_file_ok)
 */
 
 static void
-pcm_test_int (char *filename, int filetype, int long_file_ok)
+pcm_test_int (const char *filename, int filetype, int long_file_ok)
 {	SNDFILE		*file ;
 	SF_INFO		sfinfo ;
 	int		*orig, *test ;
@@ -1597,7 +1613,7 @@ pcm_test_int (char *filename, int filetype, int long_file_ok)
 			} ;
 
 	/* Seek to offset from end of file. */
-	test_seek_or_die (file, -(sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
+	test_seek_or_die (file, -1 * (sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
 
 	test_read_int_or_die (file, 0, test + 10, 4, __LINE__) ;
 	for (k = 10 ; k < 14 ; k++)
@@ -1678,7 +1694,7 @@ pcm_test_int (char *filename, int filetype, int long_file_ok)
 	{	orig [20] = pass * 2 ;
 
 		/* Write some data. */
-		test_write_int_or_die (file, 0, orig, DATA_LENGTH, __LINE__) ;
+		test_write_int_or_die (file, pass, orig, DATA_LENGTH, __LINE__) ;
 
 		test_read_write_position_or_die (file, __LINE__, pass, (pass - 1) * DATA_LENGTH, pass * DATA_LENGTH) ;
 
@@ -1871,7 +1887,7 @@ pcm_test_int (char *filename, int filetype, int long_file_ok)
 			} ;
 
 	/* Seek to offset from end of file. */
-	test_seek_or_die (file, -(sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
+	test_seek_or_die (file, -1 * (sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
 
 	test_readf_int_or_die (file, 0, test + 20, 2, __LINE__) ;
 	for (k = 20 ; k < 24 ; k++)
@@ -1893,7 +1909,7 @@ pcm_test_int (char *filename, int filetype, int long_file_ok)
 */
 
 static void
-pcm_test_float (char *filename, int filetype, int long_file_ok)
+pcm_test_float (const char *filename, int filetype, int long_file_ok)
 {	SNDFILE		*file ;
 	SF_INFO		sfinfo ;
 	float		*orig, *test ;
@@ -2002,7 +2018,7 @@ pcm_test_float (char *filename, int filetype, int long_file_ok)
 			} ;
 
 	/* Seek to offset from end of file. */
-	test_seek_or_die (file, -(sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
+	test_seek_or_die (file, -1 * (sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
 
 	test_read_float_or_die (file, 0, test + 10, 4, __LINE__) ;
 	for (k = 10 ; k < 14 ; k++)
@@ -2083,7 +2099,7 @@ pcm_test_float (char *filename, int filetype, int long_file_ok)
 	{	orig [20] = pass * 2 ;
 
 		/* Write some data. */
-		test_write_float_or_die (file, 0, orig, DATA_LENGTH, __LINE__) ;
+		test_write_float_or_die (file, pass, orig, DATA_LENGTH, __LINE__) ;
 
 		test_read_write_position_or_die (file, __LINE__, pass, (pass - 1) * DATA_LENGTH, pass * DATA_LENGTH) ;
 
@@ -2276,7 +2292,7 @@ pcm_test_float (char *filename, int filetype, int long_file_ok)
 			} ;
 
 	/* Seek to offset from end of file. */
-	test_seek_or_die (file, -(sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
+	test_seek_or_die (file, -1 * (sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
 
 	test_readf_float_or_die (file, 0, test + 20, 2, __LINE__) ;
 	for (k = 20 ; k < 24 ; k++)
@@ -2298,7 +2314,7 @@ pcm_test_float (char *filename, int filetype, int long_file_ok)
 */
 
 static void
-pcm_test_double (char *filename, int filetype, int long_file_ok)
+pcm_test_double (const char *filename, int filetype, int long_file_ok)
 {	SNDFILE		*file ;
 	SF_INFO		sfinfo ;
 	double		*orig, *test ;
@@ -2407,7 +2423,7 @@ pcm_test_double (char *filename, int filetype, int long_file_ok)
 			} ;
 
 	/* Seek to offset from end of file. */
-	test_seek_or_die (file, -(sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
+	test_seek_or_die (file, -1 * (sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
 
 	test_read_double_or_die (file, 0, test + 10, 4, __LINE__) ;
 	for (k = 10 ; k < 14 ; k++)
@@ -2488,7 +2504,7 @@ pcm_test_double (char *filename, int filetype, int long_file_ok)
 	{	orig [20] = pass * 2 ;
 
 		/* Write some data. */
-		test_write_double_or_die (file, 0, orig, DATA_LENGTH, __LINE__) ;
+		test_write_double_or_die (file, pass, orig, DATA_LENGTH, __LINE__) ;
 
 		test_read_write_position_or_die (file, __LINE__, pass, (pass - 1) * DATA_LENGTH, pass * DATA_LENGTH) ;
 
@@ -2681,7 +2697,7 @@ pcm_test_double (char *filename, int filetype, int long_file_ok)
 			} ;
 
 	/* Seek to offset from end of file. */
-	test_seek_or_die (file, -(sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
+	test_seek_or_die (file, -1 * (sfinfo.frames - 10), SEEK_END, 10, sfinfo.channels, __LINE__) ;
 
 	test_readf_double_or_die (file, 0, test + 20, 2, __LINE__) ;
 	for (k = 20 ; k < 24 ; k++)
@@ -2704,7 +2720,7 @@ pcm_test_double (char *filename, int filetype, int long_file_ok)
 */
 
 static void
-create_short_file (char *filename)
+create_short_file (const char *filename)
 {	FILE *file ;
 
 	if (! (file = fopen (filename, "w")))
@@ -2739,3 +2755,5 @@ truncate (const char *filename, int ignored)
 } /* truncate */
 
 #endif
+
+
